/*------------------------------------------------------------------------
 * MAGNET bidProcessor component
 * $Id: BidCollectorFactory.java,v 1.5.2.1 2002/05/05 19:41:44 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Fri May  4 15:04:23 2001  John Collins
 *   handle exceptions locally
 * 
 * Fri Apr 20 23:34:14 2001  John Collins
 *   get rid of System.getProperty
 * 
 * Mon Apr  9 15:10:52 2001  Len Josephs
 *   Initial Entry
 */
package edu.umn.magnet.client.bid;

import edu.umn.magnet.util.MagnetProperties;

import org.apache.log4j.Category;

/**
 * This is a factory for returning BidCollector instances depending on the
 * BidCollectorType property.
 * @author Len Josephs
 * @version 1.0 
 */
public class BidCollectorFactory {

  // init the log category
  private static Category log =
    Category.getInstance(BidCollectorFactory.class.getName());

  // Default bid collector class
  private static final String DEFAULT_CLASS =
    "edu.umn.magnet.client.xac.XBidCollector";

  /**
   * This is a static factory method for returning BidCollector
   * instances depending on the BidCollectorType property.  If the property
   * is not set, the default XBidCollector object is returned.
   * @author Len Josephs
   * @version 1.0
   */
  public static final BidCollector getInstance()
  {
    // If the BidCollector class isn't set, the default is XBidCollector...
    String bcClassName
	= MagnetProperties.getProperty("BidCollector.Class", DEFAULT_CLASS);
    log.debug("Trying BidCollector of class \""+bcClassName+"\".");

    Class bidCollectorClass = null;

    // Now, try to get the class...
    try {
      bidCollectorClass = Class.forName(bcClassName);
    } catch (ClassNotFoundException cnfe) {
        log.error(cnfe.toString());
        log.info("Using default bid collector class \""+DEFAULT_CLASS+"\".");
        // The default class should really work, since we set it at
	// compile time. 
        try {
            bidCollectorClass
		= Class.forName(BidCollectorFactory.DEFAULT_CLASS);
        } catch (ClassNotFoundException cnf) {
            log.error(cnf.toString());
            log.error("Verify your CLASSPATH settings!");
            return null;  // Nasty, but at this point, fail...
        }
    }

    // ...and return an instance of the class.
    BidCollector result = null;
    try {
      result = (BidCollector)bidCollectorClass.newInstance();
    }
    catch (InstantiationException inex) {
      log.error(inex.toString());
    }
    catch (IllegalAccessException illex) {
      log.error(illex.toString());
    }
    return result;
  }
}
