/*------------------------------------------------------------------------
 * Magnet customer agent bid manager components
 * $Id: PropOfferedTDA.java,v 1.3.2.1 2002/05/05 19:41:45 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Thu Jan 11 13:17:25 2001  John Collins
 *   use new logging system
 * 
 * Wed Sep 27 15:09:10 2000  John Collins
 *   Move to bid package
 * 
 * Sat Sep  5 21:44:18 1998  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.bid;

import edu.umn.magnet.client.eval.TaskBid;
import edu.umn.magnet.client.status.PlanStatus;
import edu.umn.magnet.client.status.TaskStatus;

import org.apache.log4j.Category;

import java.util.Properties;

/**
 * Adjusts the duration of a task for computation of the RFQ schedule.
 * This implementation adjusts duration of offered tasks by a fixed
 * amount.  
 * @author John Collins
 */
public class PropOfferedTDA extends TaskDurationAdjuster
{
  static Category log = Category.getInstance(PropOfferedTDA.class.getName());

  protected double proportionalAdjustment = 1.0;

  /**
   * The default constructor probably won't be used.
   */
  public PropOfferedTDA ()
  {
    super();
  }

  /**
   * The principal constructor takes a property list and tries to
   * extract the TDA.DurationMult value from it.
   */
  public PropOfferedTDA (Properties props)
  {
    super();
    init(props);
  }

  /**
   * For completeness, here's a constructor that takes a proportion
   * value directly.
   */
  public PropOfferedTDA (double proportion)
  {
    super();
    proportionalAdjustment = proportion;
  }

  /**
   * Initialize an empty one from input properties.
   */
  protected void init (Properties props)
  {
    String prop = props.getProperty("TDA.DurationMult", null);
    log.info("prop: TDA.DurationMult = " + prop);
    if (prop == null) {
      log.warn("No TDA.DurationMult value found.  Using 1/CPMSlack");
      prop = props.getProperty("CPMSlack", null);
      if (prop == null) {
	log.warn("No CPMSlack value found.  Using default.");
	proportionalAdjustment = 0.8;
      }
      else {
	proportionalAdjustment = 1.0 / (Double.valueOf(prop)).doubleValue();
      }
    }
    else {
      proportionalAdjustment = (Double.valueOf(prop)).doubleValue();
    }    
  }

  /**
   * This is the operator method for this visitor.  Just in case the
   * state of this task is OFFERED, we multiply
   * the expected task duration by the adjustment value.
   */
  public void processTaskBid (TaskBid task)
  {
    if (getPlanStatus().getTaskState(task.getTask())
	.equals(TaskStatus.OFFERED)) {
      task.setAdjustedDuration(Math.round(task.getExpectedDuration()
					  * proportionalAdjustment));
    }
    else {
      task.setAdjustedDuration(Math.round(task.getExpectedDuration()));
    }
  }
}
