/*------------------------------------------------------------------------
 * Magnet bid evaluation components
 * $Id: AbstractBidSelector.java,v 1.1.2.1 2002/05/05 19:41:46 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Jan  5 13:53:01 2000  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.eval;


/**
 * Abstract implementation of BidSelector that provides default
 * implementations of hasMoreChoices and recycle.
 * @author John Collins
 */
public abstract class AbstractBidSelector implements BidSelector
{

  /**
   * Sets the current annealing temperature, if that make sense.
   * Selectors need not do anything with this method if they don't
   * care. 
   */
  public void setTemperature (double temp) {
  }

  /**
   * Returns false; the default bid selector doesn't have choices to
   * cycle through
   */
  public boolean hasMoreChoices () {
    return false;
  }

  /**
   * Does nothing; the default bid selector has no state to worry
   * about when the engine recycles
   */
  public void recycle () {
  }
}
