/*------------------------------------------------------------------------
 * MAGNET client bid evaluation components
 * $Id: AbstractSearchEngine.java,v 1.4.2.3 2002/05/05 19:41:46 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Fri Jan 12 17:04:36 2001  John Collins
 *   Move the coverage test to BidEvaluator.
 * 
 * Mon Nov 27 10:40:02 2000  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.eval;

import java.util.Iterator;
import java.util.HashMap;

/**
 * This class exists to hold common functionality for all search
 * engines.
 * @author John Collins
 */
public abstract class AbstractSearchEngine implements SearchEngine
{
  protected BidEvaluator bidEval = null;
  protected long baseTime = 0L;
  protected TaskBidPlan firstPlan = null;
  protected int planCount = 0;

  /**
   * The constructor just captures the BidEvaluator reference
   */
  public AbstractSearchEngine ()
  {
    super();
  }

  /**
   * The default search type is "Unknown".
   */
  public String getSearchType ()
  {
    return "Unknown";
  }

  public void setBidEvaluator (BidEvaluator eval)
  {
    bidEval = eval;
  }

  public BidEvaluator getBidEvaluator ()
  {
    return bidEval;
  }

  /**
   * Sets the initial search node for continued searches
   */
  public void setInitialNode (TaskBidPlan node)
  {
    // default implementation does nothing.
  }

  /**
   * Runs a search, measuring run time from the given base time
   */
  public TaskBidPlan search (long baseTime)
  {
    this.baseTime = baseTime;
    return this.search();
  }

  /**
   * Returns the first plan found, if any
   */
  public TaskBidPlan getFirstPlan ()
  {
    return firstPlan;
  }

  /**
   * Returns the node count for whatever search has been completed so far.
   */
  public int getNodeCount ()
  {
    return 0;
  }

  /**
   * Returns the number of covered, feasible plans found
   */
  public int getPlanCount ()
  {
    return planCount;
  }

  /**
   * Returns the current run time, as wall-clock offset from the base time. 
   */
  public long getRunTime ()
  {
    return System.currentTimeMillis() - baseTime;
  }

  /**
   * Keeps track of solutions found. We assume without checking that
   * this only gets called on covered, feasible nodes.
   */
  protected void processSolution (TaskBidPlan node)
  {
    node.setSearchData(getSearchType(), getRunTime(), getNodeCount());
    planCount++;
    if (firstPlan == null)
      firstPlan = node;
  }

  //----------- Interactive pause/resume --------------
  public synchronized void pause ()
  {
    try {
      wait();
    }
    catch (InterruptedException ie) {
    }
  }

  public synchronized void resume ()
  {
    notifyAll();
  }

  // ------------ Search Engine Registration & Factory ------------
  private static HashMap registrations = new HashMap();

  protected static void register (String name, Class cls)
  {
    return;
  }

  public static SearchEngine createSearchEngine (String type)
  {
    return null;
  }
}
