/*------------------------------------------------------------------------
 * Magnet Bid Evaluation components
 * $Id: TabuList.java,v 1.1.2.1 2002/05/05 19:41:49 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Tue Sep 29 18:04:58 1998  John E. Collins
 *   Add contains method.
 * 
 * Tue Sep 29 10:28:09 1998  John E. Collins
 *   Rearrange to overcome jgl cloning problem.
 * 
 * Mon Sep 28 14:19:10 1998  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.eval;

import java.util.LinkedList;

/**
 * This class implements a first-in, first-out Tabu List in support of
 * stochastic search.  A Tabu List is a queue with a limited length.
 * Once the length limit is reached, as you add items to the front,
 * they fall off the other end.
 * @author John Collins
 */
public class TabuList extends LinkedList
{
  int m_maxLength = 10;

  /**
   * The default constructor creates a tabu list of length 10.
   */
  public TabuList () {
    super();
  }

  /**
   * You get to specify the length when you create one.  The length
   * must be positive, and in general should be less than 20, although
   * there is no hard limit.
   */
  public TabuList (int maxLength) {
    super();
    m_maxLength = maxLength;
  }

  /**
   * Adds an element to the list, possibly dropping an element off the
   * other end.
   */
  public boolean add (Object newItem) {
    super.add(newItem);
    if (size() > m_maxLength) 
      removeFirst();
    return true;
  }

  /**
   * Returns the maximum length of this list.
   */
  public int getMaxLength () {
    return m_maxLength;
  }

  /**
   * Changes the maximum length of this list.
   */
  public void setMaxLength (int newLength) {
    m_maxLength = newLength;
  }
}
