/*----------------------------------------------------------------------------
 * Magnet Bid Evaluation components
 * $Id: TaskDescriptor.java,v 1.3.2.3 2002/05/05 19:41:49 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License. 
 *----------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Feb 20 09:12:55 2002  John Collins  #0000196
 *   Add sorted bids accessor.
 * 
 * Wed Dec 13 22:10:47 2000  John Collins
 *   Remove average price calculation - it is insupportable when bid
 *   breakdowns go away.
 * 
 * Sun Jan 10 14:59:53 1999  John E. Collins
 *   Correct calculation of average price.
 */

package edu.umn.magnet.client.eval;

import edu.umn.magnet.protocol.cust.Bid;
import edu.umn.magnet.protocol.cust.SubtaskRequest;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import org.apache.log4j.Category;
import java.util.Comparator;

/**
 * This is awfully similar to {@link
 * edu.umn.magnet.client.status.TaskStatus TaskStatus}.  The real
 * difference is that these exist only within the context of a single
 * {@link BidEvaluator}, and the bids included are only those that are
 * currently thought to be "usable" by that evaluator.  In other
 * words, any bids that get thrown out of evaluation due to constraint
 * violations will be removed from their respective tasks.
 * @author Rashmi Sundareswara, John Collins
 * @version $Revision $
 */
public class TaskDescriptor 
{
  static Category log =
    Category.getInstance(TaskDescriptor.class.getName());

  private SubtaskRequest m_task = null;
  private HashSet m_bids = new HashSet();
  //private Bid[] m_sortedBids = null;
  
  /** Default Constructor  */
  public TaskDescriptor()
  {
    super();
  }
    
  /**
   * The normal constructor takes a subtask
   */
  public TaskDescriptor(SubtaskRequest aSubtaskRq)
  {
    super();
    m_task = aSubtaskRq; 
  }
  
  public String getTaskId()
  {
    return m_task.getId();
  }
  
  /**
   * Adds a Bid, which presumably includes a bid on this subtask, to the
   * set of bids for this subtask.  The subtask bid specifying this subtask
   * is extracted and recorded as well.  The subtaskBids are to be added in 
   * the order of the least expensive to the most, while the bids are arranged
   * in the order of the ones with the most discount to the least 
   */ 
  public void addBid(Bid aBid)
  {
    m_bids.add(aBid);
    //m_sortedBids = null;
  }

  /**
   * Removes a bid from this TaskDescriptor.  Nothing happens if the
   * bid is not in the bid list.
   */
  public void removeBid (Bid aBid)
  {
    m_bids.remove(aBid);
  }
    
  /**
   * Method getSubtaskRequest return s the SubtaskRequest that is
   * assigned to the Task Descriptor
   */
  public SubtaskRequest getSubtaskRequest() 
  {
    return m_task;
  }
  
  /** method getBidCount() returns the number of Bids containing a 
   *  particular subtask
   */
  public int getBidCount()
  {
    return m_bids.size();
  }
  
  /**
   * Returns an Iterator of Bids that contain this task
   */
  public Iterator getBids()
  {
    Iterator e = m_bids.iterator();
    if (!e.hasNext())
      return null;
    else return e;
  }

  /**
   * Returns the set of bids as a Set
   */
  public Set getBidSet ()
  {
    return m_bids;
  }
}
