/*------------------------------------------------------------------------
 * MAGNET status component
 * $Id: AbstractStatus.java,v 1.1.2.1 2002/05/05 19:41:49 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed May  2 08:38:11 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.status;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * This is the abstract superclass for all the agent status classes.
 * It supports distribution of StatusEvents by wrapping a
 * PropertyChangeSupport instance.
 * @author John Collins
 * @version $Revision: 1.1.2.1 $
 */
public abstract class AbstractStatus
{
  private PropertyChangeSupport eventSupport
    = new PropertyChangeSupport(this);

  /**
   * Adds the specified <tt>PropertyChangeListener</tt> to the event
   * listener list of this object.
   */
  public void addListener (PropertyChangeListener pcl)
  {
    eventSupport.addPropertyChangeListener(pcl);
  }

  /**
   * Adds the specified <tt>PropertyChangeListener</tt> to the event
   * listener list of this object for changes to the property with the
   * given name.
   */
  public void addListener (String propertyName, PropertyChangeListener pcl)
  {
    eventSupport.addPropertyChangeListener(propertyName, pcl);
  }

  /**
   * Removes the specified <tt>PropertyChangeListener</tt> from the event
   * listener list of this object.
   */
  public void removeListener (PropertyChangeListener pcl)
  {
    eventSupport.removePropertyChangeListener(pcl);
  }

  /**
   * Removes the specified <tt>PropertyChangeListener</tt> from the event
   * listener list of this object for changes to the property with the
   * given name.
   */
  public void removeListener (String propertyName, PropertyChangeListener pcl)
  {
    eventSupport.removePropertyChangeListener(propertyName, pcl);
  }

  /**
   * Distributes the specified StatusEvent to all registered
   * listeners.
   */
  public void sendStatusEvent (StatusEvent event)
  {
    eventSupport.firePropertyChange(event);
  }
}
