/*------------------------------------------------------------------------
 * MAGNET TaskPlan test data generator
 * $Id: SampleTaskPlan.java,v 1.2.2.1 2002/05/05 19:41:52 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Feb 25 09:53:03 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client;

import edu.umn.magnet.protocol.cust.TaskPlan;
import edu.umn.magnet.protocol.cust.SubtaskRequest;

import edu.umn.magnet.util.time.AgentTime;

import junit.framework.Assert;

import java.util.Date;

/**
 * This thing just generates sample TaskPlan data for other tests.
 * Invoke the static "generateTaskPlan()" method.
 * @author John Collins
 * @version $Revision: 1.2.2.1 $
 */
public class SampleTaskPlan extends Assert
{
  // ------------ Static TaskPlan generator for other tests ------------

  private static final long HOUR = 3600000l;

  public static TaskPlan generateTaskPlan ()
  {
    TaskPlan plan = new TaskPlan();
    // populate the plan with 6 tasks
    try {
      plan.addTask(new SubtaskRequest("T1"));
      plan.addTask(new SubtaskRequest("T2"));
      plan.addSuccessor("T1", "T2");
      plan.addTask(new SubtaskRequest("T3"));
      plan.addSuccessor("T1", "T3");
      plan.addTask(new SubtaskRequest("T4"));
      plan.addSuccessor("T2", "T4");
      plan.addTask(new SubtaskRequest("T5"));
      plan.addSuccessor("T3", "T5");
      plan.addTask(new SubtaskRequest("T6"));
      plan.addSuccessor("T4", "T6");
      plan.addSuccessor("T3", "T6");
    }
    catch (Exception e) {
      fail("Exception creating plan: " + e);
    }
    // We now have
    //   2-4-6
    //  /   /
    // 1   /
    //  \ /
    //   3-5
    //System.out.println("Subtask Tree:");
    //printSuccessorTree(plan, plan.getSuccessorRoots(), 1);
    // put timings on the tasks
    long baseTime = (AgentTime.getDate().getTime()) + HOUR * 12;
    try {
      plan.getTaskData("T1").setEarliestTimeToStart(new Date(baseTime));
      plan.getTaskData("T1").setDeadline(new Date(baseTime + HOUR * 175));
      plan.getTaskData("T2").setEarliestTimeToStart
	                       (new Date(baseTime + HOUR * 75));
      plan.getTaskData("T2").setDeadline(new Date(baseTime + HOUR * 250));
      plan.getTaskData("T3").setEarliestTimeToStart
	                       (new Date(baseTime + HOUR * 75));
      plan.getTaskData("T3").setDeadline(new Date(baseTime + HOUR * 325));
      plan.getTaskData("T4").setEarliestTimeToStart
	                       (new Date(baseTime + HOUR * 150));
      plan.getTaskData("T4").setDeadline(new Date(baseTime + HOUR * 325));
      plan.getTaskData("T5").setEarliestTimeToStart
	                       (new Date(baseTime + HOUR * 150));
      plan.getTaskData("T5").setDeadline(new Date(baseTime + HOUR * 400));
      plan.getTaskData("T6").setEarliestTimeToStart
	                       (new Date(baseTime + HOUR * 225));
      plan.getTaskData("T6").setDeadline(new Date(baseTime + HOUR * 400));
    }
    catch (Exception e) {
      fail("Exception setting task timings: " + e);
    }
    return plan;
  }
}
