/*-------------------------------------------------------------------------
 * Magnet unit tests
 * $Id: EvalTest.java,v 1.2.2.1 2002/05/05 19:41:52 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *-------------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Dec 10 13:33:50 2000  John Collins
 *   Convert to JUnit format
 * 
 * Thu Oct  1 11:09:44 1998  John E. Collins
 *   Add tests for subtask bid expansion
 * 
 * Mon Sep 28 22:08:43 1998  John E. Collins
 *   Initial entry
 */

package edu.umn.magnet.client.eval;

import edu.umn.magnet.protocol.cust.TaskPlan;
import edu.umn.magnet.protocol.cust.Bid;
import edu.umn.magnet.client.eval.Eval;

import edu.umn.magnet.util.time.AgentTime;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.*;
import java.io.*;

/**
 * This is a tester for edu.umn.magnet.client.eval.Eval and related
 * stuff.
 * @author John Collins
 */
public class EvalTest extends TestCase
{

  private static final long s_msph = 3600000l;

  private TaskPlan plan = null;
  private TaskBidPlan tbp = null;
  private TaskBidPlan tbp1 = null;
  private TaskBidPlan tbp2 = null;
  private TaskBidPlan tbp3 = null;
  private TaskBidPlan tbp4 = null;
  private TaskBidPlan tbp5 = null;
  private Eval eval = null;
  private Bid b1 = new Bid();
  private Bid b2 = new Bid();
  private Bid b3 = new Bid();
  private Bid b4 = new Bid();
  private Bid b5 = new Bid();
  
  public EvalTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(EvalTest.class);
  }

  private void setup1 ()
  {
    plan = new TaskPlan();
    // populate the plan with 6 tasks
    try {
      plan.addTask(new SubtaskRequest("T1"));
      plan.addTask(new SubtaskRequest("T2"));
      plan.addSuccessor("T1", "T2");
      plan.addTask(new SubtaskRequest("T3"));
      plan.addSuccessor("T1", "T3");
      plan.addTask(new SubtaskRequest("T4"));
      plan.addSuccessor("T2", "T4");
      plan.addTask(new SubtaskRequest("T5"));
      plan.addSuccessor("T3", "T5");
      plan.addTask(new SubtaskRequest("T6"));
      plan.addSuccessor("T4", "T6");
      plan.addSuccessor("T3", "T6");
    }
    catch (Exception e) {
      fail("Exception creating plan: " + e);
    }
    // We now have
    //   2-4-6
    //  /   /
    // 1   /
    //  \ /
    //   3-5
    //System.out.println("Subtask Tree:");
    //printSuccessorTree(plan, plan.getSuccessorRoots(), 1);
    // put timings on the tasks
    long baseTime = (AgentTime.getDate().getTime()) + s_msph * 12;
    try {
      plan.getTaskData("T1").setEarliestTimeToStart(new Date(baseTime));
      plan.getTaskData("T1").setDeadline(new Date(baseTime + s_msph * 175));
      plan.getTaskData("T2").setEarliestTimeToStart
	                       (new Date(baseTime + s_msph * 75));
      plan.getTaskData("T2").setDeadline(new Date(baseTime + s_msph * 250));
      plan.getTaskData("T3").setEarliestTimeToStart
	                       (new Date(baseTime + s_msph * 75));
      plan.getTaskData("T3").setDeadline(new Date(baseTime + s_msph * 325));
      plan.getTaskData("T4").setEarliestTimeToStart
	                       (new Date(baseTime + s_msph * 150));
      plan.getTaskData("T4").setDeadline(new Date(baseTime + s_msph * 325));
      plan.getTaskData("T5").setEarliestTimeToStart
	                       (new Date(baseTime + s_msph * 150));
      plan.getTaskData("T5").setDeadline(new Date(baseTime + s_msph * 400));
      plan.getTaskData("T6").setEarliestTimeToStart
	                       (new Date(baseTime + s_msph * 225));
      plan.getTaskData("T6").setDeadline(new Date(baseTime + s_msph * 400));
    }
    catch (Exception e) {
      fail("Exception setting task timings: " + e);
    }

    // Create some Bids against the TP
    System.out.println();
    System.out.println("Step 2: Create bids");
    b1 = new Bid();
    b2 = new Bid();
    b3 = new Bid();
    b4 = new Bid();
    b5 = new Bid();
    try {
      SubtaskBid[] sb1
	= {new SubtaskBid(), new SubtaskBid(), new SubtaskBid()};
      sb1[0].setId("B1T2");
      sb1[0].setSubtaskRequestId("T2");
      sb1[0].setCost(110);
      sb1[0].setStartBy(new Date(baseTime + s_msph * 80));
      sb1[0].setTimeToDo(s_msph * 80);
      sb1[0].setEndBy(new Date(baseTime + s_msph * 200));
      sb1[1].setId("B1T4");
      sb1[1].setSubtaskRequestId("T4");
      sb1[1].setCost(120);
      sb1[1].setStartBy(new Date(baseTime + s_msph * 160));
      sb1[1].setTimeToDo(s_msph * 110);
      sb1[1].setEndBy(new Date(baseTime + s_msph * 280));
      sb1[2].setId("B1T6");
      sb1[2].setSubtaskRequestId("T6");
      sb1[2].setCost(90);
      sb1[2].setStartBy(new Date(baseTime + s_msph * 270));
      sb1[2].setTimeToDo(s_msph * 120);
      sb1[2].setEndBy(new Date(baseTime + s_msph * 400));
      b1.setId("B1");
      b1.setCost(300);
      b1.setSubtaskBids(sb1);

      SubtaskBid[] sb2
	= {new SubtaskBid(), new SubtaskBid()};
      sb2[0].setId("B2T1");
      sb2[0].setSubtaskRequestId("T1");
      sb2[0].setCost(110);
      sb2[0].setStartBy(new Date(baseTime + s_msph * 20));
      sb2[0].setTimeToDo(s_msph * 60);
      sb2[0].setEndBy(new Date(baseTime + s_msph * 100));
      sb2[1].setId("B2T3");
      sb2[1].setSubtaskRequestId("T3");
      sb2[1].setCost(150);
      sb2[1].setStartBy(new Date(baseTime + s_msph * 90));
      sb2[1].setTimeToDo(s_msph * 80);
      sb2[1].setEndBy(new Date(baseTime + s_msph * 200));
      b2.setId("B2");
      b2.setCost(240);
      b2.setSubtaskBids(sb2);

      SubtaskBid[] sb3
	= {new SubtaskBid(), new SubtaskBid(), new SubtaskBid()};
      sb3[0].setId("B3T1");
      sb3[0].setSubtaskRequestId("T1");
      sb3[0].setCost(60);
      sb3[0].setStartBy(new Date(baseTime + s_msph * 20));
      sb3[0].setTimeToDo(s_msph * 70);
      sb3[0].setEndBy(new Date(baseTime + s_msph * 150));
      sb3[1].setId("B3T3");
      sb3[1].setSubtaskRequestId("T3");
      sb3[1].setCost(120);
      sb3[1].setStartBy(new Date(baseTime + s_msph * 130));
      sb3[1].setTimeToDo(s_msph * 100);
      sb3[1].setEndBy(new Date(baseTime + s_msph * 250));
      sb3[2].setId("B3T5");
      sb3[2].setSubtaskRequestId("T5");
      sb3[2].setCost(180);
      sb3[2].setStartBy(new Date(baseTime + s_msph * 230));
      sb3[2].setTimeToDo(s_msph * 50);
      sb3[2].setEndBy(new Date(baseTime + s_msph * 300));
      b3.setId("B3");
      b3.setCost(350);
      b3.setSubtaskBids(sb3);

      SubtaskBid[] sb4
	= {new SubtaskBid(), new SubtaskBid(), new SubtaskBid()};
      sb4[0].setId("B4T1");
      sb4[0].setSubtaskRequestId("T1");
      sb4[0].setCost(70);
      sb4[0].setStartBy(new Date(baseTime + s_msph * 20));
      sb4[0].setTimeToDo(s_msph * 80);
      sb4[0].setEndBy(new Date(baseTime + s_msph * 120));
      sb4[1].setId("B4T2");
      sb4[1].setSubtaskRequestId("T2");
      sb4[1].setCost(140);
      sb4[1].setStartBy(new Date(baseTime + s_msph * 120));
      sb4[1].setTimeToDo(s_msph * 110);
      sb4[1].setEndBy(new Date(baseTime + s_msph * 240));
      sb4[2].setId("B4T3");
      sb4[2].setSubtaskRequestId("T3");
      sb4[2].setCost(140);
      sb4[2].setStartBy(new Date(baseTime + s_msph * 240));
      sb4[2].setTimeToDo(s_msph * 80);
      sb4[2].setEndBy(new Date(baseTime + s_msph * 325));
      b4.setId("B4");
      b4.setCost(300);
      b4.setSubtaskBids(sb4);
    }
    catch (Exception e) {
      fail("Exception creating bids: " + e);
    }
    tbp = new TaskBidPlan(plan);
    eval = new Eval();
  }

  // --------------------------------------------------------------------
  // Create a TaskBidPlan and check it out
  public void testTBPCreate ()
  {
    setup1();
    assertSame("Compare plan", plan, tbp.getTaskPlan());

    // Check for correct number of TaskBid objects
    Enumeration tbs = tbp.getTaskBids();
    int tbCount = 0;
    while (tbs.hasMoreElements()) {
      tbCount++;
      tbs.nextElement();
    }
    assertEquals("Correct count of TaskBid objects", 6, tbCount);

    // Make sure we can find the root of the underlying TaskPlan
    Enumeration rts = tbp.findRoots();
    Vector rtv = new Vector();
    while (rts.hasMoreElements()) {
      rtv.addElement(rts.nextElement());
    }
    assertEquals("Only one root", 1, rtv.size());
    assertEquals("Correct root task",
		 "T1",
		 ((SubtaskRequest)rtv.elementAt(0)).getId());
    // There's lots more to test here...rtv.elementAt(0);
  }

  // --------------------------------------------------------------------
  // Create an Eval and try it out on the root node
  public void testEvalCreate ()
  {
    setup1();
    eval.evaluate(tbp);

    // the root node should be feasible but not covered
    assert("Root node should be feasible", tbp.isFeasible());
    assert("Root node should not be covered", !tbp.isCovered());
    assertEquals("Root node cost should be 0", 0, tbp.getCost());
    assertEquals("Root node eval should be 12000",
		 12000.0,
		 tbp.getEvaluation(),
		 0.000001);
  }

  // --------------------------------------------------------------------
  // Expand the root node and evaluate.
  private void setup2 ()
  {
    setup1();
    tbp1 = tbp.expandNode(b1, 1);
    eval.evaluate(tbp1);
  }

  public void testExpand1 ()
  {
    setup2();
    // this node should also be feasible but not covered
    assert("Should be feasible", tbp1.isFeasible());
    assert("Should not be covered", !tbp1.isCovered());
    // This should be node 1, and the parent of this node should be node 0
    assertEquals("Should be node 1", 1, tbp1.getNodeNumber());
    assertSame("Parent should be root node", tbp, tbp1.getParent());
    assertEquals("Check cost calc", 300, tbp1.getCost());
    assertEquals("Check overall eval",
		 6300.0,
		 tbp1.getEvaluation(),
		 0.000001);
  }

  // --------------------------------------------------------------------
  // Expand again and test bid locking
  private void setup3 ()
  {
    setup2();
    tbp1.lockBid(b1);
    tbp2 = tbp1.expandNode(b2, 2);
    tbp3 = tbp1.expandNode(b4, 3);
  }

  public void testExpand2 ()
  {
    setup3();
    assert("Test successful expansion", (tbp2 != null));

    eval.evaluate(tbp2);
    // this node should also be feasible but not covered
    assert(tbp2.isFeasible());
    assert(!tbp2.isCovered());
    assertEquals("Check cost", 540, tbp2.getCost());
    assertEquals("Check eval",
		 2540.0,
		 tbp2.getEvaluation(),
		 0.000001);
    // make sure correct tasks are uncovered...
  }

  public void testLocking ()
  {
    assert("Expansion should have failed", (tbp3 == null));
    tbp3 = tbp2.expandNode(b4, 3);
    assert("Locking should propagate to children", (tbp3 == null));
  }

  // --------------------------------------------------------------------
  // Add a bid that achieves coverage
  public void setup4 ()
  {
    setup3();
    tbp3 = tbp2.expandNode(b3, 3);
  }

  public void testCoverage ()
  {
    setup4();
    assert("Expansion should succeed", tbp3 != null);
    eval.evaluate(tbp3);
    // this node should be feasible and covered
    assert(tbp3.isFeasible());
    assert(tbp3.isCovered());
    assertEquals("Check cost ", 650, tbp3.getCost());
    assertEquals("Check eval", 650.0, tbp3.getEvaluation(), 0.000001);
    // should be just bids b1 and b3..
  }

  // --------------------------------------------------------------------
  // Check operation of tabu list & been-tried list
  public void testDuplicate ()
  {
    setup4();
    tbp4 = tbp3.expandNode(b3, 4);
    assert("Detect duplicate bid", (tbp4 == null));
  }

  public void testTabu ()
  {
    setup4();
    tbp4 = tbp3.expandNode(b2, 4);
    assert("Detect tabu bid", (tbp4 == null));
  }

  public void testTried ()
  {
    setup4();
    tbp4 = tbp2.expandNode(b3, 4);
    assert("Detect been-tried bid", (tbp4 == null));
  }

  // --------------------------------------------------------------------
  // Create nodes with subtask bids
  private void setup5 ()
  {
    setup4();
    tbp4 = tbp.expandNode(b1.getSubtaskBids()[2], 4);
  }

  public void testSTBid ()
  {
    setup5();
    assert("ST Bid Expansion", (tbp4 != null));
    eval.evaluate(tbp4);
    assertEquals("Cost", 90, tbp4.getCost());
    assertEquals("Eval", 10090.0, tbp4.getEvaluation(), 0.000001);
  }

  public void testStLock ()
  {
    setup5();
    tbp4.lockBid(b1.getSubtaskBids()[2]);
    // This one should be rejected due to overlap with locked subtask bid
    TaskBidPlan tbp5 = tbp4.expandNode(b1, 5);
    assert("Locked ST Bid test", (tbp5 == null));
  }

  public void testStExpand ()
  {
    setup5();
    tbp4.lockBid(b1.getSubtaskBids()[2]);
    tbp5 = tbp4.expandNode(b3, 5);
    assert("Expansion should succeed", (tbp5 != null));
    eval.evaluate(tbp5);
    assertEquals("Cost", 440, tbp5.getCost());
    assertEquals("Eval", 4440.0, tbp5.getEvaluation(), 0.000001);
  }
}

