/*-------------------------------------------------------------------------
 * Magnet unit tests
 * $Id: TaskBidPlanTest.java,v 1.1.2.1 2002/05/05 19:41:52 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *-------------------------------------------------------------------------
 * Revision History:
 * 
 * Mon Jan 29 10:05:43 2001  John Collins
 *   Split out Search Node functionality from TaskBidPlan
 * 
 * Fri Dec 29 16:40:28 2000  John Collins
 *   Eliminate subtask bids
 * 
 * Sun Dec 10 13:33:50 2000  John Collins
 *   Convert to JUnit format
 * 
 * Thu Oct  1 11:09:44 1998  John E. Collins
 *   Add tests for subtask bid expansion
 * 
 * Mon Sep 28 22:08:43 1998  John E. Collins
 *   Initial entry
 */

package edu.umn.magnet.client.eval;

import edu.umn.magnet.client.SampleTaskPlan;
import edu.umn.magnet.protocol.cust.Bid;
import edu.umn.magnet.protocol.cust.TaskPlan;
import edu.umn.magnet.protocol.InvalidDataException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
//import java.io.*;

/**
 * This is a tester for edu.umn.magnet.client.eval.Eval and related
 * stuff.
 * @author John Collins
 */
public class TaskBidPlanTest extends TestCase
{

  private static final long s_msph = 3600000l;

  private TaskPlan plan = null;
  private TaskBidPlan tbp = null;
  private TaskBidPlan tbp1 = null;
  private TaskBidPlan tbp2 = null;
  private TaskBidPlan tbp3 = null;
  private TaskBidPlan tbp4 = null;
  private TaskBidPlan tbp5 = null;
  private Bid b1 = new Bid();
  private Bid b2 = new Bid();
  private Bid b3 = new Bid();
  private Bid b4 = new Bid();
  private Bid b5 = new Bid();
  private Bid b6 = new Bid();
  
  public TaskBidPlanTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(TaskBidPlanTest.class);
  }

  private void setup1 ()
  {
    plan = SampleTaskPlan.generateTaskPlan();
    tbp = new TaskBidPlan(plan);
  }

  // --------------------------------------------------------------------
  // Create a TaskBidPlan and check it out
  public void testTBPCreate ()
  {
    setup1();
    assertSame("Compare plan", plan, tbp.getTaskPlan());
    assertSame("Correct root", tbp, tbp.getRoot());

    // Check for correct number of TaskBid objects
    Iterator tbs = tbp.getTaskBids().iterator();
    int tbCount = 0;
    while (tbs.hasNext()) {
      tbCount++;
      tbs.next();
    }
    assertEquals("Correct count of TaskBid objects", 6, tbCount);

    // Make sure we can find the root of the underlying TaskPlan
    Iterator rts = tbp.findRoots();
    Vector rtv = new Vector();
    while (rts.hasNext()) {
      rtv.addElement(rts.next());
    }
    assertEquals("Only one root", 1, rtv.size());
    assertEquals("Correct root task",
		 "T1",
		 (String)rtv.elementAt(0));
    // There's lots more to test here...rtv.elementAt(0);
  }

  // --------------------------------------------------------------------
  // Expand the root node and evaluate.

  private void setup2 ()
  {
    setup1();

    // Create some Bids against the TP
    try {
      b1 = new Bid();
      b1.setId("B1");
      b1.addTask("T2");
      b1.addTask("T4");
      b1.addTask("T6");
      
      b2 = new Bid();
      b2.setId("B2");
      b2.addTask("T1");
      b2.addTask("T3");
      
      b3 = new Bid();
      b3.setId("B3");
      b3.addTask("T1");
      b3.addTask("T3");
      b3.addTask("T5");
      
      b4 = new Bid();
      b4.setId("B4");
      b4.addTask("T1");
      b4.addTask("T2");
      b4.addTask("T3");
    }
    catch (InvalidDataException ide) {
      fail("InvalidDataException");
    }
  }

  public void testExpand1 ()
  {
    setup2();
    // Run the first expansion
    Set removals = tbp.addBid(b1);
    // there should be one bid
    assertEquals("One bid", 1, tbp.getBids().size());
    // TaskBid for T2 should have B1 as bid
    TaskBid tb1 = tbp.findTaskBid("T2");
    assertEquals("B1 mapped to T2", "B1", tb1.getBid().getId());
    // No bids to remove
    assertEquals("No bids to remove", 0, removals.size());
  }

  public void testExpand2 ()
  {
    setup2();
    Set removals = tbp.addBid(b1);
    removals = tbp.addBid(b4);
    // One bid to remove
    assertEquals("One bid to remove", 1, removals.size());
    Iterator bs = removals.iterator();
    Bid rbid = (Bid)bs.next();
    assertEquals("It's B1", b1, rbid);
    // there should be one bid left
    assertEquals("One bid", 1, tbp.getBids().size());
    // TaskBid for T2 should have B1 as bid
    TaskBid tb1 = tbp.findTaskBid("T2");
    assertEquals("B4 mapped to T2", "B4", tb1.getBid().getId());
  }
    
  // ---------- create TBP from another TBP ----------
  private void setup3 ()
  {
    setup2();
    tbp.addBid(b1);
  }

  public void testDup ()
  {
    setup3();
    TaskBidPlan tbpx = new TaskBidPlan(tbp);
    assertSame("Same TP", tbp.getTaskPlan(), tbpx.getTaskPlan());
    assertSame("Same root", tbp, tbpx.getRoot());
    assertEquals("Same bid count", 1, tbpx.getBids().size());
  }
}
