/*------------------------------------------------------------------------
 * MAGNET utility test components
 * $Id: TaskStatusTest.java,v 1.2.2.1 2002/05/05 19:41:53 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Jan  9 11:44:50 2002  John Collins  #0000037
 *   Test bid add/remove
 * 
 * Fri Dec 29 23:05:02 2000  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.status;

import edu.umn.magnet.protocol.cust.SubtaskRequest;
import edu.umn.magnet.protocol.cust.Bid;
import edu.umn.magnet.protocol.InvalidDataException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.Collection;
import java.util.List;
import java.util.ArrayList;

/**
 * This is a tester for edu.umn.magnet.client.Something
 * @author John Collins
 */
public class TaskStatusTest extends TestCase
{
  private SubtaskRequest task = new SubtaskRequest("t1");
  private Bid b1 = null;
  private Bid b2 = null;
  
  public TaskStatusTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(TaskStatusTest.class);
  }
  
  public void testCreate ()
  {
    TaskStatus ts = new TaskStatus(task);
    assertEquals("Correct task", task, ts.getTask());
    assertEquals("Default state", TaskStatus.NONE, ts.getState());
    assertEquals("Empty bid set", 0, ts.getBids().size());
  }

  public void testCreate2 ()
  {
    TaskStatus ts = new TaskStatus();
    assertEquals("No task", null, ts.getTask());
    ts.setTask(task);
    assertEquals("CorrectTask", task, ts.getTask());
    ts.setTask(new SubtaskRequest("t2"));
    assertEquals("Task not changed", task, ts.getTask());
  }

  public void testState ()
  {
    TaskStatus ts = new TaskStatus(task);
    ts.setState(TaskStatus.OFFERED);
    assertEquals("Correct state", TaskStatus.OFFERED, ts.getState());
    ts.setState(TaskStatus.ACTIVE);
    assertEquals("Changed state", TaskStatus.ACTIVE, ts.getState());
  }

  public void testStateSet ()
  {
    TaskStatus ts = new TaskStatus(task);
    ts.setState(TaskStatus.OFFERED);
    Collection states = new ArrayList();
    states.add(TaskStatus.PENDING);
    states.add(TaskStatus.AWARDED);
    assert("Not in states", !ts.isInStates(states));
    ts.setState(TaskStatus.AWARDED);
    assert("In states", ts.isInStates(states));
  }

  private void setup1()
  {
    try {
      b1 = new Bid();
      b1.setId("B1");
      b1.addTask("t1");
      b1.addTask("t4");

      b2 = new Bid();
      b2.setId("B1");
      b2.addTask("t1");
      b2.addTask("t2");
    }
    catch (InvalidDataException ide) {
      fail("InvalidDataException");
    }
  }

  public void testBids1 ()
  {
    TaskStatus ts = new TaskStatus(task);
    assertEquals("No bids present initially", 0, ts.getBids().size());
  }

  public void testBids2 ()
  {
    setup1();
    TaskStatus ts = new TaskStatus(task);
    ts.addBid(b1);
    List theBids = ts.getBids();
    assertEquals("One bid present", 1, theBids.size());
    assertEquals("Correct bid present", b1, theBids.get(0));
  }

  public void testBids3 ()
  {
    setup1();
    TaskStatus ts = new TaskStatus(task);
    ts.addBid(b1);
    assertEquals("One bid present", 1, ts.getBids().size());
    ts.removeBid(b1);
    assertEquals("No bids present", 0, ts.getBids().size());
  }

  public void testBids4 ()
  {
    setup1();
    TaskStatus ts = new TaskStatus(task);
    ts.addBid(b1);
    ts.addBid(b2);
    List theBids = ts.getBids();
    assertEquals("Two bids present", 2, theBids.size());
    assertEquals("Correct bid present", b1, theBids.get(0));
    assertEquals("Correct bid present", b2, theBids.get(1));
  }

  public void testBids5 ()
  {
    setup1();
    TaskStatus ts = new TaskStatus(task);
    ts.addBid(b1);
    ts.addBid(b2);
    assertEquals("Two bids present", 2, ts.getBids().size());
    ts.removeBid(b1);
    assertEquals("One bid present", 1, ts.getBids().size());
    ts.removeBid(b2);
    assertEquals("No bids present", 0, ts.getBids().size());
  }
}
