/*------------------------------------------------------------------------
 * Magnet plan generation components
 * $Id: PlanFactory.java,v 1.2.2.1 2002/05/05 19:41:53 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Jan  6 09:52:04 1999  John E. Collins
 *   Make this an abstract class, move printSuccessorTree in from
 *   MixedPlanFactory. 
 * 
 * Sat Jan  2 13:53:05 1999  John E. Collins
 *   Move to plan package
 */

package edu.umn.magnet.client.plan;

import edu.umn.magnet.protocol.cust.TaskPlan;
import edu.umn.magnet.protocol.cust.SubtaskRequest;

import org.apache.log4j.Category;

import java.util.Date;
import java.util.Iterator;

/**
 * Ancestor for all of the PlanFactory objects - to create all kinds
 * of TaskPlans for testing.  The makePlan methods are static, since
 * you don't really need a factory object, just the plan.
 * @author Max Tsvetovat
 * @version 0.0.1
 */
public abstract class PlanFactory 
{
  /**
   * This method must be overridden by each factory type
   */
  public static TaskPlan makePlan (int numberOfTasks,
				   Date startDate,
				   int planSlack,
				   int CPMSlack)
  {
    return null;
  }

  /**
   * Print a TaskPlan as a tree based on precedence relations.
   */
  public static void printSuccessorTree (Category log,
					 TaskPlan plan,
					 Iterator taskList,
					 int indent) 
  {
    SubtaskRequest task;
    StringBuffer output = new StringBuffer();

    while (taskList.hasNext()) {
      for (int i = 0; i < indent; i++) {
        output.append("  ");
      }
      task = (SubtaskRequest)taskList.next();
      output.append(task.getId());
      log.info(output);
      output.setLength(0);
      printSuccessorTree(log, plan, plan.getSuccessors(task), indent + 1);
    }
  }
}
