/*------------------------------------------------------------------------
 * Magnet UI components
 * $Id: XAutoCustomer.java,v 1.1.2.2 2002/05/02 03:40:29 jcollins Exp $
 * Copyright (c) 1998-2001 University of Minnesota
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Mon Feb 25 12:32:40 2002  John Collins  #0000198
 *   Dump out random number seeds at the beginning of each problem.
 * 
 * Wed Dec 12 21:31:09 2001  John Collins  #0000176
 *   Move to xac package.  Was "1.28 2001/12/07"
 * 
 * Sun Oct  7 19:02:16 2001  John Collins  #0000162
 *   Check event type before assuming it's a new result in SearchMonitor
 * 
 * Wed Jul 18 12:24:54 2001  John Collins
 *   Add ability to collect multiple search results and calculate
 *   Weibull MLE parameter values.
 * 
 * Fri May  4 14:42:37 2001  John Collins
 *   Revise for new status implementation
 * 
 * Sun Apr 22 12:52:43 2001  John E. Collins
 *   #0000026 make JobInfoFactory be a singleton
 * 
 * Fri Apr 20 23:45:06 2001  John Collins
 *   Eliminate use of System properties
 *
 * Mon Apr 16 12:57:00 2001  Len Josephs
 *   Added usage of the BidCollectorFactory.
 *
 * Tue Mar 20 21:50:00 2001  John Collins
 *   Load list of Scheme scripts from command line args
 * 
 * Tue Mar  6 12:18:04 2001  John Collins
 *   Move Factory stuff out of protocol.JobInfo into plan.JobInfoFactory
 * 
 * Sun Feb 25 09:42:06 2001  John Collins
 *   Move to bidevalTest component, split out XBidCollector
 * 
 * Tue Nov 28 10:05:38 2000  John Collins
 *   New logging format
 * 
 * Mon Sep 25 11:10:46 2000  John Collins
 *   Clean up selection of search type
 * 
 * Tue Dec  7 11:28:30 1999  John E. Collins
 *   Fill in the session ID in the BidEvaluator.
 * 
 * Wed May 10 23:40:33 1999 Rashmi Sundareswara
 *   Pick the appropriate Search based on the property in the property file
 *
 * Sun Jan  3 11:47:48 1999  John E. Collins
 *   Use PropertyFile to handle input parameters, change to single-run
 *   model, make Plan Factories static.
 * 
 * Sun Dec 20 18:57:23 1998  John E. Collins
 *   Restructured to get rid of unnecessary recursion
 *   Use BidEvaluator to absorb bids
 *
 * Wed Sep 30 16:24:32 CDT 1998 Max Tsvetovat
 *   Automated experiments 
 *   Use of PlanFactories
 * 
 * Fri Jul 31 18:58:47 1998  John E. Collins
 *   Update agent structure
 *
 * Mon Jun 29 10:31:23 1998  John E. Collins
 *   Package update
 */

package edu.umn.magnet.client.xac;

import edu.umn.magnet.client.bid.BidCollector;
import edu.umn.magnet.client.bid.BidCollectorFactory;
import edu.umn.magnet.client.bid.BidCommand;
import edu.umn.magnet.client.bid.BidManager;
import edu.umn.magnet.client.eval.BidEvaluator;
import edu.umn.magnet.client.eval.TaskBidPlan;
import edu.umn.magnet.client.plan.JobInfoFactory;
import edu.umn.magnet.client.plan.PlanFactory;
import edu.umn.magnet.client.plan.RandomPlanFactory;
import edu.umn.magnet.client.status.AgentStatus;
import edu.umn.magnet.client.status.InvalidStateException;
import edu.umn.magnet.client.status.PlanStatus;
import edu.umn.magnet.client.status.RFQStatus;
import edu.umn.magnet.client.status.SessionIdFactory;
import edu.umn.magnet.client.status.SessionStatus;
import edu.umn.magnet.client.sup.SupplyMonster;
import edu.umn.magnet.protocol.MagnetException;
import edu.umn.magnet.protocol.cust.Bid;
import edu.umn.magnet.protocol.cust.RFQ;
import edu.umn.magnet.protocol.cust.TaskPlan;
import edu.umn.magnet.util.MagnetProperties;
import edu.umn.magnet.util.Copyright;
import edu.umn.magnet.util.time.AgentTime;
import edu.umn.magnet.util.rvars.RVFManager;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import org.apache.log4j.Category;
import org.apache.log4j.Priority;
import org.apache.log4j.FileAppender;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.PropertyConfigurator;
import silk.SI;

/**
 * Experimental harness for testing bid-evaluation search
 */
public class XAutoCustomer 
{
  // init the log category
  static Category log = Category.getInstance(XAutoCustomer.class.getName());
  static Category resultLog = Category.getInstance("Result");

  static int s_idCounter = 0;
  static final long HOUR = 3600000L;
  static final long MINUTE = 60000L;

  // option constants
  static final int OPTION_DEFAULT = 0;
  static final int OPTION_PLAN_OUT= 1;
  static final int OPTION_PLAN_IN = 2;
  static final int OPTION_PROBLEM_OUT = 3;
  static final int OPTION_PROBLEM_IN  = 4;

  // option specification table
  static OptionSpec[] s_options = {new OptionSpec("-planOut",
						  OPTION_PLAN_OUT,
						  true),
				   new OptionSpec("-planIn",
						  OPTION_PLAN_IN,
						  true),
				   new OptionSpec("-problemOut",
						  OPTION_PROBLEM_OUT,
						  true),
				   new OptionSpec("-problemIn",
						  OPTION_PROBLEM_IN,
						  true)};

  // static class database
  private static ArrayList s_optionRequests = new ArrayList();
  private static String s_filename = null;
  private static int s_numSubtasks = 10;
  private static int s_nRuns = 1;
  private static int s_nSearches = 1;
  private static boolean s_logSeeds = false;
  private static double[] s_tolerance = null;
  private static double s_defaultTolerance = 0.00001;

  /**
   * Default constructor
   */
  public XAutoCustomer() 
  {
    super ();
  }

  /**
   * Runs a single experiment as specified by properties.
   */
  public void runExperiment ()
  {
    Date now = AgentTime.getDate();
    String sessionId
	= SessionIdFactory.getInstance("")
	  .getSessionId(MagnetProperties.getProperty("ExperimentId", "1"));
    log.info("-------------------------------------------------");
    System.out.println("Starting run " + sessionId);
    log.info("New run: session id = " + sessionId);
    if (s_logSeeds) {
      RVFManager.cycleSeeds();
      RVFManager.logSeeds(Priority.INFO);
    }
    
    String planType = MagnetProperties.getProperty("PlanType", "random");

    //Create the plan and hand it off to the Bid Manager
    TaskPlan plan = RandomPlanFactory.makePlan
	(s_numSubtasks);
    PlanFactory.printSuccessorTree(log, plan, plan.getSuccessorRoots(), 0);

    // Make jobs start on the hour to make the trace output easier to
    // read...
    Date startTime = AgentTime.getDate();
    startTime.setTime((startTime.getTime() + (HOUR * 2))
		      - (startTime.getTime() % HOUR));
    if (s_nSearches == 1)
      processStandard(sessionId, plan, startTime);
    else
      processMultiple(sessionId, plan, startTime);
  }

  private void processStandard (String sessionId, 
				TaskPlan plan,
				Date startTime)
  {
    try {
      PlanStatus planStatus
	  = AgentStatus.getInstance()
	    .createPlanStatus(new TaskBidPlan(plan),
			      startTime, null);
      SessionStatus sessionStatus
	  = planStatus.createSessionStatus(sessionId);
      BidManager mgr = new BidManager(sessionStatus,
				      MagnetProperties.getProperties(),
				      true);
      mgr.waitForCompletion();
      planStatus.setPlanState(PlanStatus.ABORTED);
    }
    catch (InvalidStateException ise) {
      log.error("Exception creating status objects: " + ise);
    }
  }

  private void processMultiple (String sessionId,
				TaskPlan plan,
				Date startTime)
  {
    Date now = new Date();
    Date n1 = new Date(now.getTime() + MINUTE);
    Date n2 = new Date(n1.getTime() + MINUTE);
    try {
      PlanStatus planStatus
	  = AgentStatus.getInstance()
	    .createPlanStatus(new TaskBidPlan(plan),
			      startTime, null);
      SessionStatus sessionStatus
	  = planStatus.createSessionStatus(sessionId);
      BidManager mgr = new BidManager(sessionStatus,
				      MagnetProperties.getProperties(),
				      false);
      //mgr.waitForCompletion();
      resultLog.info("-------- Start multiple-search session "
		     + sessionId + " --------");

      // acquire some bids
      BidCommand cmd = new BidCommand(plan.getTasks(), n1, n2, n2, n2);
      RFQ rfq = mgr.composeRFQ(cmd);
      RFQStatus rstat = mgr.createRFQStatus(rfq);
      rstat.acquireBids();
      while (rstat.getState() != RFQStatus.BIDS_ACQUIRED)
	Thread.sleep(200);

      Properties props = MagnetProperties.getProperties();
      String originalSearch = props.getProperty("Search");
      BidEvaluator be = new BidEvaluator();
      TaskBidPlan optimum = null;
      String omitIP = props.getProperty("Search.omitIP");
      if (omitIP == null || !omitIP.equals("true")) {
	// create a BidEvaluator to run the IP search - that will give
	// us the optimum value to work with.  If no result is possible,
	// we should give up at this point.
	props.setProperty("Search", BidEvaluator.IPCON);
	be.setProperties(props);
	be.setSessionId(sessionStatus.getSessionId());
	be.setPlanStatus(planStatus);
	be.setRFQStatus(rstat);
	for (Iterator theBids = rstat.getBids().iterator();
	     theBids.hasNext(); ) {
	  be.addBid((Bid)theBids.next());
	}
	optimum = be.evaluateBids();
	props.setProperty("Search", originalSearch);
	if (optimum == null) {
	  System.out.println("No solution, aborting");
	  planStatus.setPlanState(PlanStatus.ABORTED);
	  return;
	}
      }

      // for the specified number of trials, create a new BidEvaluator
      // and run the SA search engine.
      be.setSearchProperty();
      SearchMonitor sm = new SearchMonitor(be, optimum);
      for (int i = 0; i < s_nSearches; i++) {
	be.evaluateBids();
	sm.nextSearch();
      }
      sm.logWeibullMLE();

      planStatus.setPlanState(PlanStatus.ABORTED);
    }
    catch (InvalidStateException ise) {
      log.error("Exception creating status objects: " + ise);
    }
    catch (MagnetException me) {
      log.error("Exception acquiring bids: " + me);
    }
    catch (InterruptedException ie) {
      log.info("Interrupted...");
    }
  }

  /**
   * Main method reads command-line args, properties file, Scheme
   * scripts, then runs experiments as requested.  Results are in the
   * Result log file, as specified by the log4j config file
   * "client.lcf", except that the name of the results file will be
   * "<tt>&lt;ExperimentId&gt;.result</tt>" where
   * <tt>ExperimentId</tt> is taken from the property of that name in
   * the properties file.  The execution trace info is similarly put
   * into a file called "<tt>&lt;ExperimentId&gt;.trace</tt>".
   * <p>
   * <strong>Command line syntax:</strong><br>
   * <tt>XAutoCustomer [option]* props [script]*</tt><br>
   * where each <tt>[option]</tt> is one of
   * <dl>
   * <dt>blank</dt>
   *   <dd>
   *   Run the set of experiments specified by the props file.  Each
   *   run involves generating a new plan, generating bids for that
   *   plan, and evaluating the bids.
   *   </dd>
   * <dt><tt>-planOut &lt;filename&gt;</tt></dt>
   *   <dd>
   *   Generate a single plan according to the properties settings, and
   *   save it in the named file.
   *   </dd>
   * <dt><tt>-planIn &lt;filename&gt;</tt></dt>
   *   <dd>
   *   Run the set of experiments specified by the props file, except
   *   that the plan contained in the specified file is used for each
   *   one instead of generating new plans.
   *   </dd>
   * <dt><tt>-problemOut &lt;filename&gt;</tt></dt>
   *   <dd>
   *   Generate a single plan and a set of bids according to the
   *   properties settings, and save plan and bids in the named file.
   *   </dd>
   * <dt><tt>-problemIn &lt;filename&gt;</tt></dt>
   *   <dd>
   *   Run the set of experiments specified by the props file, except
   *   that the plan and bid set contained in the specified file is
   *   used for each one instead of generating new plans and bids.
   *   </dd>
   * </dl>
   * <tt>props</tt> is the name of the properties file, and each
   * <tt>script</tt> is a filename of a Scheme script (presumably
   * associated with some hook) that will be loaded and evaluated at
   * system startup.
   * </p>
   * <p><strong>Note:</strong> The planIn, planOut, problemIn, and
   * problemOut options are not implemented yet.
   * </p>
   */
  static public void main(String[] args) 
  {
    Copyright.print();
    if (args.length<1) 
    {
      System.out.println("Usage: XAutoCustomer [options] <property file> <script>...");
      System.exit(0);
    }

    int currentArg = 0;
    int newArg = 0;

    // collect command-line options
    while ((newArg = processOptions(args, currentArg)) > currentArg)
      currentArg = newArg;

    // init properties
    MagnetProperties.readProps(args[currentArg++]);

    // Init the log4j logging system.  Default log file names come from
    // client.lcf, but are overridden by the experimentId property if
    // it exists.  Note that the filename used by the older tools.Log
    // system may be in conflict, so older property files may need to
    // be changed.
    //   here's the default log setup:
    PropertyConfigurator.configure
	(MagnetProperties.getProperty("LogConfiguration", "client.lcf"));
    //   now we modify it if we have a valid experiment ID
    String expid = MagnetProperties.getProperty("ExperimentId");
    if (expid != null) {
      // modify the filenames of the trace and result logfiles
      Category rootC = Category.getRoot();
      Category resultC = Category.getInstance("Result");
      rootC.removeAllAppenders();
      resultC.removeAllAppenders();
      try {
	PatternLayout logLayout = new PatternLayout("%-5p %c{2}: %m%n");
	FileAppender logFile
	    = new FileAppender(logLayout, (expid + ".trace"), false);
	rootC.addAppender(logFile);
	PatternLayout resLayout = new PatternLayout("%m%n");
	FileAppender resultFile
	    = new FileAppender(resLayout, (expid + ".result"), false);
	resultC.addAppender(resultFile);
      }
      catch (IOException ioe) {
	System.out.println("Can't open log file");
	System.exit(0);
      }
    }

    // Load any Scheme scripts provided on the command line
    edu.umn.magnet.util.silk.SilkHook.init();
    for (int i = currentArg; i < args.length; i++) { 
      try {
        SI.load(new java.io.FileReader(new java.io.File(args[i])));
        log.info("Loaded Scheme script " + args[i]);
      } catch (java.io.FileNotFoundException fnf) {
        System.out.println("+++ Can't find " + args[i]);
        log.error("Can't find Scheme script " + args[i]);
      }
    }

    // Set the JobInfoFactory instance, then process the TaskInfo data
    JobInfoFactory.instantiate();
    JobInfoFactory JIF = (JobInfoFactory)JobInfoFactory.getInstance();
    // If there is a taskType spec in the properties, use it.  Otherwise,
    // get the JobInfo from the server.
    String taskTypes = MagnetProperties.getProperty("TaskTypes");
    if (taskTypes != null) {
      JIF.createFromSpec(taskTypes);
      log.info("TaskTypes = " + taskTypes);
    }
    else {
        log.error("Properties file does not contain enough information to "+
		  "get JobInfo from the server.  Exiting.");
        System.exit(-1);
    } // Else, get JobInfo from the server.

    // Pick up the "number of runs" and "number of searches" properties
    String prop = MagnetProperties.getProperty("NRuns");
    if (prop != null)
    {
      try
      {
	double value = Double.valueOf(prop).doubleValue();
	s_nRuns = (int)Math.round(value);
	log.info("Number of runs = " + s_nRuns);
      }
      catch (NumberFormatException nfe)
      {
	log.error("Bad property spec " + prop
		  + " for number of runs.  Using "
		  + s_nRuns);
      }
    }
    prop = MagnetProperties.getProperty("NSearches");
    if (prop != null)
    {
      try
      {
	double value = Double.valueOf(prop).doubleValue();
	s_nSearches = (int)Math.round(value);
	log.info("Number of searches = " + s_nSearches);
      }
      catch (NumberFormatException nfe)
      {
	log.error("Bad property spec " + prop
		  + " for number of searches.  Using "
		  + s_nSearches);
      }
    }

    // Pick out the LogSeeds property    
    prop = MagnetProperties.getProperty("LogSeeds");
    if (prop != null)
    {
      s_logSeeds = Boolean.valueOf(prop).booleanValue();
      log.info("LogSeeds = " + s_logSeeds);
    }
    
    // Pick out the "PlanSize" property
    try {
      s_numSubtasks = (int)Math.round((Double.valueOf
			   (MagnetProperties.getProperty("PlanSize", "10"))).doubleValue());
      log.info("PlanSize = " + s_numSubtasks);
    } catch (NumberFormatException nfe) {
      log.error("Error in property file. Exiting", nfe);
      nfe.printStackTrace();
      System.exit(1);
    }
    
    // Pick out the "Search Tolerance" property.  This can be a single
    // number or a space-separated sequence of numbers.  Each one is
    // used to capture the node-count when the search produces a
    // result that is within the specified tolerance of the optimum
    // value.  See the SearchMonitor inner class for how this is used.
    prop = MagnetProperties.getProperty("XA.tolerance");
    if (prop != null) {
      log.debug("XA.tolerance=" + prop);
      StringTokenizer tokens = new StringTokenizer(prop);
      int tcount = tokens.countTokens();
      s_tolerance = new double[tcount];
      int index = 0;
      while (tokens.hasMoreTokens())
      {
	String token = tokens.nextToken();
	try
	{
	  s_tolerance[index] = Double.valueOf(token).doubleValue();
	  log.info("Search Tolerance[" + index
		   + "] = " + s_tolerance[index]);
	  index += 1;
	}
	catch (NumberFormatException nfe)
	{
	  log.error("Bad property spec " + token
		    + " for search tolerance.");
	}
      }
    }
    else { // make sure the s_tolerance array isn't empty
      s_tolerance = new double[1];
      s_tolerance[0] = s_defaultTolerance;
    }

    // Set standard experiment-mode properties (overriding the
    // interactive stuff)
    MagnetProperties.setProperty("BidManager.Autopilot", "true");
    MagnetProperties.setProperty("RFQStatus.Bids.DontWait", "true");

    if (s_logSeeds) {
      // Instantiate all the RandomVariableFactories, to make the first
      // run the same as all the rest
      RVFManager.getRVF("PlanSeed");
      RVFManager.getRVF("BidSeed");
      RVFManager.getRVF("SearchSeed");
    }

    // Remainder depends on selected option
    if (s_optionRequests.size() == 0)
      defaultOp();
    else
      for (Iterator rqs = s_optionRequests.iterator();
	      rqs.hasNext(); ) {
	OptionRq request = (OptionRq)rqs.next();
	if (request.op == OPTION_PLAN_OUT)
	  planOutOp(request.fn);
	else if (request.op == OPTION_PLAN_IN)
	  planInOp(request.fn);
	else if (request.op == OPTION_PROBLEM_OUT)
	  problemOutOp(request.fn);
	else if (request.op == OPTION_PROBLEM_IN)
	  problemInOp(request.fn);
    }

    // (debugging) list active threads
    Thread[] tarray = new Thread[20];
    int tcount = Thread.currentThread().enumerate(tarray);
    for (int i = 0; i < tcount; i++) {
      System.out.println(tarray[i]);
    }
  }

  /**
   * Default operation: Runs the specified number of experiments
   */
  private static void defaultOp ()
  {
    // Run the specified number of experiments
    for (int i = 0; i < s_nRuns; i++)
    {
      XAutoCustomer ac = new XAutoCustomer();
      ac.runExperiment();
    }
  }

  private static void planOutOp (String filename)
  {
  }

  private static void planInOp (String filename)
  {
  }

  private static void problemOutOp (String filename)
  {
  }

  private static void problemInOp (String filename)
  {
  }

  /**
   * Processes command-line options.  Only one option is allowed,
   * which must begin with a "-" character.  If the option takes a
   * filename, that will also be processed.
   * @param the String array of command-line arguments.
   * @return the integer index of the first unprocessed command-line
   *   arg.
   */
  private static int processOptions(String[] args, int posn)
  {
    int result = posn;
    if (args[result].charAt(0) == '-') {
      // We have an option spec.
      String option = args[result];
      OptionSpec spec = null;
      int optionIndex = 0;
      while (spec == null && optionIndex < s_options.length) {
	if (option.equals(s_options[optionIndex].name))
	  spec = s_options[optionIndex];
      }
      if (spec != null) {
	String fn = null;
	result++;
	if (spec.needsFn) {
	  fn = args[result++];
	}
	s_optionRequests.add(new OptionRq(spec.op, fn));
      }
      else {
	// bad command line
	System.out.println("Bad option name " + args[result]);
	System.exit(0);
      }
    }
    return result;
  }

  /**
   * Inner class for specifying command-line option input format
   */
  private static class OptionSpec
  {
    String name;      // name that appears on command line  eg. "-op"
    int op;           // operation ID
    boolean needsFn;  // true if filename follows option

    OptionSpec (String name, int op, boolean needsFn)
    {
      this.name = name;
      this.op = op;
      this.needsFn = needsFn;
    }
  }

  /**
   * Inner class for capturing command-line option data
   */
  private static class OptionRq
  {
    int op;     // requested operation
    String fn;  // associated filename

    OptionRq (int op, String fn)
    {
      this.op = op;
      this.fn = fn;
    }
  }

  /**
   * Inner class for monitoring search results.  This is assumed to be
   * used only when multiple search experiments are being run.
   */
  private class SearchMonitor implements PropertyChangeListener
  {
    final double EPSILON = 0.0001;
    BidEvaluator be = null;
    TaskBidPlan optimum = null;
    int toleranceIndex = 0;
    int searchIndex = 0;
    // the results array is indexed first by tolerance values, and
    // second by search number.
    double[][] results = null;
    double[] weibullAlpha = null;
    double[] weibullBeta = null;

    SearchMonitor (BidEvaluator eval, TaskBidPlan opt)
    {
      be = eval;
      optimum = opt;
      eval.addListener(this);
      log.debug("SM setup: tolerances = " + s_tolerance.length
		+ ", nSearches = " + s_nSearches);
      if (s_nSearches > 1) {
	results = new double[s_tolerance.length][s_nSearches];
      }
    }

    /**
     * Handles the <code>PropertyChangeEvent</code> emitted by the
     * {@link edu.umn.magnet.client.eval.BidEvaluator BidEvaluator}
     * when a new search result is produced.  If the new result
     * satisfies the current tolerance, it is added to the appropriate
     * list in the <code>results</code> array.  Once that happens, the
     * next tolerance is selected (they must be sorted in descending
     * numerical order), or if there are no more the search is aborted
     * and the tolerance list is restarted.
     */
    public void propertyChange (PropertyChangeEvent pce)
    {
      if (!pce.getPropertyName().equals(BidEvaluator.NEW_RESULT))
	// ignore this if it's not a new result
	return;
      TaskBidPlan newtbp = (TaskBidPlan)pce.getNewValue();
      //System.out.println("opt=" + optimum + ", new=" + newtbp);
      log.debug("New result: Eval = " + newtbp.getEvaluation()
		+ ", nodeCount = " + newtbp.getNodeCount());
      while ((toleranceIndex < s_tolerance.length)
	     && (newtbp.getEvaluation()
		 <= (optimum.getEvaluation()
		     * (1.0 + s_tolerance[toleranceIndex])))) {
	results[toleranceIndex][searchIndex]
	    = (double)newtbp.getNodeCount();
	resultLog.info("SM: tol= " + s_tolerance[toleranceIndex]
		       + ", time = " + newtbp.getSearchTime()
		       + ", nc = " + newtbp.getNodeCount());
	toleranceIndex += 1;
      }
      if (toleranceIndex >= s_tolerance.length) {
	// we've run off the end
	log.info("Optimum value found by SA - aborting");
	be.abortSearch();
	// toleranceIndex = 0;
	// once the search ends, whether it's aborted or not,
	// nextSearch must be called.
      }
    }

    /**
     * Sets up for the next search.  This must be called regardless of
     * how the search ended.
     */
    void nextSearch ()
    {
      toleranceIndex = 0;
      searchIndex += 1;
    }

    /**
     * Writes the Weibull MLE data out to the Result log file
     */
    public void logWeibullMLE ()
    {
      StringBuffer out = new StringBuffer();
      out.append("Weibull MLE Parameters: ");
      for (int i = 0; i < s_tolerance.length; i++) {
	out.append(getWeibullAlpha(i)).append(", ");
	out.append(getWeibullBeta(i)).append(", ");
      }
      resultLog.info(out);
    }

    /**
     * Ensures that the Maximum Likelihood Estimators have been
     * computed for a Weibull
     * distribution to match the node-count values in the results
     * arrays.  The output is stored in the weibullMLE array.
     */
    private void ensureWeibullMLE ()
    {
      if (weibullAlpha != null)
	return;
      // compute alpha and beta values
      weibullAlpha = new double[s_tolerance.length];
      weibullBeta = new double[s_tolerance.length];
      for (int i = 0; i < s_tolerance.length; i++) {
	log.debug("Computing MLE values for tolerance " + s_tolerance[i]);
	weibullAlpha[i] = computeWAlpha(results[i]);
	weibullBeta[i] = computeWBeta(results[i], weibullAlpha[i]);
      }
    }

    /**
     * Computes the Weibull Alpha value for an array of ints.  The
     * computation involves an initial estimate followed by a
     * relaxation based on Newton's method.  There is lots of summing
     * of the array and summing of elements of the array to the power
     * of the current estimate.  The source is Law & Kelton,
     * Simulation Modeling and Analysis, McGraw-Hill 1991, pp 334.
     */
    private double computeWAlpha (double[] values)
    {
      double n = (double)values.length;
      // first the initial estimate:
      double sumlog = 0.0;
      double sumlogsq = 0.0;
      for (int i = 0; i < values.length; i++) {
	double vlog = Math.log(values[i]);
	sumlog += vlog;
	sumlogsq += vlog * vlog;
      }
      double num = ((sumlogsq - (sumlog * sumlog / n))
		    * (6.0 / (Math.PI * Math.PI)));
      double alphak1 = Math.pow((num / (n - 1.0)), -0.5);
      log.debug("initial Alpha: " + alphak1);
      
      // At this point, we have the initial alpha estimate.  Now
      // starts the fixed-point calculation.
      double alphak = alphak1 * 2;
      double a = sumlog / n;
      while (Math.abs(1.0 - (alphak / alphak1)) > EPSILON) {
	alphak = alphak1;
	double b = 0.0;
	double c = 0.0;
	double h = 0.0;
	for (int i = 0; i < values.length; i++) {
	  double vlog = Math.log(values[i]);
	  double xalpha = Math.pow(values[i], alphak);
	  b += xalpha;
	  c += xalpha * vlog;
	  h += xalpha * vlog * vlog;
	}
	alphak1 = (alphak
		   + ((a + (1.0 / alphak) - (c / b))
		      / ((1.0 / (alphak * alphak))
			 + ((b * h - c * c) / (b * b)))));
      }
      log.debug("final Alpha: " + alphak1);
      return alphak1;
    }

    /**
     * Computes the Weibull Beta value for the given array and alpha
     * value.
     */
    private double computeWBeta (double[] values, double alpha)
    {
      double beta = 0.0;
      double sumx = 0.0;
      for (int i = 0; i < values.length; i++) {
	sumx += Math.pow(values[i], alpha);
      }
      beta = Math.pow((sumx / (double)values.length),
		      (1.0 / alpha));
      log.debug("final Beta: " + beta);
      return beta;
    }

    /**
     * Indexed getter for a Weibull alpha value.
     */
    public double getWeibullAlpha (int index)
    {
      ensureWeibullMLE();
      return weibullAlpha[index];
    }

    /**
     * Indexed getter for a Weibull beta value.
     */
    public double getWeibullBeta (int index)
    {
      ensureWeibullMLE();
      return weibullBeta[index];
    }
  }
}
