/*------------------------------------------------------------------------
 * MAGNET bid evaluation experiment support component
 * $Id: XBidCollector.java,v 1.1.2.1 2002/05/05 19:41:55 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Dec 12 21:55:28 2001  John Collins  #0000176
 *   Move to xac package.  Revision was 1.7 2001/10/17
 * 
 * Fri Apr 20 23:46:03 2001  John Collins
 *   Eliminate use of system properties
 *
 * Sun Apr 15 21:08:00 2001  Len Josephs
 *   Added default constructor.
 * 
 * Sun Feb 25 11:37:37 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.client.xac;

import edu.umn.magnet.protocol.cust.RFQ;
import edu.umn.magnet.client.bid.AbstractBidCollector;
import edu.umn.magnet.client.sup.SupplyMonster;
import edu.umn.magnet.util.MagnetProperties;

import org.apache.log4j.Category;

import java.util.Collection;

/**
 * This class is an implementation of the BidCollector interface that
 * uses the SupplyMonster as a source of bids.  You use this by
 * creating an instance with a SupplyMonster, then handing the
 * instance to the BidManager.  The BidManager will then use it to
 * collect bids at the appropriate point in its operation.
 * @author John Collins
 * @version $Revision: 1.1.2.1 $ */
public class XBidCollector extends AbstractBidCollector
{
  private static Category log =
    Category.getInstance(XBidCollector.class.getName());

  private SupplyMonster m_sm = null;

  /**
   * If you already have a SupplyMonster somehow, you can use that.
   */
  public XBidCollector (SupplyMonster sm)
  {
    super();
    m_sm = sm;
  }

  /**
   * The default (parameterless) constructor will create an XBidCollector
   * using a Supply Monster based on the system properties.
   */
  public XBidCollector() {
    super();
    m_sm = SupplyMonster.create(MagnetProperties.getProperties());
  }

  public Collection collectBids (RFQ cfb)
  {
    return m_sm.gimmeTheBids(cfb);
  }
}
