/* ---------------------------------------------------------------------
 * $Id: CompletionAccept.java,v 1.2 2001/03/06 21:08:13 jcollins Exp $
 * Copyright (c) 1998 University of Minesota
 * ---------------------------------------------------------------------
 * Revision History:
 * 
 * Thu Feb 22 17:17:50 2001  John Collins
 *   Move to customer protocol component.
 * 
 * Thu Oct 29 15:59:46 1998  John E. Collins
 *   Add serialization signature.
 * 
 * Sat Jun 27 09:41:50 1998  John E. Collins
 *   Move to protocol package.
 * 
 * Wed Jun  3 14:56:43 1998  John E. Collins
 *   Clean up redundant code, fix formatting
 */

package edu.umn.magnet.protocol.cust;

import edu.umn.magnet.protocol.ProtocolElement;

/**
 * @author Brad Henrickson
 * An assertion by the customer that it is satisfied with the completion
 * asserted by a supplier.
 */

public class CompletionAccept extends ProtocolElement
{
  // private data members
  private	String	_bidId;
  private	String	_subtaskBidId;

  // public member functions
  
  /**
   * Default Constructor
   */
  public CompletionAccept()
  {
    super();
    _bidId = null;
    _subtaskBidId = null;
  }
  
  /**
   * Returns the ID of the presently associated Customer
   *
   * @return	a <code>String</code> which is the ID of the
   *            presently registered bid
   */
  public String getBidId()
  {
    return _bidId; 
  }

  /**
   * Assigns the Id of the <code>Bid</code> associated with this
   *   <code>CompletionAccept</code>
   * 
   * @param  the newBidId is the unique Id for the Bid  which will be
   *    associated with this <code>CompletionAccept</code>
   *
   * @throws  a <code>InvalidDataException</code> if the value to be
   *          changed to is not legal
   */
  public void setBidId(String newBidId)
  {
    _bidId = newBidId;
  }
}
