/* ---------------------------------------------------------------------
 *  Magnet Protocol elements
 *  $Id: RFQ.java,v 1.4.2.1 2002/05/05 19:41:56 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 * ---------------------------------------------------------------------
 * Revision History:
 *
 * Mon Apr 16 14:50:00 2001  Len Josephs
 *   Added a toString() method for debugging.
 *
 * Thu Feb 22 17:15:12 2001  John Collins
 *   Move to customer protocol component.
 * 
 * Tue Nov 23 12:15:26 1999  John Collins
 *   Remove System.out printouts
 * 
 * Thu Oct 29 15:55:21 1998  John E. Collins
 *   Add serialization signature.
 * 
 * Sat Jun 27 09:37:30 1998  John E. Collins
 *   Move to protocol package.
 * 
 * Wed Jun  3 14:43:23 1998  John E. Collins
 *   Remove redundant inherited code, fix up formatting.
 */
package edu.umn.magnet.protocol.cust;

import edu.umn.magnet.protocol.ProtocolElement;
import edu.umn.magnet.protocol.InvalidDataException;

import edu.umn.magnet.util.time.AgentTime;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.String;
import java.util.Date;

//import edu.umn.magnet.server.Session;

/**
 *  A request for bids on a subtasks w/in a task plan.
 *  @author      Ben Youngdahl
 */
public class RFQ
  extends ProtocolElement
  implements java.io.Serializable
{
  private Date _bidDeadline;
  private Date _earliestOffer;
  private Date _earliestConsider;
  private TaskPlan _taskplan;
  //protected byte[] _taskPlanS;
  
  public RFQ ()
  {
    //System.out.println("Making a call-for-bids");
    _bidDeadline = AgentTime.getDate ();
    _earliestOffer = AgentTime.getDate ();
    _earliestConsider = AgentTime.getDate();
    //_taskPlanS = null;
    _taskplan = null;
  }
  
  public void setBidDeadline (Date date)
    throws InvalidDataException
  {
    if (!isLegalBidDeadline(date))
      throw new InvalidDataException ("Bid deadline invalid.");
    _bidDeadline = date;
  }
  
  public Date getBidDeadline ()
  {
    return _bidDeadline;
  }
  
  private boolean isLegalBidDeadline (Date date)
  {
    return (date != null);
  }
  
  public void setEarliestOffer (Date date)
    throws InvalidDataException
  {
    if (!isLegalEarliestOffer (date))
      throw new InvalidDataException ("Earliest offer invalid.");
    _earliestOffer = date;
  }
  
  public Date getEarliestOffer ()
  {
    return _earliestOffer;
  }
  
  private boolean isLegalEarliestOffer (Date date)
  {
    return (date != null);
  }
  
  public void setEarliestConsider (Date date)
    throws InvalidDataException
  {
    if (!isLegalEarliestConsider (date))
      throw new InvalidDataException ("Earliest consider invalid.");
    _earliestConsider = date;
  }

  public Date getEarliestConsider ()
  {
    return _earliestConsider;
  }

  private boolean isLegalEarliestConsider (Date date)
  {
    return (date != null);
  }

  public void setTaskPlan (TaskPlan plan)
    throws InvalidDataException
  {
    //System.out.println("marshalling task plan");
    if (!isLegalTaskPlan (plan))
      throw new InvalidDataException ("Task plan invalid.");
    _taskplan = plan;
    //try {
    //  ByteArrayOutputStream sw = new ByteArrayOutputStream();
    //  ObjectOutputStream os = new ObjectOutputStream(sw);
    //  os.writeObject(plan);
    //  _taskPlanS = sw.toByteArray();
    //}
    //catch (Exception e) {
    //  e.printStackTrace();
    //}
  }

  public TaskPlan getTaskPlan ()
  {
    return _taskplan;
    //System.out.println("unmarshalling task plan");
//      try {
//        if (_taskPlanS == null) {
//  	System.out.println("Null task plan");
//  	throw new Exception("Null task plan");
//        }
//        ByteArrayInputStream sr = new ByteArrayInputStream(_taskPlanS);
//        ObjectInputStream is = new ObjectInputStream(sr);
//        return (TaskPlan)is.readObject();
//      }
//      catch (Exception e) {
//        e.printStackTrace();
//        return null;
//      }
  }

  private boolean isLegalTaskPlan (TaskPlan plan)
  {
    return (plan != null);
  }

  /**
   * Return true for all Session participants. In other words, the CFB
   * can be seen by everyone who is registered with a Session.
   */
  /*
  public boolean isVisible (String participantId, int role)
  {
    if (role == Session.CUSTOMER || role == Session.SUPPLIER)
      return true;
    else
      return false;
  }
  */
  public String toString() {
    return new String("{bidDeadline="+getBidDeadline().toString()+
      ", earliestOffer="+getEarliestOffer().toString()+
      ", earliestConsider="+getEarliestConsider().toString()+
      ", taskPlan="+getTaskPlan().toString()+"}");
  }
}
