/*----------------------------------------------------------------------
 * Magnet Protocol Elements
 * $Id: TaskWithPrecedence.java,v 1.2.2.1 2002/05/05 19:41:57 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *----------------------------------------------------------------------
 * Revision History:
 * 
 * Tue May 22 21:55:20 2001  John Collins  #0000123
 *   Add updated serialVersionUID.  From now on, if you add or remove
 *   or change the types of non-transient member fields, you risk
 *   lossage when it comes to loading old plan files.
 * 
 * Sun Sep 24 21:32:27 2000  John Collins
 *   Change reference SubtaskRqClass -> SubtaskRqSchedule
 * 
 * Thu Oct 29 17:07:32 1998  John E. Collins
 *   Add serialization signature.
 * 
 * Mon Sep 28 13:44:20 1998  John E. Collins
 *   Deprecated getTaskID in favor of getTaskId.
 * 
 * Tue Sep 15 19:00:06 1998  John E. Collins
 *   Replace SubtaskRequest to SubtaskRqClass to permit access to time
 *   data.  Change task accessor getID -> getId.
 * 
 * Sat Jun 27 16:33:02 1998  John E. Collins
 *   Move to protocol package.
 */

package edu.umn.magnet.protocol.cust;

import java.io.Serializable;
import java.util.Vector;
import java.util.Stack;
import java.util.Iterator;
import java.util.NoSuchElementException;


  //--------------------------------------------------------------------
  /**
   * We define a local class that encapsulates a task and its precedence
   * relationships.
   */

public class TaskWithPrecedence implements Serializable
{
  // Computed for v1.1:
  static final long serialVersionUID = 8152081220006136763L;

  protected SubtaskRequest m_TaskData;
  protected Vector m_predecessors = new Vector();
  protected Vector m_successors = new Vector();
  
  
  public TaskWithPrecedence () {
    super();
  }
  
  public  TaskWithPrecedence (SubtaskRequest td) {
    super();
    m_TaskData = td;
  }
  
  public TaskWithPrecedence (TaskWithPrecedence twp) {
    // make a new one out of an old one
    super();
    m_predecessors = twp.m_predecessors;
    m_successors = twp.m_successors;
    m_TaskData = new SubtaskRequest((SubtaskRequest)twp.m_TaskData);
  }
  
  public void addPredecessor (TaskWithPrecedence otherTask) {
    if (m_predecessors.indexOf(otherTask) == -1)
      m_predecessors.addElement(otherTask);
  }
  
  public void removePredecessor (TaskWithPrecedence otherTask) {
    m_predecessors.removeElement(otherTask);
  }
  
  public void addSuccessor (TaskWithPrecedence otherTask) {
    if (m_successors.indexOf(otherTask) == -1)
      m_successors.addElement(otherTask);
  }
  
  public void removeSuccessor (TaskWithPrecedence otherTask) {
    m_successors.removeElement(otherTask);
  }
  
  /**
   * Returns the ID of the underlying task
   * @deprecated  Use getTaskId instead.
   */
  public String getTaskID () {
    return m_TaskData.getId();
  }

  /**
   * Returns the ID of the underlying task
   */
  public String getTaskId () {
    return m_TaskData.getId();
  }
  
  public SubtaskRequest getTaskData () {
    return m_TaskData;
  }
  
  public Vector getPredecessors () {
    return m_predecessors;
  }
  
  public Vector getSuccessors () {
    return m_successors;
  }
  
  public boolean isInSuccessors (TaskWithPrecedence otherTask) {
    if (m_successors.indexOf(otherTask) != -1) {
      return true;
    }
    else {
      for (Iterator e = m_successors.iterator();
	   e.hasNext(); ) {
	if (((TaskWithPrecedence)e.next()).
	    isInSuccessors(otherTask)) {
	  return true;
	}
      }
      return false;
    }
  }
}
