/*------------------------------------------------------------------------
 * MAGNET Manager User Interface component
 * $Id: AgentTimeClockDialog.java,v 1.5.2.1 2002/05/05 19:41:59 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 16May2001 PBG: Bug #0000090 Require ability to see agent time from all screens
 * 23May2001 PBG: Only update Agent Clock once per human second
 * 02Jun2001 DFK: Bug #0000134: Park the agent clock in lower LH corner.
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.*;
import edu.umn.magnet.util.time.AgentTime;   // for example date strings


/** This is the main class for the Magnet Agent Time Clock Dialog Box
 *
 *  This will contain the dialog box and the respective Panel information for
 *  displaying the current Agent Time.  Ideally, this would be desired to
 *  just be part of our main frame.  However, the current Gannt Plan
 *  starts viciously flashing as it re-paints itself.  This close to
 *  demo it will go in this dialog for now.  If the GanntPlan gets fixed,
 *  the original code can be uncommented out and this dialog can go away.
 *
 * This is a non-modal dialog, it should be able to allow for the following:
 *  - dispay and update the current agent time
 */
public class AgentTimeClockDialog {

  /** Dialog instance */
  private JDialog _agentTimeDialog = null;



  /** Data used for layout */
  private Frame _parent = null;
  private JPanel _clockPanel = new JPanel();
  private JLabel _agentTimeString = new JLabel();
  /** Timer for updating time string */
  javax.swing.Timer _agentTimeTimer;


  /** X-coord of dialog. (default 0) */
  private int XLocation = 0;
  /** Y-coord of dialog. (default 0) */
  private int YLocation = 0;

  /** Constructor */
  public AgentTimeClockDialog(Frame parent) {
   _parent = parent;
   displayDialog();

  } // end of AgentTimeClockDialog

  /**
    * This displays the Dialog Box
   */
  public void displayDialog() {

    // Now add the agent time display
    /** @todo: should make this really big and bold */
    _agentTimeString.setText(GuiUtil.getAgentTimeString());
    // Need to update this per agent second
    double rate = AgentTime.getRate();
    if (rate == 0) rate=1;
    _agentTimeTimer = new javax.swing.Timer((int) (GuiUtil.ONE_SECOND),
                    new ActionListener() {
                        public void actionPerformed(ActionEvent evt) {
                          _agentTimeString.setText(GuiUtil.getAgentTimeString());
                        }
                      });
    _agentTimeTimer.start();
    _clockPanel.add(_agentTimeString);

    _agentTimeDialog = new JDialog(_parent, "Current Agent Time", false);

    _agentTimeDialog.getContentPane().add(_clockPanel, BorderLayout.CENTER);
   // _agentTimeDialog.setSize(new Dimension(800, 800));
    _agentTimeDialog.setSize(new Dimension(200,50));
    parkLowerLHCorner();
    _agentTimeDialog.setLocation(XLocation, YLocation);
    _agentTimeDialog.show();

  } // end of displayDialog

  /** Position the Time window to the lower left hand corner of
   *  the current display.
   *  @todo Consider generalizing this method, and pass in Enums for
   *          UPPER_RH, LOWER_RH, etc..
   */
  public void parkLowerLHCorner() {
    Dimension screenSize =  Toolkit.getDefaultToolkit().getScreenSize();
    XLocation = 0;  //(int) screenSize.getWidth() - this.THISWIDTH;
    YLocation = (int) screenSize.getHeight() - _agentTimeDialog.getSize().height
                      - 30/*approx height of windoze taskbar*/;
  }


} // end of AgentTimeClockDialog Class
