/*------------------------------------------------------------------------
 * MAGNET Manager User Interface component
 * $Id: BidDetailDialog.java,v 1.1.2.1 2002/05/05 19:41:59 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 19May2001 PBG - Initial Entry
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.*;

import edu.umn.magnet.protocol.cust.Bid;     // for Bid data structure
import edu.umn.magnet.protocol.cust.SubtaskRequest; // for Subtask Structure


/** This is the main class for the Magnet Client Bid Details Display Dialog
 *
 *  This will contain the dialog box and the respective Panel information for
 *  the details of a particular bid that is passed in
 *
 * This is a modal dialog, it should be able to allow for the following:
 *  - display the details associated with a particular bid
 */
public class BidDetailDialog {

  /** Dialog instance */
  private JDialog _thisDialog = null;

  /** Data used for layout */
  private Frame _parent = null;
  private JPanel _bidPanel = null;
  JScrollPane _detailsScroll; // Put text area in scroll
  JTextArea _detailsText = new JTextArea();
  JButton _okButton = new JButton();


  /** X-coord of dialog. (default 0) */
  private int XLocation = 0;
  /** Y-coord of dialog. (default 0) */
  private int YLocation = 0;

  /** Constructor will just create the basic dialog
   */
  public BidDetailDialog(Frame parent, Bid b) {
   _parent = parent;
   displayDialog(b);

  } // end of BidDetailDialog

  /**
    * This displays the Dialog Box
   */
  public void displayDialog(Bid b) {

    // the panel that displays the evaluation information
    JPanel detailsPanel = createDetailsPanel(b);

    // create the rfq dialog
    if (b == null) {
     _thisDialog = new JDialog(_parent, "Information for Null Bid", true);
    } else {
      _thisDialog = new JDialog(_parent, "Information for Bid "+b.getId(), true);
    }
    _thisDialog.getContentPane().setLayout(new BorderLayout());
    _thisDialog.getContentPane().add(detailsPanel, BorderLayout.CENTER);
    _thisDialog.setLocation(XLocation, YLocation);
    _thisDialog.setSize(new Dimension(600, 600));
    _thisDialog.show();

  } // end of displayDialog

  /** The following method sets up the detailed information panel to be used in
   *  this dialog.
   *  The dialog consists of a text box and then perhaps a button
   */
   private JPanel createDetailsPanel(Bid b) {

    JPanel newPanel = new JPanel(new GridBagLayout());
    _detailsText.setOpaque(false);
    _detailsText.setText(createString(b));
    _detailsText.setEditable(false);

    _detailsScroll = new JScrollPane(_detailsText);
    _detailsScroll.setMaximumSize(new Dimension(575, 550));
    _detailsScroll.setMinimumSize(new Dimension(200,100));
    _detailsScroll.setPreferredSize(new Dimension(500, 400));

    _okButton.setText("OK");
    _okButton.setMnemonic('o');
    ActionListener oal = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        _thisDialog.dispose();
      }
      };
    _okButton.addActionListener(oal);

    GuiUtil.constrain(newPanel, _detailsScroll,
                      0,0,  // Grid  x & Y
                      1,1,  // Grid Width & Height
                      GridBagConstraints.NONE,  // fill
                      GridBagConstraints.CENTER,   // anchor
                      0.5, 0.5,   // weight x & y
                      GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(newPanel, _okButton,
                      0,1,  // Grid  x & Y
                      1,1,  // Grid Width & Height
                      GridBagConstraints.NONE,  // fill
                      GridBagConstraints.CENTER,   // anchor
                      0.0, 0.0,   // weight x & y
                      GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,GuiUtil.LAYOUT_MARGIN);

    return newPanel;
  } // end of createDetailsPanel

  /** The following method will create a formatted string of the bid details */
  private String createString(Bid _bid) {
     String ts = new String("");
     java.util.Set taskSet = null;

    if (_bid == null) {
      ts += "There is currently no Bid to display";
      return ts;
    }


     ts+="Bid is for RFQ:\t"+_bid.getRFQId()+"\n";
     ts+="Bid Cost:\t"+_bid.getCost()+"\n";
     /** @todo: Need to put Duration into meaningful format */
     ts+="Duration for Entire Bid:\t"+_bid.getDuration()+"\n";
     ts+="Bid Could be Started as Early As:\t"+_bid.getEarlyStart()+"\n";
     ts+="Bid Must be Started no Later Than:\t"+_bid.getLateStart()+"\n";
     ts+="Bid Could be Finished as Early As:\t"+_bid.getEarlyFinish()+"\n";
     ts+="Bid Would be Finished no Later Than:\t"+_bid.getLateFinish()+"\n";
     ts+="Bid Needs to be Accepted by:\t"+_bid.getLatestAccept()+"\n";
     ts+="\nTasks Included in this Bid:\n\n";

     taskSet = _bid.getTaskSet();
     if (taskSet.isEmpty()) {
      ts+="\tThere are no tasks for this bid.\n";
     } else {
       for (Iterator tsi = taskSet.iterator(); tsi.hasNext();) {
            String taskId = (String) tsi.next();
            ts+="Task ID:\t"+taskId+"\n";
            ts+="\tDuration:\t"+_bid.getDuration(taskId)+"\n";
            ts+="\tEarliest Start Date:\t"+_bid.getEarlyStart(taskId)+"\n";
            ts+="\tLatest Start Date:\t"+_bid.getLateStart(taskId)+"\n";
            ts+="\tEarliest Finish Date:\t"+_bid.getEarlyFinish(taskId)+"\n";
            ts+="\tLatest Finish Date:\t"+_bid.getLateFinish(taskId)+"\n";
            ts+="\tAssociated Milestone Task:\t"+_bid.getMilestone(taskId)+"\n";
            ts+="\n";
      } // end of for loop
    } // end of if-else loop
    return ts;
  } // end of createString

} // end of BidDetailDialog Class