/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: GanttTask.java,v 1.6.2.1 2002/05/05 19:42:00 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * 11Apr2001 DFK - Initial Entry
 * 02May2001 DFK - Bug #0000022: Safely handle null-arg constructor; line up Tasks w/ table rows
 * 07May2001 DFK - Bug #0000022: Make Ganttbars minimum size; line up better vertically.
 * 21May2001 DFK - Bug #0000099: Support for slack tails and multiple line widths.
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;

import edu.umn.magnet.protocol.cust.*;
import org.apache.log4j.Category;

/** A class that represents {@link SubtaskRequest} graphically. */
public class GanttTask extends GenericGanttObject {
  /** The {@link SubtaskRequest} that this obj represents */
  protected SubtaskRequest req = null;
  /** Our link to the parent {@link GanttPanel} instance. */
  protected GanttPanel ganttPanel = null;

  /** log category */
  private static Category log =
                    Category.getInstance(GanttTask.class.getName());
  /** Constructor
   *  @param ganttShape One of the <code>SHAPE_*</code> params from GenericGanttObject
   */
  public GanttTask(SubtaskRequest req, GanttPanel ganttPanel, int ganttShape) {
    super();
    this.req = req;
    if (null==req) { GuiUtil.inform(ganttPanel, "Encountered null SubtaskRequest");  return; }
    this.ganttPanel = ganttPanel;
    if (null==ganttPanel) { GuiUtil.inform(ganttPanel, "Encountered null GanttPanel");  return; }
    log.debug("GanttTask: taskID="+req.getId()+": about to calc x1 (millis="+req.getEarliestTimeToStart().getTime()
                            +") and x2 (millis="+req.getDeadline().getTime() +")");
    int x1=0,x2=0;
    if (0>ganttShape  ||  SHAPE_MAX<=ganttShape)  {  return;  }
    this.middleShape = ganttShape;
    switch (ganttShape) {
      case SHAPE_TAIL_WIDE:
      case SHAPE_TAIL_THIN:
          x1 = ganttPanel.t2x(req.getEarliestTimeToStart().getTime() +
                              (long)req.getJobInfo().getExpectedDuration());
          x2 = ganttPanel.t2x(req.getDeadline());
          this.endShape = ganttShape;
          break;
      case SHAPE_LINE_WIDE:
      case SHAPE_LINE_THIN:
      default:
          x1 = ganttPanel.t2x(req.getEarliestTimeToStart());
          x2 = ganttPanel.t2x(req.getEarliestTimeToStart().getTime() +
                              (long)req.getJobInfo().getExpectedDuration());
          break;
    }//switch
//    x1 = ganttPanel.t2x(req.getEarliestTimeToStart());
//    x2 = ganttPanel.t2x(req.getDeadline());
    log.debug("GanttTask: after t2x: coords x1="+x1+", and x2="+x2);
    if  (x2-x1<3)  x2+=3;  // REMOVE THIS ; it's just here to ensure SOME width....
    int y = 50;
    int h = 16;
    // Override y and height if they're available (from table model).
    if (null!=ganttPanel) {
      TPTableModel tm = ganttPanel.getTPTableModel();
      String taskID = req.getId();
      Rectangle r = tm.getRowRect(taskID);
      if (null!=r) {
        y = r.y;
          y+=28;  // REMOVE THIS
        h = r.height;
          if (0==h) h=16;  // REMOVE THIS
        log.debug("GanttTask: Task="+taskID
            + ", earliestDate="+DateFormat.getDateTimeInstance().format(req.getEarliestTimeToStart())
            + ", latestDate="+DateFormat.getDateTimeInstance().format(req.getDeadline())
            + ", x="+x1+",y="+y+",w="+(x2-x1)+",h="+h);
      }
    }
    this.boundingBox = new Rectangle(x1, y, (x2-x1), h);
    // for now, Just use the shape/color defaults..
  }
}
