/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: MagnetClientMain.java,v 1.50.2.3 2002/05/05 19:42:01 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Tue Feb 5 15:20:25 2002 Anne Schoolcraft
 *  Added call to constructors that creates a BidCollector.  This way if a 
 *   supplier window is created by the BidCollector, it will be available right
 *   away.
 *
 * Mon Feb 4 15:49:25 2002 Anne Schoolcraft
 *  Removed all references to GuiFrame and GuiDialog.  Bid Collectors are
 *  in charge of creating supplier windows if they see fit.
 *
 * Mon Jan 28 16:16:42 2002 Anne Schoolcraft
 *  Bug #185 Removed GuiFrame references and call a GuiDialog instead.  
 *   Now the upplier dialog box is more accessible.
 *
 * Wed Jan  9 12:55:03 2002  John Collins
 *   Fix deprecation warnings, make Supplier control window pop up
 *   initially.
 * 
 * Wed Jan  2 17:28:30 2002  John Collins #0000163
 *   Remove dependence on Borland library
 *
 * Sun Dec 16 09:00:28 2001  John Collins  #0000176
 *   Update packages for MarketListingServiceClient and JobServiceClient
 *
 * 15Apr2001 DFK - Initial Entry
 * 17Apr2001 DFK - Playing around w/ splitpane sizing/weights.
 * 20Apr2001 DFK - Bug #0000024: Make main() read in JobInfo data from props files.
 * 23Apr2001 DFK - Bug #0000026: Instantiate JobInfoFactory singleton;
 *                  Dummy data: add task Name/comments, Plan Title
 * 23Apr2001 DFK - Switched from system properties to MagnetProperties
 * 25Apr2001 PBG - Hooked in Bid Editor panel
 * 26Apr2001 DFK - Hooked up dummy handler to acknowledge Zoom button presses.
 * 30Apr2001 DFK - Make frame size a bit shorter
 * 02May2001 DFK - Make TP Table's Column Header height match Gantt's Timeframe header;
 *                  Ganged the vert scrollbars for TPTable and GanttPanel.
 * 02May2001 DFK - Bug #0000022: Hooked in the gantt panel zoom buttons.
 * 05May2001 PBG - Hooked in the Status objects for the bid
 * 07May2001 DFK - Bug #0000022: Chose more GanttPanel-timeframe-testing task dates.
 * 07May2001 PBG - Moved the get statuses to GuiUtils
 * 13May2001 PBG - Added A BidManager component
 * 13May2001 DFK - Bug #0000053: TPTable now uses highlighting row renderer for bidded tasks
 * 16May2001 PBG - Bug #0000050: Added a file menu
 * 16May2001 PBG - Bug #0000090: Added Agent Time Display to GUI
 * 19May2001 PBG - Bug #0000091, #0000101: Keep flow similar to that of agent
 * 20May2001 PBG - Bug #0000050, #0000098: Allowed save and load of plan
 * 23May2001 PBG - Bug #0000119: Fix so save dialogs say right thing
 * 25May2001 DFK - Bug #0000100: Added menu item to create a new random plan
 * 26May2001 DFK - Bug #0000127: Fix redraw problem.
 * 02Jun2001 DFK - Changed Gantt background to same gray color as table bkgd.
 * 03Jul2001 LRJ - Added ability to use SOAP clients for getting markets/jobInfo.
 * 04Aug2001 LRJ - Fixed Magnet port to come from property, not just "8080".
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.io.*;                    // for File stuff

import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.Category;
import org.apache.log4j.FileAppender;
import org.apache.log4j.PatternLayout;

import edu.umn.magnet.client.bid.BidCollectorFactory;  //for dummy BidCollector 
import edu.umn.magnet.client.status.*;  //for status objects
//import edu.umn.magnet.client.xac.MarketListingServiceClient;  // SOAP client
//import edu.umn.magnet.client.xac.JobServiceClient; // SOAP client
import edu.umn.magnet.client.bid.BidManager; // for BidManager
import edu.umn.magnet.client.plan.JobInfoFactory;  // for testing
import edu.umn.magnet.client.plan.RandomPlanFactory;
import edu.umn.magnet.client.plan.PlanFactory;
import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.protocol.InvalidDataException;
import edu.umn.magnet.protocol.JobInfo;
import edu.umn.magnet.util.time.AgentTime;   // for example date strings

import edu.umn.magnet.util.MagnetProperties;
import edu.umn.magnet.util.Copyright;

/** This is the main class for the Magnet Client gui.
 *  This app will let you create/edit/save TaskPlans of SubtaskRequests,
 *  compose RFQs, and view/award Bids.
 *
 * Gui Details:
 * The frame is split into panes as follows:
 *   1. Horizontal split, separating Views (table/gantt) from editor(s).
 *   2. Vertical split, separating navBar from Views (table/gantt).
 *   3. Vertical split, separating table from gantt.
 */
public class MagnetClientMain extends JFrame
  implements ActionListener, MagnetClientConstants
{
  /** log category */
  private static Category log =
    Category.getInstance(MagnetClientMain.class.getName());

  /** Horizontal split, separating table/gantt from editor(s). */
  JSplitPane _splitViewsEditors;// = new JSplitPane();
  /** Vertical split, separating navBar from table/gantt. */
  JSplitPane _splitNavViews;// = new JSplitPane();
  /** Vertical split, separating table from gantt. */
  JSplitPane _splitTableGantt;// = new JSplitPane();
  /** Scrollpane for table */
  JScrollPane _spTable;
  /** Scrollpane for gantt */
  JScrollPane _spGantt;
  /** The nav bar panel (to switch betw Plan/RFQ and Bid). */
  NavBarPanel _navPanel;
  /** Panel that holds the Views and the zoom buttons. */
  JPanel _viewsPanel;
  /** Button to zoom in on gantt chart. */
  JButton _btnZoomIn;
  /** Button to zoom out on gantt chart. */
  JButton _btnZoomOut;
  //  /** Label for Agent Time */
  //  JLabel _agentTimeLabel;
  //  /** Actual Agent Time */
  //  JLabel _agentTimeString;
  //  /** Timer for updating time string */
  //  javax.swing.Timer _agentTimeTimer;
  /** clock */
  static AgentTimeClockDialog _mainAgentClock;

  /** File Chooser for doing save and get */
  protected JFileChooser _fileChooser;
  /** File currently being worked with */
  File _currentFile = null;
  /** If data needs saving */
  boolean _needSave = false;

  /** Our handle to a {@link TaskPlan}. */
  TaskPlan _tp = null;
  /** Our handle to a TaskPlan table model */
  TPTableModel _tpTableModel = null;
  /** Our handle to a TaskPlan gantt model */
  TPGanttModel _tpGanttModel = null;
  /** Our table display */
  private JTable _table = null;
  /** Our gantt display */
  private GanttPanel _ganttPanel = null;
  /** Our TaskPlan editor panel */
  TaskPlanEditor4Panel _tpEditPanel;
  /** Our TaskBidPlan editor panel.. */
  BidEditorPanel _tbpEditPanel;

  /** Our handle to the Agent Status object */
  AgentStatus _agentStatus = null;

  //  /** @todo REMOVE:  our handle to an RFQ object */
  //  RFQ _rfq = null;
  //  /** @todo REMOVE:  our handle to a Bid object. */
  //  Bid _bid = null;
  public static BidManager mgr = null;

  /** Enum to indicate the TaskPlan editor panel */
  public static final int EDITOR_TASKPLAN = 1;
  /** Enum to indicate Bid editor panel */
  public static final int EDITOR_TASKBIDPLAN = 2;

  /** Constructor */
  public MagnetClientMain()
  {
    _agentStatus = AgentStatus.getInstance();
    if (_agentStatus == null) log.info("Agent is null");

    /* create the menu bar */
    JMenuBar menuBar = createMenuBar();
    setJMenuBar(menuBar);
    _fileChooser = new JFileChooser();
    _fileChooser.setCurrentDirectory(new File("."));
    //start up a BidCollector to get a supplier window if needed
    BidCollectorFactory bcf=new BidCollectorFactory();
    bcf.getInstance(); //this will pop up the supplier window if needed
    doTPSetup();
  }

  /** Constructor w/ titled frame */
  public MagnetClientMain(String title)
  {
    super(title);
    _agentStatus = AgentStatus.getInstance();
    if (_agentStatus == null) log.info("Agent is null");
    /* create the menu bar */
    JMenuBar menuBar = createMenuBar();
    setJMenuBar(menuBar);
    _fileChooser = new JFileChooser();
    _fileChooser.setCurrentDirectory(new File("."));
    
    //start up a BidCollector to get a supplier window if needed
    BidCollectorFactory bcf=new BidCollectorFactory();
    bcf.getInstance(); //this will pop up the supplier window if needed
    doTPSetup();
  }

  public void doTPSetup (TaskPlan tp)
  {
    _tp = tp;
    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void doTPSetup ()
  {
    doTPSetup(new TaskPlan());
  }

  /** Does the initial TaskPlan setup stuff */
  //public void doTPSetup(TaskPlan tp)
  public void jbInit()
  {
    // Load the data/models
    //    loadTaskPlan();
    _tpTableModel = new TPTableModel(_tp);
    _tpGanttModel = new TPGanttModel(_tpTableModel);

    _table = new JTable();
    setTableModels(_table, _tpTableModel);
    HighlightedTableCellRenderer r = new HighlightedTableCellRenderer();
    _table.setDefaultRenderer(String.class, r); // dfk 09May2001

    //      /** @todo dfk09May2001: BEGIN Remove this eventually!!!! */
    //      _tpTableModel.highlightRow(0, H_TYPE0);
    //      // Set up color 1, and use it.
    //      log.debug("MCM.doSetup(): assigning colors for highlight 1");
    //      r.setColor(r.C_HIGHLIGHT1 | r.C_FOREGROUND | r.C_SELECTED, Color.yellow);
    //      r.setColor(r.C_HIGHLIGHT1 | r.C_BACKGROUND | r.C_SELECTED, Color.green);
    //      r.setColor(r.C_HIGHLIGHT1 | r.C_FOREGROUND | r.C_NOT_SELECTED, Color.black);
    //      r.setColor(r.C_HIGHLIGHT1 | r.C_BACKGROUND | r.C_NOT_SELECTED, Color.lightGray);
    //      _tpTableModel.highlightRow(1, H_TYPE1);
    //      /** @todo dfk09May2001; END Remove eventually... */

    _ganttPanel = new GanttPanel(_tpGanttModel);

    // Create/add views
    _splitViewsEditors = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    _splitViewsEditors.setDividerLocation(0.5d);  // 50%
    _splitViewsEditors.setResizeWeight(0.5d);  // 50%
    _splitNavViews = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    _splitNavViews.setDividerLocation(0.2d);  // 20%
    _splitNavViews.setResizeWeight(0.2d);
    _splitNavViews.setMinimumSize(new Dimension(75,75));
    _splitTableGantt = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    _splitTableGantt.setDividerLocation(0.4d);  // 60%
    _splitTableGantt.setResizeWeight(0.4d);  // 60%

    _splitTableGantt.setLeftComponent(_spTable = new JScrollPane(_table));

    // Adjust table scrollpane's COLUMN_HEADER to be taller..
    setTableHeaderHeight(MagnetClientConstants.TP_TABLE_COLUMN_HEADER_HEIGHT);

    _splitTableGantt.setRightComponent(_spGantt = new JScrollPane(_ganttPanel));
    _spGantt.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    _spGantt.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    gangScrollTheseVertically(_spTable, _spGantt);
    //_spGantt.getViewport().setBackground(Color.black);
    _spGantt.getViewport().setBackground(new Color(214,211,206));
    // Put the table/gantt splitPane in its own panel, so I can
    //  also insert some zoom buttons.
    _viewsPanel = new JPanel(new BorderLayout());
    _viewsPanel.add(_splitTableGantt, BorderLayout.CENTER);
    JPanel zoomButtonsPanel = new JPanel();
    zoomButtonsPanel.setLayout(new BoxLayout(zoomButtonsPanel,
					     BoxLayout.Y_AXIS));
    _btnZoomIn = new JButton("zoom +");
    _btnZoomIn.setAlignmentX((float)0.5);
    zoomButtonsPanel.add(_btnZoomIn);
    _btnZoomOut= new JButton("zoom -");
    _btnZoomOut.setAlignmentX((float)0.5);
    zoomButtonsPanel.add(_btnZoomOut);
    _btnZoomIn.addActionListener(this);
    _btnZoomOut.addActionListener(this);
    _viewsPanel.add(zoomButtonsPanel, BorderLayout.EAST);

    _navPanel = new NavBarPanel();
    // DAVE: I know this is bad style, but rather than create a NavBarListener
    //  interface and keep track of registered listeners, just attach the main
    //  panel as a listener, manually.
    _navPanel.attachListener(this);
    _splitNavViews.setLeftComponent(_navPanel);
    _splitNavViews.setRightComponent(_viewsPanel);

    _splitViewsEditors.setTopComponent(_splitNavViews);
    //    _tpEditPanel = new TaskPlanEditor4Panel();
    //    _splitViewsEditors.setBottomComponent(_tpEditPanel);
    invokeEditor(EDITOR_TASKPLAN);

    this.getContentPane().removeAll();

    this.getContentPane().add(_splitViewsEditors);
    _tpTableModel.firePlanDataChanged("New Plan");
  }

  /** Hacky L1 test to load RFQ and bid data. */
  private void doBidSetup()
  {
    // First, construct RFQ from the tasks plus some dates.
  }

  /** Grabbed from XAutoCustomer;  generate a random plan.
   *  @todo Hook this to a button somewhere, then call Paula's initting stuff.
   */
  private void doGenRandomPlan(int numSubtasks)
  {
    //Create the plan
    TaskPlan plan = RandomPlanFactory.makePlan(numSubtasks, true);
    Iterator roots = plan.getSuccessorRoots();
    PlanFactory.printSuccessorTree(log, plan, roots, 0);
    // Hook the new taskplan into the gui
    _tp = plan;
    setNewTaskPlan(plan);
    // Fire an update
    _tp.firePlanDataChanged("Attached new random TaskPlan with "+numSubtasks+" tasks.");
  }


  /** Listens to button-press actions. */
  public void actionPerformed(ActionEvent e)
  {
    log.debug("MCM.actionPerformed(): entering.");
    if (e.getSource()==_btnZoomIn) {
      log.debug("\t'Zoom in' button pressed.");
      _ganttPanel.zoomIn();
    } else if (e.getSource()==_btnZoomOut) {
      log.debug("\t'Zoom out' button pressed.");
      _ganttPanel.zoomOut();
    } else {
      log.info("MCM.actionPerformed(): Unknown source "+e.getSource());
    }
  }

  /** Sets the table header size to match gantt panel */
  public void setTableHeaderHeight(int height)
  {
    _spTable.addNotify();  // explicitly shove the table header into
                           // the scrollpane column header.. 
    Dimension colHdrSize = _spTable.getColumnHeader().getPreferredSize();
    _spTable.getColumnHeader().setPreferredSize
	(new Dimension(colHdrSize.width, height));
    _spTable.getColumnHeader().setViewSize
	(new Dimension(colHdrSize.width, height));
  }


  /**
   * Gang-scrolls these two {@link JScrollPane} instances' vertical
   * scrollbars.
   */ 
  public void gangScrollTheseVertically(JScrollPane sp1,
					JScrollPane sp2)
  {
    final JScrollBar jsbVert1 = sp1.getVerticalScrollBar();
    final JScrollBar jsbVert2 = sp2.getVerticalScrollBar();
    AdjustmentListener adjListen = new AdjustmentListener() {
	public void adjustmentValueChanged(AdjustmentEvent e) {
	  int value = e.getValue();
	  jsbVert1.setValue(value);
	  jsbVert2.setValue(value);
	}
      };
    jsbVert1.addAdjustmentListener(adjListen);
    jsbVert2.addAdjustmentListener(adjListen);
  }


  /** Hacky L1 test to load taskplan data.
   *  @todo Eventually load plans from ...  whereever..
   */
  private void loadTaskPlan()
  {
    SubtaskRequest subtask1, subtask2, subtask3, subtask4;
    JobInfo jobInfo;
    JobInfoFactory jif = (JobInfoFactory)JobInfoFactory.getInstance();

    try {
      subtask1 = new SubtaskRequest("TaskID1");
      subtask1.setTaskName("Task Number One");
      subtask1.setEarliestTimeToStart(DateFormat.getDateTimeInstance().
				      parse("June 25, 2001 2:00:00 PM CST"));
      subtask1.setDeadline(DateFormat.getDateTimeInstance().
			   parse("June 25, 2001 3:01:00 PM CST"));
      subtask1.setJobInfo(jif.getJobInfo("Task Type 1"));
      subtask1.setTaskComments("Comments for Task Number One");

      subtask2 = new SubtaskRequest("TaskID2");
      subtask2.setTaskName("Task Number Two");
      subtask2.setEarliestTimeToStart(DateFormat.getDateTimeInstance().
				      parse("June 25, 2001 3:00:00 PM CST"));
      subtask2.setDeadline(DateFormat.getDateTimeInstance().
			   parse("June 25, 2001 5:01:00 PM CST"));
      subtask2.setJobInfo(jif.getJobInfo("Task Type 2"));
      subtask2.setTaskComments("Comments for Task Number Two");

      subtask3 = new SubtaskRequest("TaskID3");
      subtask3.setTaskName("Task Number Three");
      subtask3.setEarliestTimeToStart(DateFormat.getDateTimeInstance().
				      parse("June 25, 2001 6:00:00 PM CST"));
      subtask3.setDeadline(DateFormat.getDateTimeInstance().
			   parse("June 25, 2001 7:01:00 PM CST"));
      subtask3.setJobInfo(jif.getJobInfo("Task Type 3"));
      subtask3.setTaskComments("Comments for Task Number Three");

      subtask4 = new SubtaskRequest("TaskID4");
      subtask4.setTaskName("Task Number Four");
      subtask4.setEarliestTimeToStart(DateFormat.getDateTimeInstance().
				      parse("June 25, 2001 8:00:00 PM CST"));
      subtask4.setDeadline(DateFormat.getDateTimeInstance().
			   parse("June 25, 2001 9:01:00 PM CST"));
      subtask4.setJobInfo(jif.getJobInfo("Task Type 1"));
      subtask4.setTaskComments("Comments for Task Number Four");
      //AgentTime.setDate((new Date()).getTime(), 0, 2.0);

      _tp = new TaskPlan();
      _tp.addTask(subtask1);
      _tp.addTask(subtask2);
      _tp.addTask(subtask3);
      _tp.addTask(subtask4);
      //      _tp.addSuccessor(subtask2, subtask4);
      //      _tp.addSuccessor(subtask3, subtask4);
      _tp.setPlanName("A Sample Task Plan");
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /** Attachs the {@link TPTableModel} to this table, and
   * creates/attaches a new {@link TableColumnModel} that has the
   * correct columnIdentifiers (titles).
   */
  public void setTableModels(JTable table, TPTableModel tm)
  {
    if (null==table  ||  null==tm) {  return;  }
    //    table.setAutoCreateColumnsFromModel(false);
    table.setModel(tm);
    tm.setView(table);

    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    table.createDefaultColumnsFromModel();
    TableColumnModel tcm = table.getColumnModel();
    for (int i=0; i < tcm.getColumnCount(); i++) {
      TableColumn tc = tcm.getColumn(i);
      tc.setHeaderRenderer(new JComponentCellRenderer());
      String title = tm.getColumnName(i);
      //      log.debug("MCM.setTableModels(): col "+i+" title: " + title);
      tc.setIdentifier(title);
      tc.setHeaderValue(title);
      /** While we're at it, force (initial) col width of 120px. */
      int newWidth = 130;
      tc.setWidth(newWidth);
      tc.setMinWidth(newWidth);
      tc.setMaxWidth(newWidth);
      tc.setPreferredWidth(newWidth);
    }
    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    tm.fireTableStructureChanged();
  }

  /** Switches the bottom splitpane to the requested editor.
   *  This will probably be used as a callback, called from the NavBarPanel..
   *  @param editor Either {@link #EDITOR_TASKPLAN} or
   * {@link #EDITOR_TASKBIDPLAN}.
   */
  public void invokeEditor(int editor)
  {
    log.debug("MCM.invokeEditor: invoking editor "+editor);
    switch (editor) {
    case EDITOR_TASKPLAN:
      _tpEditPanel = new TaskPlanEditor4Panel(_tpTableModel,
					      MagnetClientMain.this);
      //          _tpEditPanel.setTPTableModel(_tpTableModel);
      _splitViewsEditors.setBottomComponent(_tpEditPanel);
      break;
    case EDITOR_TASKBIDPLAN:
      _tbpEditPanel
	  = new BidEditorPanel(GuiUtil.getRFQStatus(_agentStatus, _tp),
			       _tpTableModel,
			       _tpGanttModel);
      _splitViewsEditors.setBottomComponent(_tbpEditPanel);
      break;
    }//switch
  }

  /** Reads in the JobInfo properties stuff for testing's sake */
  public static void readProps(String[] args)
  {
    if (args.length<1) {
      System.out.println("Usage: MagnetClientMain <property file>");
      log.error("Invalid arguments passed to the main");
      System.exit(0);
    }
    // init properties
    MagnetProperties.readProps(args[0]);
  }

  /** Saves the current plan that is loaded out to the file
   *  that is passed in
   */
  void savePlanFile(File f)
  {
    try {
      ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(f));
      out.writeObject( _tp);
      _currentFile = f;
      _needSave = false;
      out.close();
    }
    catch (IOException ex) {
      GuiUtil.inform(MagnetClientMain.this, ex.toString(),
		     "Error saving file" + f);
      ex.printStackTrace();

    } // end of catch
    return;
  } // end of savePlanFile

  /** Reads in a plan from a file.
   *  Will then read this plan into memory.  And will
   *  repaint itself to refresh with new data
   *
   * This does not prompt to save the current
   */
  void loadPlanFile(File f)
  {
    // first clear out the old values
    _tp = null;
    //_tpEditPanel.clearAllTaskInputFields();

    _currentFile = f;
    _needSave = false;
    TaskPlan tp = null;
    try {
      ObjectInputStream in = new ObjectInputStream(new FileInputStream(f));
      tp = (TaskPlan) in.readObject();
      in.close();
    }
    catch (IOException ex) {
      GuiUtil.inform(MagnetClientMain.this, ex.getMessage(),
		     "Error reading file" + f);
    }
    catch (ClassNotFoundException cnfe) {
      GuiUtil.inform(MagnetClientMain.this, cnfe.getMessage(),
		     "Class Not Found Exception");
    }// end of catch
    setNewTaskPlan(tp);
    return;
  }

  /** Frees all data about a particular plan
   *  in order to do a new plan - this is called when "new" is selected
   */
  void unloadPlan()
  {
    /** Now clear out data */
    _tp = null;
    TaskPlan tp = new TaskPlan(); // This will create an empty one
    setNewTaskPlan(tp);
    _needSave = false;
    _currentFile = null;

    return;
  }

  /** Re-inits the GUI with a new plan */
  void setNewTaskPlan(TaskPlan tp)
  {
    tp.initListenerList();

    doTPSetup(tp);
    //    tp.firePlanDataChanged("Set new task plan");
    this.invalidate();
    this.validateTree();
    this.repaint();
    return;
  } // end of setNewTaskPlan

  /** Prompts the user if they want to save the
   *  current plan out to a file - in addition, this will cancel the plan
   *  status if one exists
   */
  void promptFileSave()
  {
    if (!_needSave) return; // no need to prompt

    // First cancel the status
    PlanStatus ps = GuiUtil.getPlanStatus(_agentStatus, _tp);
    if (ps != null) ps.cancel();
    mgr = null;
    _fileChooser.setDialogTitle("Current Plan is Modified.  Do you wish to save your changes?");
    if (_fileChooser.showSaveDialog(MagnetClientMain.this) !=
	JFileChooser.APPROVE_OPTION) return;  //user doesn't want to save

    Thread runner = new Thread() {
        public void run() {
          File fChosen = _fileChooser.getSelectedFile();
          savePlanFile(fChosen);
        } // end of run
      }; // end of new Thread
    runner.start();
    return;
  } // end of promptFileSave

  /** Enables all of the buttons that can do plan editing.
   *  Plan editing is disabled once a bid is submitted as an RFQ
   */
  protected void enablePlanEdit()
  {
    _tpEditPanel.setEditable(true);

  } // end of enablePlanEdit


  /** Allows the BidManager value to be set
   *  Right now it is static - perhaps it shouldn't be
   */
  public static void setBidManager(BidManager bm)
  {
    mgr = bm;
  }

  /** Allows the BidManager value to be gotten
   *  I would just pass it - but since setting it is static - I thought
   *  that getting it should be too
   */
  public static BidManager getBidManager()
  {
    return mgr;
  }

  /** Sets _needSave to true */
  public void setNeedSave()
  {
    _needSave = true;
  }


  /** Creates the Menu Bar for the GUI */
  protected JMenuBar createMenuBar()
  {
    JMenuItem item=null;
    final JMenuBar menuBar = new JMenuBar();

    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic('f');

    /** Create a new plan */
    item = new JMenuItem("New Plan");
    item.setMnemonic('n');
    ActionListener nal = new ActionListener() {
	public void actionPerformed(ActionEvent a) {
	  promptFileSave();
	  unloadPlan();
	}
      };
    item.addActionListener(nal);
    fileMenu.add(item);

    /** Create a new Random plan */
    item = new JMenuItem("New Random Plan...");
    item.setMnemonic('r');
    ActionListener ral = new ActionListener() {
	public void actionPerformed(ActionEvent a) {
	  promptFileSave();
	  _needSave = false;
	  _currentFile = null;
	  _tp=null;
	  handleNewRandomPlan();
	}
      };
    item.addActionListener(ral);
    fileMenu.add(item);

    /** Open an existing plan */
    item = new JMenuItem("Open Plan");
    item.setMnemonic('o');
    ActionListener oal = new ActionListener() {
	public void actionPerformed(ActionEvent a) {
	  // first prompt to save
	  promptFileSave();
	  _fileChooser.setDialogTitle("Select New Plan to Open");
	  if (_fileChooser.showOpenDialog(MagnetClientMain.this) !=
	      JFileChooser.APPROVE_OPTION) return;
	  Thread runner = new Thread() {
	      public void run() {
		File fChosen = _fileChooser.getSelectedFile();
		loadPlanFile(fChosen);
	      }
	    };
	  runner.start();
	} // end of actioPerformed
      }; // end of ActionListener
    item.addActionListener(oal);
    fileMenu.add(item);

    /** Save an existing plan */
    item = new JMenuItem("Save Plan");
    item.setMnemonic('s');
    ActionListener sal = new ActionListener() {
	public void actionPerformed(ActionEvent a) {
	  _fileChooser.setDialogTitle("Select File to Save Plan into");
	  if (_fileChooser.showSaveDialog(MagnetClientMain.this) !=
	      JFileChooser.APPROVE_OPTION) return;
	  Thread runner = new Thread() {
	      public void run() {
		File fChosen = _fileChooser.getSelectedFile();
		savePlanFile(fChosen);
	      } // end of run
	    };
	  runner.start();
	} // end of actioPerformed
      }; // end of ActionListener
    item.addActionListener(sal);
    fileMenu.add(item);

    /** add the seperator bar */
    fileMenu.addSeparator();

    /** Exit from the MagnetClientMain */
    item = new JMenuItem("Exit");
    item.setMnemonic('x');
    ActionListener xal = new ActionListener() {
	public void actionPerformed(ActionEvent a) {
	  promptFileSave();
	  System.exit(0);
	} // end of actionPerformed
      }; //end of ActionListener
    item.addActionListener(xal);
    fileMenu.add(item);
    menuBar.add(fileMenu);

    /** Now add the help option */
    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic('h');

    /** Show the version information */
    item = new JMenuItem("About");
    item.setMnemonic('a');
    ActionListener aal = new ActionListener() {
	public void actionPerformed(ActionEvent a) {
	  /** @todo: Get from John what should be shown */
	  GuiUtil.inform(MagnetClientMain.this,
			 "Magnet User Client for Demo. University of Minnesota. 2001.",
			 "About Magnet Client");
	} // end of actionPerformed
      }; //end of ActionListener
    item.addActionListener(aal);
    helpMenu.add(item);
    menuBar.add(helpMenu);

    return menuBar;

  } // end of createMenuBar

  static int prevRandCount = 6;
  /** Prompts user for number of random tasks in plan, then
   * create/load this plan.
   */ 
  private void handleNewRandomPlan()
  {
    JPanel promptPanel = new JPanel();
    JLabel lbl = new JLabel("How many random tasks should your plan contain? ");
    JTextField tf = new JTextField(""+prevRandCount);
    promptPanel.add(lbl);
    promptPanel.add(tf);
    int retval
	= JOptionPane.showConfirmDialog(this,
					promptPanel,
					"Enter number of random tasks..",
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE);
    if (retval!=JOptionPane.OK_OPTION)  {  return;  }
    try {
      prevRandCount = Integer.parseInt(tf.getText());
    } catch (NumberFormatException nfe) {
      GuiUtil.inform(this, "Error: number should be a nonnegative integer.", "Input Error");
      return;
    }
    doGenRandomPlan(prevRandCount);
    //    this.invalidate();
    //    this.validateTree();
    //    this.repaint();
  }

  /** main method -- launches the gui */
  public static void main(String[] args)
  {
    Copyright.print();
    readProps(args);
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) {
      e.printStackTrace();
    }

    // Init the log4j logging system.  Default log file names come from
    // client.lcf, but are overridden by the experimentId property if
    // it exists.  Note that the filename used by the older tools.Log
    // system may be in conflict, so older property files may need to
    // be changed.
    //   here's the default log setup:
    PropertyConfigurator.configure
	(MagnetProperties.getProperty("LogConfiguration", "client.lcf"));

    // Set the JobInfoFactory instance, then process the TaskInfo data
    JobInfoFactory.instantiate();
    JobInfoFactory JIF = (JobInfoFactory)JobInfoFactory.getInstance();
    // If there is a taskType spec in the properties, use it.  Otherwise,
    // get the JobInfo from the server.
    String taskTypes = MagnetProperties.getProperty("TaskTypes");
    if (taskTypes != null) {
      JIF.createFromSpec(taskTypes);
      log.info("TaskTypes = " + taskTypes);
      }
      else {
        log.error("Properties file does not contain enough information to "+
		  "get JobInfo from the server.  Exiting.");
        System.exit(-1);
    } // Else, get JobInfo from the server.

    edu.umn.magnet.util.silk.SilkHook.init();

    // Find the AgentStatus instance and extract the Agent ID
    AgentStatus as = AgentStatus.getInstance();
    String agentId = as.getAgentId();

    MagnetClientMain f = new MagnetClientMain(agentId);
    f.setSize(900,600);
    //f.setLocation(0,0);


    WindowListener wndCloser = new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      };
    f.addWindowListener(wndCloser);
    f.setVisible(true);
    // Display the clock
    _mainAgentClock = new AgentTimeClockDialog(f);
    f.show();
  }
}
