/*------------------------------------------------------------------------
 * MAGNET Manager User Interface component
 * $Id: RFQDialog.java,v 1.4.2.1 2002/05/05 19:42:02 ashutosh Exp $
 * Copyright (c) 2001 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Wed Jan  2 22:25:05 2002  John Collins  #0000163
 *   Eliminate dependence on Borland library
 *
 * 19May2001 PBG - Bug #0000094, #0000102 Initial Entry
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.*;

import edu.umn.magnet.protocol.cust.*; // for TaskPlan
import edu.umn.magnet.client.status.*; // for rfq, status objects


/** This is the main class for the Magnet ClientRFQ Display Dialog
 *
 *  This will contain the dialog box and the respective Panel information for
 *  the RFQ Submission Results Display
 *
 * This is a modal dialog, it should be able to allow for the following:
 *  - display the results of the RFQ that was submitted
 */
public class RFQDialog
{
  /** Dialog instance */
  private JDialog _thisDialog = null;

  /** Data used for layout */
  private Frame _parent = null;
  private JPanel _rfqPanel = null;
  JScrollPane _resultsScroll; // Put text area in scroll
  JTextArea _resultsText = new JTextArea();
  JButton _okButton = new JButton();

  /** X-coord of dialog. (default 0) */
  private int XLocation = 0;
  /** Y-coord of dialog. (default 0) */
  private int YLocation = 0;

  /** Constructor will just create the basic dialog
   */
  public RFQDialog(Frame parent, RFQStatus rs)
  {
   _parent = parent;
   displayDialog(rs);

  } // end of RFQDialog

  /**
    * This displays the Dialog Box
   */
  public void displayDialog(RFQStatus rs)
  {
    // the panel that displays the evaluation information
    JPanel resultsPanel = createResultsPanel(rs);

    // create the rfq dialog
    _thisDialog = new JDialog(_parent, "Submitted RFQ", false);
    _thisDialog.getContentPane().setLayout(new BorderLayout());
    _thisDialog.getContentPane().add(resultsPanel, BorderLayout.CENTER);
    _thisDialog.setLocation(XLocation, YLocation);
    _thisDialog.setSize(new Dimension(450, 300));
    _thisDialog.show();

  } // end of displayDialog

  /** The following method sets up the results panel to be used in this
   *  dialog.
   *  The dialog consists of a text box and then perhaps a button
   */
   private JPanel createResultsPanel(RFQStatus rs)
  {
    JPanel newPanel = new JPanel(new GridBagLayout());
    _resultsText.setOpaque(false);
    _resultsText.setText(createString(rs));
    _resultsText.setEditable(false);

    _resultsScroll = new JScrollPane(_resultsText);
    _resultsScroll.setMaximumSize(new Dimension(400, 275));
    _resultsScroll.setMinimumSize(new Dimension(200,100));
    _resultsScroll.setPreferredSize(new Dimension(400, 200));

    _okButton.setText("OK");
    _okButton.setMnemonic('o');
    ActionListener oal = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        _thisDialog.dispose();
      }
      };
    _okButton.addActionListener(oal);

    GuiUtil.constrain(newPanel, _resultsScroll,
                      0,0,  // Grid  x & Y
                      1,1,  // Grid Width & Height
                      GridBagConstraints.NONE,  // fill
                      GridBagConstraints.CENTER,   // anchor
                      0.5, 0.5,   // weight x & y
                      GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(newPanel, _okButton,
                      0,1,  // Grid  x & Y
                      1,1,  // Grid Width & Height
                      GridBagConstraints.NONE,  // fill
                      GridBagConstraints.CENTER,   // anchor
                      0.0, 0.0,   // weight x & y
                      GuiUtil.LAYOUT_MARGIN, GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,GuiUtil.LAYOUT_MARGIN);

    return newPanel;
  } // end of createResultsPanel

  /** The following method will create a formatted string of the results */
  private String createString(RFQStatus rs)
  {
    RFQ rfq = null;
    TaskPlan tp = null;
    StringBuffer str = new StringBuffer(1000);

    if ((rs == null) || (rs.getRFQ() == null)) {
      str.append("The RFQ was unable to be created");
      return str.toString();
    }

    rfq = rs.getRFQ();

    str.append("Submitted RFQ Details:\n");
    str.append("RFQ ID:\t").append(rfq.getId()).append("\n");
    str.append("Current RFQ State:\t").append(rs.getStateName()).append("\n");
    str.append("Deadline for Bids to arrive:\t")
	.append(rfq.getBidDeadline().toString()).append("\n");
    str.append("Earliest Bid Consider:\t")
	.append(rfq.getEarliestConsider().toString()).append("\n");
    str.append("Earliest Bid Offer:\t")
	.append(rfq.getEarliestOffer().toString()).append("\n");

    tp = rfq.getTaskPlan();
    if (tp == null) {
      str.append("No Plan associated with RFQ.\n");
      return str.toString();
    }
    str.append("Plan Start Date:\t")
	.append(tp.getPlanStartDate().toString()).append("\n");
    str.append("Plan Deadline:\t")
	.append(tp.getPlanDeadline().toString()).append("\n");
    str.append("RFQ Contains the Plan:\t")
	.append(tp.getPlanName()).append("\n");

    return str.toString();
  } // end of createString
} // end of RFQDialog Class
