/**
 * ------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: TPGanttModel.java,v 1.5 2001/10/17 03:41:08 jcollins Exp $
 * Copyright (c) 2000,2001 University of Minnesota
 * ------------------------------------------------------------------------
 * Revision History:
 *
 * 18Feb2001 DFK - Initial Entry
 * 02May2001 DFK - Bug #0000022: Listen for planChanged events from TPTableModel,
 *                  not directly from TaskPlan.
 * 13May2001 DFK - Bug #0000053: Added getView(),setView() for current GanttPanel view.
 */

package edu.umn.magnet.client.userif;

import javax.swing.event.EventListenerList;
import javax.swing.table.*;
import java.text.DateFormat;
import java.util.*;

import edu.umn.magnet.protocol.cust.*;
import org.apache.log4j.Category;

/**
 * A data model that wraps around a {@link TaskPlan} and produces a Gantt chart.
 * I only extended {@link AbstractTableModel} in order to get add/removeListener()
 * stuff for free..  may get rid of it later..
 */
public class TPGanttModel //extends AbstractTableModel
                          implements PlanListener
{
  // See {@link TaskPlan} for hints on dealing w/ listeners..
  /** List of listeners to this model. */
  protected EventListenerList _listenerList = null;
  /** The associated table model. */
  protected TPTableModel _tpTM = null;
  /** Our ref to a GanttPanel view. */
  private GanttPanel _view = null;

  /** log category */
  private static Category log =
                    Category.getInstance(TPGanttModel.class.getName());
  public TPGanttModel() {
    _listenerList = new EventListenerList();
  }
  public TPGanttModel(TPTableModel m) {
    _listenerList = new EventListenerList();
    _tpTM = m;
//    /** Attach ourselves as a listener to the TaskPlan.. */
//    TaskPlan tp = _tpTM.getTaskPlan();
//    if (null!=tp) {
//      tp.addPlanListener(this);
//    }
    // dfk 2001May01: listen to the TPTableModel instead of directly to TaskPlan.
    if (null!=m) {
      m.addPlanListener(this);
    }
  }

  /** Get the associated TPTableModel. */
  public TPTableModel getTPTableModel() {
    return _tpTM;
  }
  /** Set the associated TPTableModel. */
  public void setTPTableModel(TPTableModel m) {
    _tpTM = m;
  }

  /** Called by the {@link TaskPlan} we're currently listening to when the TaskPlan changes.
   */
  public void planChanged(TPEvent e) {
    log.debug("TPGanttModel.planChanged(): e="+e.toString());
    fireGanttDataChanged(e.getMessage());
  }

  /** Get the {@link SubtaskRequest}'s ordinal position in the {@link TPTableModel}'s flattened list.
   *  (Defers to TPTableModel.)
   *  @return -1 if not found, or the position [0..??] in the list.
   */
  public int taskId2TaskNum(String taskID) {
    return ((null==_tpTM)  ?  -1  :  _tpTM.taskId2TaskNum(taskID));
  }


  // -------------------------------
  /** Methods to support/feed GanttListener instances.
   *  For more ideas, consult {@link EventListenerList} and {@link AbstractTableModel} javadocs.
   */
    //DFK
  /** Adds a listener to the list that's notified each time a change to the data model occurs. */
  public void addGanttListener(GanttListener l) {
    _listenerList.add(GanttListener.class, l);
  }
  /** Remove a listener from the notify-list. */
  public void removeGanttListener(GanttListener l) {
    _listenerList.remove(GanttListener.class, l);
  }
  /** Indicate (in a vague sense) that the plan has somehow been changed. */
  public void fireGanttDataChanged() {
    TPGanttEvent myTPGanttEvent = new TPGanttEvent(this, "chart has been modified");  // generic for now
    fireGanttDataChanged(myTPGanttEvent);
  }
  /** Indicate that the chart has somehow been changed.
   *  @param aMessage Some arbitrary message (currently not used for anything useful)
   */
  public void fireGanttDataChanged(String aMessage) {
    TPGanttEvent myTPGanttEvent = new TPGanttEvent(this, aMessage);  // generic for now
    fireGanttDataChanged(myTPGanttEvent);
  }

  /** Indicate that the chart has somehow been changed.
   *  @param evt The {@link TPGanttEvent} to broadcast to all listeners.
   */
  public void fireGanttDataChanged(TPGanttEvent evt) {
    log.debug("TPGanttModel: firing event e="+evt.toString());
    // Guaranteed to return a non-null array
    Object[] listeners = _listenerList.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length-2; i>=0; i-=2) {
//      System.err.println("dfk: firePlanDataChanged(): listeners["+i+"]="+listeners[i]);
//       if (listeners[i] instanceof edu.umn.magnet.userif.PlanListener) {
       if (listeners[i] == GanttListener.class) {
//           System.err.println("dfk: firePlanDataChanged(): listeners["+i+"] is a PlanListener");
           ((GanttListener)listeners[i+1]).chartChanged(evt);
       }
    }
  }
  // -------------------------------

  /** Set the current view to the provided GanttPanel. */
  public void setView(GanttPanel view) {
    _view = view;
  }
  /** Get the current GanttPanel view, or null if none exists. */
  public GanttPanel getView() {
    return _view;
  }
}
