/*------------------------------------------------------------------------
 * MAGNET User Interface components
 * $Id: TaskPlanEditor4Panel.java,v 1.45.2.1 2002/05/05 19:42:02 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Thu Jan  3 19:24:43 2002  John Collins  #0000128
 *   Don't allow submit RFQ if task plan has no tasks.
 * 
 * Wed Jan  2 22:28:01 2002  John Collins  #0000163
 *   Eliminate dependence on Borland library
 *
 * Wed May 23 14:32:32 2001  John Collins  #0000107
 *   Fill in default values for task start and end times.
 *
 * Wed May 23 11:56:32 2001  John Collins  #0000124
 *   Rearrange buttons on task detail panel, open up layout
 *
 * Sun May 20 21:58:39 2001  John Collins
 *   Add ability to recompute schedule.
 *
 * Wed May 16 09:45:08 2001  John Collins  #0000083
 *   Add default bid cycle timings
 *
 * 15Apr2001 DFK - Initial Entry
 * 16Apr2001 DFK - Intermediate checkin: just paste a hello-world jlabel onto panel
 * 16Apr2001 DFK - Intermediate checkin: adding components into panel
 * 20Apr2001 DFK - Intermediate checkin: hook up listeners to buttons;  JobInfo hndlr.
 * 23Apr2001 DFK - Intermediate checkin: JobInfo now selected via combobox, & updated from table.
 * 24Apr2001 DFK - Bug #0000021: added textfield for task dependencies
 * 26Apr2001 DFK - Add "Submit RFQ" btn; Initial additions to support GUI error handler
 * 27Apr2001 DFK - Finish up checkBeforeSaveNew() error-checking and doSaveNew().
 * 30Apr2001 DFK - Finish up SaveNew and Commit stuff.
 *                 Bug #0000032: TaskPlan summary / editor support.
 * 02May2001 DFK - Bug #0000022: Some debug printfs during commit
 * 05May2001 PBG - Hooked in RFQ Status Object creation
 * 07May2001 PBG - Comment out a println, Change so uses GuiUtil.get*Status
 * 13May2001 PBG - Added in the status requirements and created BidCommand
 * 14May2001 PBG - Bug #0000067: Display reason why create RFQ Status fails
 * 15May2001 DFK - Bug #0000077: Rework doCommit() do delete old task before
 *                    saving new task;  also print exception messageText in inform()'s.
 * 15May2001 DFK - Bug #0000079: Convert Time/Date specs to mm/dd/yy hh:mm;
 *                 Bug #0000073: Don't display JobInfo ExpectedDuration in msec.
 * 19May2001 PBG - Bug #0000091, #0000101: Keep flow similar to that of agent
 * 02Jun2001 JCollins - added Resource/Supplier availabilities fields
 */

package edu.umn.magnet.client.userif;

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;

import org.apache.log4j.Category;

import edu.umn.magnet.protocol.cust.*;
import edu.umn.magnet.protocol.MagnetException;
import edu.umn.magnet.protocol.InvalidDataException;
import edu.umn.magnet.protocol.JobInfo;
import edu.umn.magnet.client.plan.JobInfoFactory;  // for testing
import edu.umn.magnet.util.*;  // for testing
import edu.umn.magnet.util.time.AgentTime;   // for example date strings
import edu.umn.magnet.client.status.*; // for status objects
import edu.umn.magnet.client.bid.*; // for BidManager & bidcommand
import edu.umn.magnet.client.eval.TaskBidPlan;


public class TaskPlanEditor4Panel extends JPanel
  implements ListSelectionListener, TableModelListener,
	     ActionListener, MagnetClientConstants,
	     ErrorObjectHandler.ErrorFocus
{
  // log category
  private static Category log =
    Category.getInstance(TaskPlanEditor4Panel.class.getName());

  String _titlePlanRFQEditor;
  String _titleAddEditTask;
  JLabel _lblTaskID = new JLabel();
  JTextField _tfTaskID = new JTextField();
  JLabel _lblTaskName = new JLabel();
  JTextField _tfTaskName = new JTextField();
  JLabel _lblDeps = new JLabel();
  JTextField _tfDeps = new JTextField();
  String _titleJobInfo;
  String _pfxTaskType;
  JLabel _lblTaskType = new JLabel();
  JComboBox _cbTaskType = new JComboBox();
  DefaultComboBoxModel _cbmJobInfos = new DefaultComboBoxModel();
  String _pfxExpDuration;
  JLabel _lblExpDuration = new JLabel();
  String _pfxExpPrice;
  JLabel _lblExpPrice = new JLabel();
  String _pfxResourceAvail;
  JLabel _lblResourceAvail = new JLabel();
  String _pfxSupplierAvail;
  JLabel _lblSupplierAvail = new JLabel();
  JButton _btnChangeJobInfo = new JButton();
  JLabel _lblEarliestStart = new JLabel();
  JTextField _tfEarliestStart = new JTextField();
  JLabel _lblDeadline = new JLabel();
  JTextField _tfDeadline = new JTextField();
  JLabel _lblComments = new JLabel();
  JScrollPane _spComments = new JScrollPane();
  JTextArea _taComments = new JTextArea();
  JButton _btnDelTask = new JButton();
  JButton _btnSaveNew = new JButton();
  String _pfxCommit;
  String _pfxSave;
  JButton _btnCommit = new JButton();
  JButton _btnClear = new JButton();
  JButton _btnHelp = new JButton();

  String _titleTPDetails;
  String _pfxPlanTitle;
  JLabel _lblPlanTitle = new JLabel();
  JTextField _tfPlanTitle = new JTextField();
  String _pfxPlanStartDate;
  JLabel _lblPlanStartDate = new JLabel();
  JTextField _tfPlanStartDate = new JTextField();
  String _pfxBestCompletion;
  JLabel _lblBestCompletion = new JLabel();
  JTextField _tfBestCompletion = new JTextField();
  String _pfxPlanDeadline;
  JLabel _lblPlanDeadline = new JLabel();
  JTextField _tfPlanDeadline = new JTextField();
  String _pfxTotNumTasks;
  JLabel _lblTotNumTasks = new JLabel();
  JTextField _tfTotNumTasks = new JTextField();
  JButton _btnChangePlanInfo = new JButton();
  JButton _btnRevertPlanInfo = new JButton();
  JButton _btnReschedule = new JButton();

  String _titleRFQSettings;
  //JLabel _lblEarliestBidConsider = new JLabel();
  //JTextField _tfEarliestBidConsider = new JTextField();
  JLabel _lblEarliestBidOffer = new JLabel();
  JTextField _tfEarliestBidOffer = new JTextField();
  JLabel _lblBidDeadline = new JLabel();
  JTextField _tfBidDeadline = new JTextField();
  //JButton _btnSubmitRFQ = new JButton();
  JButton _btnSubmitRFQWithDefaults = new JButton();

  private JList _bogusList = new JList(new String[]{" "});
  /** For Earliest-Start and deadline checking of a task */
  private boolean legibleTaskDates;
  private Date dateEStart;
  private Date dateDeadline;
  private Date dateEBidConsider;
  private Date dateEBidOffer;
  private Date dateBidDeadline;

  private long _defaultBidDeadline = 10000;
  private long _defaultEarlyConsider = 5000;
  private long _defaultEarlyOffer = 60000;
  private boolean _defaultsRead = false;

  private Date _planStartDate = null;

  private MagnetClientMain _parent = null;

//  JPanel _jpAddEditTask = new JPanel();
//  JPanel _jpTaskPlanDetails = new JPanel();
//  JPanel _jpRFQGlobalSettings = new JPanel();

  TPTableModel _tpTableModel = null;

  /** Constructor */
  public TaskPlanEditor4Panel() {
    // Add components to panel
    addComponentsToPanel();
    // load values
    loadDynamicComponentValues();
  }
  /** Constructor */
  public TaskPlanEditor4Panel(TPTableModel m, MagnetClientMain parent) {
    _parent = parent;  // need to set so can change screens
    setTPTableModel(m);
    // Add components to panel
    addComponentsToPanel();
    // load values
//    loadDynamicComponentValues();
    /** If table's already selected a row, highlight that; otherwise do default no-selected. */
    ListSelectionModel lsm=null;
    if (null!=m  &&
        null!=m.getView()  &&
        null!=(lsm=m.getView().getSelectionModel())  &&
        !lsm.isSelectionEmpty()
       )  {
      //load the selected row's data
      loadDataFromTable(m, lsm.getMinSelectionIndex());
    } else {
      loadDataFromTable(m, -1);   // populate placeholders
    }
  }

  /** Add the components to the panel.
   *  @todo Make the necessary labels RED and BOLDFACE..
   */
  private void addComponentsToPanel() {
    loadStaticStrings();
    initJobInfoComboBox();

    // Do anything special to make red and BOLDFACE??
    //  For now, mark them like this:  "**Earliest Start:" .

    // Add listeners?
    attachButtonListeners();

    /**
     * Place the components
     */

    // make 4 add/edit subpanels, then add those to the add/edit panel.
    JPanel add1 = new JPanel(new GridBagLayout());
    JPanel add2 = new JPanel(new GridBagLayout());
    JPanel add3 = new JPanel(new GridBagLayout());
    JPanel add4 = new JPanel(new GridBagLayout());
    JPanel addEditBtnsPanel = new JPanel(new FlowLayout());
    JPanel addEditPanel = new JPanel(new GridBagLayout());

    // Task ID/Name
    GuiUtil.constrain(add1, _lblTaskID,
                      0, 0,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA);
    GuiUtil.constrain(add1, _tfTaskID,
                      1,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA);
    GuiUtil.constrain(add1, _lblTaskName,
                      0,1,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA);
    GuiUtil.constrain(add1, _tfTaskName,
                      1,1,
                      2,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA);
    GuiUtil.constrain(add1, _lblDeps,
                      2,0,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA);
    GuiUtil.constrain(add1, _tfDeps,
                      3,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA,
                      GuiUtil.LAYOUT_INTRA);

    //JobInfo stuff
    add2.setBorder(BorderFactory.createTitledBorder(_titleJobInfo));
    GuiUtil.constrain(add2, _lblTaskType,
                      0,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_INTRA,
                      0,
                      0,
                      0);
    GuiUtil.constrain(add2, _cbTaskType,
                      1,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      0);
    GuiUtil.constrain(add2, _lblExpDuration,
                      0,1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      0);
    GuiUtil.constrain(add2, _lblResourceAvail,
                      1,1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      0);
    GuiUtil.constrain(add2, _lblExpPrice,
                      0,2,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      0);
    GuiUtil.constrain(add2, _lblSupplierAvail,
                      1,2,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      0);
//    GuiUtil.constrain(add2, _btnChangeJobInfo,
//                      1,0,
//                      1,3,
//                      GridBagConstraints.VERTICAL, GridBagConstraints.EAST,
//                      0.0, 0.0,
//                      0,//GuiUtil.LAYOUT_MARGIN,
//                      0,
//                      0,
//                      0);

    // Earliest start / deadline
    GuiUtil.constrain(add3, _lblEarliestStart,
                      0,0,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(add3, _tfEarliestStart,
                      1,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(add3, _lblDeadline,
                      2,0,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(add3, _tfDeadline,
                      3,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      0);


    // Comments
    GuiUtil.constrain(add4, _lblComments,
                      0,0,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.NORTHWEST,
                      0.0, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      GuiUtil.LAYOUT_MARGIN);
    _taComments.setWrapStyleWord(true);
    _taComments.setLineWrap(true);
    _spComments = new JScrollPane(_taComments);
    _spComments.setPreferredSize(new Dimension(50,60));
    GuiUtil.constrain(add4, _spComments,
                      1,0,
                      1,3,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      0,//GuiUtil.LAYOUT_MARGIN,
                      0,
                      0,
                      0);

    // The row of editor buttons
    addEditBtnsPanel.add(_btnHelp);
    addEditBtnsPanel.add(_btnDelTask);
    addEditBtnsPanel.add(_btnClear);
    addEditBtnsPanel.add(_btnSaveNew);
    addEditBtnsPanel.add(_btnCommit);

    //
    // insert the add/edit subpanels in the add/edit parent panel.
    //
    addEditPanel.setBorder(BorderFactory.createTitledBorder(
                                                _titleAddEditTask));
    GuiUtil.constrain(addEditPanel, add1,
                      0,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(addEditPanel, add2,
                      0,1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(addEditPanel, add3,
                      0,2,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(addEditPanel, add4,
                      0,3,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(addEditPanel, addEditBtnsPanel,
                      0,4,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.EAST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);

    //
    // The TaskPlan details panel.
    //
    JPanel tpDetailsPanel = new JPanel(new GridBagLayout());
    tpDetailsPanel.setBorder(BorderFactory.createTitledBorder(
                                                _titleTPDetails));
    GuiUtil.constrain(tpDetailsPanel, _lblPlanTitle,
                      0,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _tfPlanTitle,
                      1,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _lblPlanStartDate,
                      0,1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _tfPlanStartDate,
                      1,1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _lblBestCompletion,
                      0,2,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _tfBestCompletion,
                      1,2,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    _tfBestCompletion.setEditable(false);
    GuiUtil.constrain(tpDetailsPanel, _lblPlanDeadline,
                      0,3,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _tfPlanDeadline,
                      1,3,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _lblTotNumTasks,
                      0,4,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(tpDetailsPanel, _tfTotNumTasks,
                      1,4,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    _tfTotNumTasks.setEditable(false);
    JPanel tpButtonPanel = new JPanel(new FlowLayout());
    GuiUtil.constrain(tpDetailsPanel, tpButtonPanel,
                      0,5,
                      2,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    tpButtonPanel.add(_btnChangePlanInfo, null);
    tpButtonPanel.add(_btnRevertPlanInfo, null);
    tpButtonPanel.add(_btnReschedule, null);

    //
    // The RFQ Global Settings panel.
    //
    JPanel rfqPanel = new JPanel(new GridBagLayout());
    rfqPanel.setBorder(BorderFactory.createTitledBorder(_titleRFQSettings));
//      GuiUtil.constrain(rfqPanel, _lblEarliestBidConsider,
//                        0,0,
//                        1,1,
//                        GridBagConstraints.NONE, GridBagConstraints.WEST,
//                        0.0, 0.0,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN);
//      GuiUtil.constrain(rfqPanel, _tfEarliestBidConsider,
//                        1,0,
//                        1,1,
//                        GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
//                        0.5, 0.0,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(rfqPanel, _lblEarliestBidOffer,
                      0,1,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(rfqPanel, _tfEarliestBidOffer,
                      1,1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(rfqPanel, _lblBidDeadline,
                      0,0,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.WEST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(rfqPanel, _tfBidDeadline,
                      1,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
//      GuiUtil.constrain(rfqPanel, _btnSubmitRFQ,
//                        1,3,
//                        1,1,
//                        GridBagConstraints.NONE, GridBagConstraints.EAST,
//                        0.0, 0.0,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN,
//                        GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(rfqPanel, _btnSubmitRFQWithDefaults,
                      1,2,
                      1,1,
                      GridBagConstraints.NONE, GridBagConstraints.EAST,
                      0.0, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);

    //
    // Insert add/edit, TaskPlanDetails, RFQ panels into main panel.
    //
    this.setLayout(new GridBagLayout());
    this.setBorder(BorderFactory.createTitledBorder(_titlePlanRFQEditor));
    GuiUtil.constrain(this, addEditPanel,
                      0,0,
                      1,2,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(this, tpDetailsPanel,
                      1,0,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);
    GuiUtil.constrain(this, rfqPanel,
                      1,1,
                      1,1,
                      GridBagConstraints.HORIZONTAL, GridBagConstraints.EAST,
                      0.5, 0.0,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN,
                      GuiUtil.LAYOUT_MARGIN);


//    this.add(new JLabel("DAVE: TaskPlanEditor4Panel goes here"));  // DELETE THIS !!!

  }

  /** Attach an actionListener to the buttons. */
  public void attachButtonListeners() {
//    _btnChangeJobInfo.addActionListener(this);
    _btnDelTask.addActionListener(this);
    _btnSaveNew.addActionListener(this);
    _btnCommit.addActionListener(this);
    _btnClear.addActionListener(this);
    _btnHelp.addActionListener(this);
    _btnChangePlanInfo.addActionListener(this);
    _btnRevertPlanInfo.addActionListener(this);
    _btnReschedule.addActionListener(this);
    //_btnSubmitRFQ.addActionListener(this);
    _btnSubmitRFQWithDefaults.addActionListener(this);
  }

  /** Load the non-changing strings (eg. textfield labels).
   *  @todo Load and set ToolTips too!!
   * @todo eventually load strings from a resourceBundle...
   */
  private void loadStaticStrings() {
    _titlePlanRFQEditor = new String("Edit Plan/RFQ");
    _titleAddEditTask = new String("Add/Edit Task");
    _lblTaskID.setText("Task ID:");
    _lblTaskName.setText("Task Name:");
    _lblDeps.setText("Starts after tasks:");
    _titleJobInfo = new String("JobInfo Template for this task");
    _pfxTaskType = new String("Task Type:");
    _pfxExpDuration = new String("Expected Duration: ");
    _pfxExpPrice = new String("Expected Price: ");
    _pfxResourceAvail = "Resource Availability: ";
    _pfxSupplierAvail = "Supplier Availability: ";
    _btnChangeJobInfo.setText("Change...");
    _lblEarliestStart.setText("Earliest Start:");
    _lblDeadline.setText("Deadline:");
    _lblComments.setText("Comments:");
    _btnDelTask.setText("Delete");
    _btnSaveNew.setText("New");
    _pfxCommit = "Commit";
    _pfxSave = "  Save  ";
    _btnCommit.setText(_pfxCommit);
    _btnClear.setText("Clear");
    _btnHelp.setText("Help");

    _titleTPDetails = "Plan details";
    _pfxPlanTitle = "Plan Title:";
    _pfxPlanStartDate = "Plan Start Date:";
    _pfxBestCompletion = "Best-Case Completion:";
    _pfxPlanDeadline = "Plan Deadline:";
    _pfxTotNumTasks = "Total number of tasks:";
    _lblPlanTitle.setText(_pfxPlanTitle);
    _lblPlanStartDate.setText(_pfxPlanStartDate);
    _lblBestCompletion.setText(_pfxBestCompletion);
    _lblPlanDeadline.setText(_pfxPlanDeadline);
    _lblTotNumTasks.setText(_pfxTotNumTasks);
    _btnChangePlanInfo.setText("Change");
    _btnRevertPlanInfo.setText("Revert");
    _btnReschedule.setText("Reschedule");
    _titleRFQSettings = new String("RFQ Details");
    //_lblEarliestBidConsider.setText("**Earliest Bid Consider:");
    _lblBidDeadline.setText("Bid Interval (H:M:S):");
    _lblEarliestBidOffer.setText("Minimum Eval Time (H:M:S):");
    _tfBidDeadline.setText
	(GuiUtil.interval2String(getDefaultBidDeadline()));
    _tfEarliestBidOffer.setText
	(GuiUtil.interval2String(getDefaultEarlyOffer()));
    //_btnSubmitRFQ.setText("**Submit RFQ...");
    _btnSubmitRFQWithDefaults.setText("Submit RFQ...");

    // DAVE: load tool tips here??
  }

  /** Set up the JobInfo combobox and model. */
  public void initJobInfoComboBox() {
    // iterate thru JobInfos, adding entries to _cbm..
//    TreeMap hm = /*static call*/ edu.umn.magnet.client.plan.JobInfoFactory.sortedList();
    TreeMap hm = ((JobInfoFactory)JobInfoFactory.getInstance()).sortedList();
    if ((null==hm) || (hm.isEmpty())) { return; }
    int count=0;
    for (Iterator jobs = hm.values().iterator(); jobs.hasNext(); ) {
      JobInfo result = (JobInfo)(jobs.next());
      log.debug("\tFound JobInfo#"+(count++)+": " + result.getTaskType() );
      _cbmJobInfos.addElement(result);
      count++;
    }
    log.debug("\tFound "+count+" JobInfo instances.");

    _cbTaskType.setModel(_cbmJobInfos);     // attach _cb to _cbm..
//    _cbTaskType.setEditable(false);
//    _cbTaskType.setEditor((ComboBoxEditor)null);
    // attach renderer to CB that understands JobInfos..
    _cbTaskType.setRenderer(new DefaultListCellRenderer (){
         public Component getListCellRendererComponent(
             JList list,
             Object value,
             int index,
             boolean isSelected,
             boolean cellHasFocus)
//         {
//             JLabel aLabel = new JLabel();
//             if (value instanceof JobInfo  &&  null!=value) {
//               aLabel.setText( ((JobInfo)value).getTaskType() );
//             }
//             aLabel.setBackground(isSelected ? Color.red : Color.white);
//             aLabel.setForeground(isSelected ? Color.white : Color.black);
//             return aLabel;
//         }

//         // Yes I stole this from the source for DefaultListCellRenderer, shame on me..
//         {
//              JLabel aLabel = new JLabel();
//              // I hated the "default" foreground/bkgd colors, so I'm overriding 'em..
//              list.setSelectionBackground(Color.blue);
//              list.setSelectionForeground(Color.white);
//              list.setBackground(Color.white);
//              list.setForeground(Color.black);
//              aLabel.setComponentOrientation(list.getComponentOrientation());
//              if (isSelected) {
//                  aLabel.setBackground(list.getSelectionBackground());
//                  aLabel.setForeground(list.getSelectionForeground());
//              }
//              else {
//                  aLabel.setBackground(list.getBackground());
//                  aLabel.setForeground(list.getForeground());
//              }
//  //              if (value instanceof Icon) {
//  //                  setIcon((Icon)value);
//  //                  setText("");
//  //              }
//  //              else {
//  //                  setIcon(null);
//  //                  setText((value == null) ? "" : value.toString());
//  //              }
//             if (value instanceof JobInfo  &&  null!=value) {
//               aLabel.setText( ((JobInfo)value).getTaskType() );
//             }
//              aLabel.setEnabled(list.isEnabled());
//              aLabel.setFont(list.getFont());
//              aLabel.setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);
//              return aLabel;  //this;
//         }
         // From JComponentCellRenderer....
         {  // start of getting renderer component
//                DefaultTableCellRenderer dRenderer = new DefaultTableCellRenderer();
//                dRenderer.getTableCellRendererComponent(/*table*/new JTable(1,1), value,
//                        isSelected, /*hasFocus*/cellHasFocus, /*row, column*/0,0);
//             global;   JList _bogusList = new JList(new String[]{" "});
             DefaultListCellRenderer dRenderer = new DefaultListCellRenderer();
             dRenderer.getListCellRendererComponent(_bogusList, " ", 0, isSelected, cellHasFocus);
             JLabel aLabel = new JLabel();
             aLabel.setOpaque(true);
             aLabel.setBackground(dRenderer.getBackground());
             aLabel.setForeground(dRenderer.getForeground());
             if (value instanceof JobInfo  &&  null!=value) {
               aLabel.setText( ((JobInfo)value).getTaskType() );
//                  ((JLabel)value).setOpaque(true);
//                  ((JLabel)value).setBackground(dRenderer.getBackground());
//                  ((JLabel)value).setForeground(dRenderer.getForeground());
             }
//                if (value instanceof JLabel) {
//                  ((JLabel)value).setOpaque(true);
//                  ((JLabel)value).setBackground(dRenderer.getBackground());
//                  ((JLabel)value).setForeground(dRenderer.getForeground());
//                } else {
//                  setText(value.toString());
//                }
                return (JComponent)aLabel; //value;
         }
     });
    // attach listener/handler to cb?
    _cbTaskType.addActionListener(new ActionListener(){
        public void actionPerformed(ActionEvent e) {
          JobInfo ji = (JobInfo)_cbmJobInfos.getSelectedItem();
          showJobInfoData(ji);
        }
    });
    showJobInfoData((JobInfo)null);  // init w/ prefix vals..
  }

  /**
   * Given a JobInfo's TaskType string, load the corresponding JobInfo
   * into the related fields.
   *  @param jiTaskType The TaskType string
   */
  public void showJobInfoData(String jiTaskType) {
    showJobInfoData((JobInfo)null);     // display prefixes, if nothing else..
    if (null==jiTaskType  ||  0==jiTaskType.length())  {  return;  }
    log.debug("loading data for JobInfo " + jiTaskType);
    JobInfoFactory jif = (JobInfoFactory)JobInfoFactory.getInstance();
    JobInfo result = jif.getJobInfo(jiTaskType);
    if (result != null) {
      showJobInfoData(result);
      return;
    }
    log.error("No JobInfo named " + jiTaskType);
  }

  /** Load the requested JobInfo into the related fields. */
  public void showJobInfoData(JobInfo ji) {
    String taskType = _pfxTaskType;//combobox does this..;
    long lExpDuration = (null==ji) ? 0L : (long)ji.getExpectedDuration();
    String expDuration = (_pfxExpDuration
			  + TPTableModel.timeSpan(new Date(0L),
						  new Date(lExpDuration)));
    String expPrice = (_pfxExpPrice
		       + ((null==ji) ? "" : ""+ji.getExpectedPrice()));
    String resourceAvail =
	(_pfxResourceAvail
	 + ((ji == null) ? "" :
	    Double.toString(ji.getResourceAvailability())));
    String supplierAvail =
	(_pfxSupplierAvail
	 + ((ji == null) ? "" :
	    Double.toString(ji.getSupplierAvailability())));
    _cbmJobInfos.setSelectedItem(ji);
    _lblTaskType.setText(taskType);
    _lblExpDuration.setText(expDuration);
    _lblExpPrice.setText(expPrice);
    _lblResourceAvail.setText(resourceAvail);
    _lblSupplierAvail.setText(supplierAvail);
  }

  /** Load just prefixes into the dynamic-valued components. */
  public void loadDynamicComponentValues() {
    log.debug("** TaskPlanEditor4Panel.loadDynamicComponentValues():  not implemented yet!");
    _lblTaskType.setText(_pfxTaskType); //+ "BOGUS");
    _lblExpDuration.setText(_pfxExpDuration); //+ "BOGUS");
    _lblExpPrice.setText(_pfxExpPrice); //+ "BOGUS");
  }

  /** Clear out all TaskEditor textfields / text areas / etc..
   *  This will probably be called when the "Clear" button is pushed.
   */
  public void clearAllTaskInputFields() {
    _tfTaskID.setText("");
    _tfTaskName.setText("");
    _tfDeps.setText("");
    _tfEarliestStart.setText("");
    _tfDeadline.setText("");
    _taComments.setText("");
  }


  /** Set the current TPTableModel, so we can query it for data. */
  protected void setTPTableModel(TPTableModel m) {
    if (null==m  &&  null!=_tpTableModel)  {  _tpTableModel.removeTableModelListener(this);  }
    _tpTableModel = m;
    if (null!=m)  {  m.addTableModelListener(this);  }
    // Also attach us as a listener to the model's associated JTable.
    listenToTable( m.getView() );
  }
  /** Return the current TPTableModel. */
  protected TPTableModel getTPTableModel() {
    return _tpTableModel;
  }

  /** Respond to table selection change events */
  public void valueChanged(ListSelectionEvent e) {
    log.debug("TPE4P.valueChanged(): e="+e.toString());
    // Only consider non-changing indices; bail otherwise.
    if (e.getValueIsAdjusting()==true) { return; }
    /** The event's index info is FLAKEY, so let's ask the selection model directly. */
    if (e.getSource() instanceof DefaultListSelectionModel) {
      DefaultListSelectionModel m = (DefaultListSelectionModel)e.getSource();
      /** @todo Should keep a "editor dirty" flag so we prompt before blasting any
       *  in-progress hand-edits..  For now, just blast 'em..
       */
      if (null!=_tpTableModel) {
        if (!m.isSelectionEmpty()) {
          int changedRow = m.getMinSelectionIndex();
          loadDataFromTable(_tpTableModel, changedRow);
	  setCommitMode();
        }
      }
    } else {
      System.err.println("TPE4P.valueChanged(): error: Unexpected event src "+e.getSource());
    }
  }

  /** The calling class (w/ table) can sign us up as a table listener. */
  public void listenToTable(JTable table) {
//    _table = table;
    if (null==table) { return; }
    listenForTableChangeEvents(table);
  }

  /** Hook up a change listener to the JTable, and populate fields as necessary. */
  public void listenForTableChangeEvents(JTable table) {
    if (null==table) {  return;  }
    ListSelectionModel lsm = table.getSelectionModel();
    if (null==lsm) {
      table.setSelectionModel(lsm = new DefaultListSelectionModel());
    }
    lsm.addListSelectionListener(this);
  }


  /** From TableModelListener interface; When the table changes, update the summary panel.
    * @todo Put TaskPlanEditorPanel.recalcSummaryPanel() calculations into TPTableModel!
    */
  public void tableChanged(TableModelEvent e) {
    recalcSummaryPanel();
  }

  /** Recalculate the Earliest Start and latest Deadline dates in the plan, number of tasks, etc.  Redraw.
   *
   */
  private void recalcSummaryPanel() {
    log.debug("TPE4P.recalcSummaryPanel(): entering");
    TPTableModel m = _tpTableModel;
    if (null==m) { return; }
    TaskPlan tp = m.getTaskPlan();
    if (null==tp) { return; }
    _tfPlanTitle.setText(m.getTaskPlan().getPlanName());
    _tfTotNumTasks.setText(Integer.toString(tp.size()));
    // Find the earliest start and latest deadline of all tasks.
    if (tp.size() <= 0) {
      _tfPlanStartDate.setText("");
      _tfBestCompletion.setText("");
      _tfPlanDeadline.setText("");
    } else {
      Date eStart = m.getTask(0).getEarliestTimeToStart();
      Date deadline = m.getTask(0).getDeadline();
      for (int i=1; i < tp.size(); i++) {
        if (eStart.after(m.getTask(i).getEarliestTimeToStart())) {
          eStart = m.getTask(i).getEarliestTimeToStart();
        }
        if (deadline.before(m.getTask(i).getDeadline())) {
          deadline = m.getTask(i).getDeadline();
        }
      }
      _tfPlanStartDate.setText(DateFormat.getDateTimeInstance(
                        DateFormat.SHORT, DateFormat.SHORT).format(eStart));
      _tfBestCompletion.setText(DateFormat.getDateTimeInstance(
                      DateFormat.SHORT, DateFormat.SHORT).format(deadline));
    }
  }



  /** Load task data from the selected JTable row into our editor fields. */
  private void loadDataFromTable(TPTableModel m, int aRow) {
    // dfk: these JobInfo items are now done in {@link this#showJobInfoData(JobInfo)}..
   log.debug("loadDataFromTable(m="+m+",row="+aRow+"): entering");
    showJobInfoData( (String)m.getValueAt(aRow, m.COL_JOBINFO_TASK_TYPE) );
    // For TaskPlan details, call recalcSummaryPanel().
    recalcSummaryPanel();
    if (null==m)  {  return;  }
    String s;
    _tfTaskID.setText(s=(String)m.getValueAt(aRow, m.COL_TASK_ID));
    _tfTaskName.setText(s=(String)m.getValueAt(aRow, m.COL_TASK_NAME));
    _tfDeps.setText((String)m.getValueAt(aRow, m.COL_TASK_PREDS));
    _tfEarliestStart.setText((String)m.getValueAt(aRow, m.COL_TASK_ESTART));
    _tfDeadline.setText((String)m.getValueAt(aRow, m.COL_TASK_DEADLINE));
    _taComments.setText((String)m.getValueAt(aRow, m.COL_TASK_COMMENTS));
  }

  /** Listen to button-press actions. */
  public void actionPerformed(ActionEvent e) {
    log.debug("TPE4P.actionPerformed(): entering.");
    if (e.getSource()==_btnChangeJobInfo) {
      log.debug("\t'Change JobInfo' button pressed.");
//      doChangeJobInfo();
    } else if (e.getSource()==_btnDelTask) {
      log.debug("\t'Delete Task' button pressed.");
      doDeleteTask();
    } else if (e.getSource()==_btnSaveNew) {
      log.debug("\t'Save New' button pressed.");
      doNew();
      //doSaveNew();
    } else if (e.getSource()==_btnCommit) {
      log.debug("\t'Commit' button pressed.");
      doCommit();
    } else if (e.getSource()==_btnClear) {
      log.debug("\t'Clear' button pressed.");
      doClear();
    } else if (e.getSource()==_btnHelp) {
      log.debug("\t'Help' button pressed.");
      doHelp();
    } else if (e.getSource()==_btnChangePlanInfo) {
      log.debug("'Change Plan Info' button pressed.");
      doChangePlanInfo();
    } else if (e.getSource()==_btnRevertPlanInfo) {
      log.debug("'Revert Plan Info' button pressed.");
      doRevertPlanInfo();
    } else if (e.getSource()==_btnReschedule) {
      log.debug("'Reschedule' button pressed.");
      doReschedule();
//      } else if (e.getSource()==_btnSubmitRFQ) {
//        log.debug("\t'Submit RFQ' button pressed.");
//        doSubmitRFQ();
    } else if (e.getSource()==_btnSubmitRFQWithDefaults) {
      log.debug("\t'Submit RFQ w/ Defaults' button pressed.");
      doSubmitRFQWithDefaults();
    } else {
      log.debug("TPE4P.actionPerformed(): Unknown source "+e.getSource());
    }
  }

  /** A quick temporary method to pop up the string in an info dialog.. */
  public void inform(String s) {
    int result = JOptionPane.showConfirmDialog(this,
                        new JLabel(s),
                        "Info..", JOptionPane.CANCEL_OPTION,
                        JOptionPane.INFORMATION_MESSAGE);
  }

  /** Handle the TaskEditor "Delete Task" button.
   *  If problems are encountered, it fails silently.
   */
  public void doDeleteTask() {
    if (null!=_tpTableModel) {
      int aRow = _tpTableModel.getView().getSelectedRow();
      if (-1!=aRow) {  // a row selected
        TaskPlan tp = _tpTableModel.getTaskPlan();
        if (null!=tp) {
          int taskNum = aRow+1;   //off-by-1
          String taskID = _tpTableModel.taskNum2TaskId(taskNum);
          if (null!=taskID  &&  taskID.length()!=0) {
            try {
              tp.removeTask( tp.getTWP(taskID).getTaskData() );
            } catch (InvalidTaskException e) {
            }
            /** Update the summary panel w/ the current set of tasks.. */
            /** @todo Why do I need to explicitly call this?
             *  The plan listener (or whatever) should do this..  right??
             */
            recalcSummaryPanel();
            // Reselect the row of the same position, if possible
            int numRows = _tpTableModel.getRowCount();
            if (0==numRows) {
              // do nothing
            } else {
              if (aRow < numRows) {
                _tpTableModel.getView().getSelectionModel().setSelectionInterval(aRow,aRow);
              } else {
                // select the last row in the table
                _tpTableModel.getView().getSelectionModel().setSelectionInterval(numRows-1,numRows-1);
              }
            }
            _parent.setNeedSave();
          }
        }
      }
    }
  }

  /**
   * Puts the Commit/Save button in Commit mode
   */
  private void setCommitMode ()
  {
    _btnCommit.setText(_pfxCommit);
  }

  /**
   * Puts the Commit/Save button in Save mode
   */
  private void setSaveMode ()
  {
    _btnCommit.setText(_pfxSave);
  }

  /**
   * Returns true just in case the Commit/Save button is in Commit
   * mode
   */
  private boolean isCommitMode ()
  {
    boolean result = _btnCommit.getText().equals(_pfxCommit);
    log.debug("isCommitMode returns " + result);
    return result;
  }

  /**
   * Handles the New button.  All we do here is deselect all rows in
   * the table, and put the Commit/Save button in Save mode.
   */
  public void doNew ()
  {
    if (_tpTableModel != null)
      _tpTableModel.getView().getSelectionModel().clearSelection();
    setSaveMode();
  }

  /**
   * Handles the TaskEditor "Save" function.  This is invoked when the
   * Commit/Save button is poked in Save mode.
   */
  public void doSaveNew() {
    log.debug("doSaveNew(): entering");
    if (!checkBeforeSaveNew()) {
      log.debug("doSaveNew(): bail on rules-checking");
      return;
    }
    // Create a new SubtaskRequest and insert it into the TaskPlan.
    SubtaskRequest req;
    if (null==(req=createNewSubtaskRequest())) {
      log.debug("doSaveNew(): bail on null subtask request");
      return;
    }

    /* Before adding to TaskPlan, get a list of taskIDs corresponding to
     *  the new task's dependencies.  (Once it's added, if task was inserted
     *  "early in the list", it'll throw off some/all of the Task Numbers.)
     */
    Vector predecessors = new Vector();
    if (null==(predecessors=createListOfPredecessorTaskIDs())) {
      log.debug("doSaveNew(): bail on null predecessors");
      return;
    }

    // Add task to taskplan
    TaskPlan tp = _tpTableModel.getTaskPlan();
    try {
      tp.addTask(req);
    } catch (InvalidTaskException ite) {
      inform("Failure inserting task into task plan: "
	     + ite.getLocalizedMessage() );
      return;
    }

    String tempListIDs = new String("");
    for (int zz=0; zz < _tpTableModel.getRowCount(); zz++) {
      tempListIDs += _tpTableModel.getTask(zz).getId() + ", ";
    }
    log.debug("TPE4P.doSaveNew(): TaskPlan now contains IDs: " + tempListIDs);

    // Dependencies/predecessors are applied AFTER task is inserted in
    // taskplan..
    if (false==assignPredecessors(predecessors, tp, req)) {
      // Remove this half-added task from the TaskPlan..
      try {  tp.removeTask(req);
      } catch (InvalidTaskException ite3) {
        inform("Failure backing-out uncompleted addition of this task: "
	       + ite3.getLocalizedMessage());
      }
      return;  // bail if fail
    }
    _parent.setNeedSave();
  }

  /** Check for incorrect fields before proceeding w/ Save New. */
  public boolean checkBeforeSaveNew() {
    Vector errors = new Vector();      // error collector

    // Task Plan
    if (null==_tpTableModel  ||  null==_tpTableModel.getTaskPlan()) {
      errors.addElement(new ErrorObject(ERRFOCUS_GENERAL,
			 "No Task Plan is present;  Unrecoverable error?"));
      handleErrors(errors);
      return false;
    }
    TaskPlan tp = _tpTableModel.getTaskPlan();

    // taskID:
    // Is taskID populated and unique?  If not, produce an error.
    if (0==_tfTaskID.getText().length()) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_ID,
					"Task ID must be non-empty."));
    }
    else if (null!=tp.getTaskData(_tfTaskID.getText())) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_ID,
					"Task ID must be unique."));
      /** @todo mention which task# it collides w/ ? */
    }

    // taskName:
    // Is taskName populated?  If not, produce an error.
    if (0==_tfTaskName.getText().length()) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_NAME,
					"Task Name must be non-empty."));
    }

    // Dependencies:
    ArrayList preds = checkDependencies(errors);

    // JobInfo
    boolean infoValid = true;
    if (-1==_cbTaskType.getSelectedIndex()) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_JOBINFO,
          "A JobInfo Task Type must be selected."));
      infoValid = false;
    }

    // earliestStart, deadline:
    checkEStartDeadlineDates(errors, preds, infoValid);

    // Comments -- Nothing to test..

    /* At the end, if there are no errors, return true;
     *  else display errors in focusing Dialog and return false.
     */
    return handleErrors(errors);
  }//checkBeforeSaveNew()


  /** If no errors, return true;  If errors, launch errors dialog, then return false. */
  public boolean handleErrors(Vector errors) {
    if (0==errors.size()) {
      return true;
    } else {
      // Display the errors (if any)
        ErrorObjectHandler eH = new ErrorObjectHandler(errors);
        eH.parkUpperRHCorner();  // reposition error window.
        eH.displayErrors(GuiUtil.getFrame(this), this, null, null);
        return false;
    }
  }

  /**
   * Chopped out some common code that creates a new SubtaskRequest
   * from field data.
   * @return a new SubtaskRequest, or null if errors encountered.
   */
  private SubtaskRequest createNewSubtaskRequest() {
    SubtaskRequest req = new SubtaskRequest();
    try {
      req.setId(_tfTaskID.getText());
      req.setTaskName(_tfTaskName.getText());
      // Deps:  done below.
      req.setJobInfo( (JobInfo)_cbmJobInfos.getSelectedItem());
      DateFormat df1 = DateFormat.getDateTimeInstance(DateFormat.SHORT,
						      DateFormat.SHORT);
      df1.setLenient(true);
      if (!_tfEarliestStart.getText().equals(""))
	req.setEarliestTimeToStart(df1.parse(_tfEarliestStart.getText()));
      if (!_tfDeadline.getText().equals(""))
	req.setDeadline(df1.parse(_tfDeadline.getText()));
    } catch (Exception e1) {
      // should never happen
      inform("Error constructing Task from data: " + e1.toString());
      return null;
    }
    req.setTaskComments(_taComments.getText());
    return req;
  }

  /**
   * Common code that creates list of predecessor task IDs from field
   * data.
   * @return a new Vector of preds (might be empty), or null if
   *   errors occurred.
   */
  private Vector createListOfPredecessorTaskIDs() {
    /* Before adding to TaskPlan, get a list of taskIDs corresponding to
     *  the new task's dependencies.  (Once it's added, if task was inserted
     *  "early in the list", it'll throw off some/all of the Task Numbers.)
     */
    Vector predecessors = new Vector();
    if (_tfDeps.getText().length() > 0) {
      StringTokenizer st = new StringTokenizer(_tfDeps.getText(), ",");
      while (st.hasMoreTokens()) {
        String anItem = st.nextToken();
        int item=0;
        try {
          item = Integer.parseInt(anItem);
        } catch (NumberFormatException nfe) {  // should not happen..
          inform("Error parsing dependencies list: "
		 + nfe.getLocalizedMessage());
          return null;  // bail if it failed
        }
        String tempID = _tpTableModel.taskNum2TaskId(item);
        predecessors.addElement(tempID);
      }//while
    }//if preds exist
    return predecessors;
  }

  /**
   * Common code that assigns predecessors to the newly-added SubtaskRequest.
   *  @param predecessors The Vector (possibly empty) of predecessors
   *     for this task.
   *  @param tp The TaskPlan in which the tasks reside.
   *  @param req The SubtaskRequest that needs predecessors added.
   *  @return true if no errors encountered, or false otherwise.
   */
  private boolean assignPredecessors(Vector predecessors,
				     TaskPlan tp,
				     SubtaskRequest req) {
    for (int i=0; i < predecessors.size(); i++) {
      String tempID = (String)predecessors.elementAt(i);
      // Assign deps
      try {
        tp.addSuccessor(tempID, req.getId());
  //        } catch (CircularDependencyException cde) {
  //        } catch (NullPointerException npe) {
      } catch (Exception e2) {
        inform("Failure setting task ID " + tempID
	       + " as this task's predecessor: "
	       + e2.getLocalizedMessage());
        return false;  // bail if it failed
      }
    }//end-if predecessors
    return true;
  }

  /**
   * Handles the TaskEditor "Commit" button.
   * Commit implies replacing the TaskPlanTable's
   * currently-highlighted row (ie. a Task) with the new data.
   */
  public void doCommit() {
    if (!isCommitMode()) {
      doSaveNew();
      return;
    }
    if (!checkBeforeCommit()) {  return;  }
    TaskPlan tp = _tpTableModel.getTaskPlan();
    if (null==tp) {
      return;
    }
    // Create a new SubtaskRequest.
    SubtaskRequest req;
    if (null==(req=createNewSubtaskRequest())) {
      return;
    }

    /* Before adding to TaskPlan, get a list of taskIDs corresponding to
     *  the new task's dependencies.  (Once it's added, if task was inserted
     *  "early in the list", it'll throw off some/all of the Task Numbers.)
     */
    Vector predecessors = new Vector();
    if (null==(predecessors=createListOfPredecessorTaskIDs())) {
      return;
    }

    // Before adding to taskplan (which will upset the TPTable),
    // capture the row# and TaskID of the currently-selected item
    // (which we'll be later deleting.)
    int aRow = _tpTableModel.getView().getSelectedRow();
    String oldTaskID =
	(String)_tpTableModel.getValueAt(aRow, TPTableModel.COL_TASK_ID);

    // Before adding, remove the "old" task, so we don't get taskID collisions.
    // Remove the old task we're replacing.
    SubtaskRequest oldTask=tp.getTaskData(oldTaskID);
    try {
      tp.removeTask(oldTask);
    } catch (InvalidTaskException ite3) {
      inform("Failure backing-out uncompleted addition of this task: "
	     + ite3.getLocalizedMessage());
      return;  // bail if fail
    }

    /** finally we need to make sure that we can save */
    _parent.setNeedSave();

    // Add task to taskplan
    try {
      tp.addTask(req);
    } catch (InvalidTaskException ite) {
      inform("Failure inserting task into task plan: "
	     + ite.getLocalizedMessage() );
      return;
    }

    // Dependencies/predecessors are applied AFTER task is inserted in
    // taskplan..
    if (false==assignPredecessors(predecessors, tp, req)) {
      // Remove this half-added task from the TaskPlan..
      try {  tp.removeTask(req);
      } catch (InvalidTaskException ite3) {
        inform("Failure removing task ID=" + req.getId()
	       + " from task plan: " + ite3.getLocalizedMessage());
      }
      return;  // bail if fail
    }
  }//doCommit()

  /**
   * Does rule checking before it's safe to perform a Commit.
   * Commit implies replacing the TaskPlanTable's
   * currently-highlighted row (ie. a Task) with the new data.
   */
  public boolean checkBeforeCommit() {
    Vector errors = new Vector(); // collection of detected errors

    // Task Plan
    if (null==_tpTableModel  ||  null==_tpTableModel.getTaskPlan()) {
      errors.addElement(new ErrorObject(ERRFOCUS_GENERAL,
			 "No Task Plan is present;  Unrecoverable error?"));
      handleErrors(errors);
      return false;
    }
    TaskPlan tp = _tpTableModel.getTaskPlan();

    // TaskPlan Table:
    // Is a task highlighted in the table?  If not, produce an error.
    int aRow = _tpTableModel.getView().getSelectedRow();
    if (-1==aRow) {
      errors.addElement(new ErrorObject(ERRFOCUS_GENERAL,
              "You must select a Task to replace in the TaskPlan table."));
      handleErrors(errors);
      return false;
    }

    // taskID:
    // Is taskID populated and NON-unique?  If not, produce an error.
    String oldTaskID =
	(String)_tpTableModel.getValueAt(aRow, TPTableModel.COL_TASK_ID);
    SubtaskRequest oldTask = tp.getTaskData(oldTaskID);
    SubtaskRequest newTask;
    if (0==_tfTaskID.getText().length()) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_ID,
					"Task ID must be non-empty."));
    }
    else if (null!=(newTask=tp.getTaskData(_tfTaskID.getText()))  &&
             !oldTaskID.equals(newTask.getId()) ) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_ID,
					"Task ID must be unique."));
      /** @todo mention which task# it collides w/ ? */
    }

    // Dependencies:
    ArrayList preds = checkDependencies(errors);

    // JobInfo
    boolean infoValid = true;
    if (-1==_cbTaskType.getSelectedIndex()) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_JOBINFO,
          "A JobInfo Template must be selected."));
      infoValid = false;
    }

    // earliestStart, deadline:
    checkEStartDeadlineDates(errors, preds, infoValid);

    // Comments -- Nothing to test..

    /* At the end, if there are no errors, return true;
     *  else display errors in focusing Dialog and return false.
     */
    return handleErrors(errors);
  }//checkBeforeCommit()

  /**
   * Common processing of EarliestStart / Deadline date fields checking.
   * Note that legibleTaskDates, dateEStart and dateDeadline are now
   * member variables, and this method uses/sets 'em for others to
   * use..
   * <p>
   * The vector of ErrorObjects is passed in and used, possibly w/
   * more errors added.
   *  @param errors The existing vector of ErrorObjects
   *  @param predecessors an ArrayList of the Task IDs of the
   *    predecessors of this task.
   */
  private void checkEStartDeadlineDates(Vector errors,
					ArrayList predecessors,
					boolean jobInfoValid)
  {
    // Verify that earliestStart and deadline fields are non-empty.
    /*boolean */legibleTaskDates = true;

    // Figure out if we have to default something
    boolean emptyEStart = false;
    boolean emptyLFinish = false;
    if (0==_tfEarliestStart.getText().length()) {
      emptyEStart = true;
    }
    if (0==_tfDeadline.getText().length()) {
      emptyLFinish = true;
    }

    /*Date */dateEStart = null;
    /*Date */dateDeadline = null;
    DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT,
						   DateFormat.SHORT);
    df.setLenient(false);
    String exampleDateString = df.format(AgentTime.getDate());
    String formatBadDateString =
	"{0}: Bad format.  Please format like this: {1}";
    if (!emptyEStart) {
      // Verify that they successfully parse.
      try {
        dateEStart = new Date(df.parse(_tfEarliestStart.getText()).getTime());
      } catch (ParseException pe1) {
        legibleTaskDates = false;
        String badEStartString =
	    MessageFormat.format(formatBadDateString,
				 new Object[]{ "Earliest Start",
					       exampleDateString });
        errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_ESTART,
					  badEStartString));
      }
    }
    if (!emptyLFinish) {
      try {
        dateDeadline = new Date( df.parse(_tfDeadline.getText()).getTime());
      } catch (ParseException pe1) {
        legibleTaskDates = false;
        String badDeadlineString =
	    MessageFormat.format(formatBadDateString,
				 new Object[]{ "Deadline",
					       exampleDateString });
        errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_DEADLINE,
					  badDeadlineString));
      }
    }

    // At this point, we either have legible dates or empty dates.
    // Empty dates are interpreted as requests to fill in defaults.
    if (legibleTaskDates) {
      JobInfo info = null;
      long expDur = 0L;
      if (jobInfoValid) {
	info = (JobInfo)_cbmJobInfos.getSelectedItem();
	expDur = (long)info.getExpectedDuration();
      }
      if (emptyEStart) {
	if (emptyLFinish) {
	  // Fill in both EStart and Deadline
	  TaskPlan tp = _tpTableModel.getTaskPlan();
	  if (predecessors != null && predecessors.size() > 0) {
	    long lastFinish = 0L;
	    for (Iterator preds = predecessors.iterator();
		 preds.hasNext(); ) {
	      SubtaskRequest sr =
		  tp.getTaskData((String)preds.next());
	      Date finish = sr.getDeadline();
	      if (finish.getTime() > lastFinish)
		lastFinish = finish.getTime();
	    }
	    dateEStart = new Date(lastFinish);
	    _tfEarliestStart.setText(df.format(dateEStart));
	  }
	  else if (tp.getPlanStartDate() != null) {
	    // no predecessors, use the plan start date
	    dateEStart = tp.getPlanStartDate();
	    _tfEarliestStart.setText(df.format(dateEStart));
	  }
	  if (info != null && dateEStart != null) {
	    dateDeadline = new Date(dateEStart.getTime() + expDur);
	    _tfDeadline.setText(df.format(dateDeadline));
	  }
	}
	else {
	  // Fill in EStart as Deadline - expected duration
	  if (info != null)
	    dateEStart = new Date(dateDeadline.getTime() - expDur);
	    _tfEarliestStart.setText(df.format(dateEStart));
	}
      }
      else if (emptyLFinish) {
	// Fill in deadline as EStart + expected duration
	if (info != null) {
	  dateDeadline = new Date(dateEStart.getTime() + expDur);
	  _tfDeadline.setText(df.format(dateDeadline));
	}
      }
      else if (!dateEStart.before(dateDeadline)) {
	// Verify that earliestStart < deadline.
        legibleTaskDates = false;
        errors.addElement(new ErrorObject(ERRFOCUS_TP_TASK_DEADLINE,
                           "Earliest Start must occur before Deadline."));
      }
    }
    return;
  }

  /**
   * Common code to check dependencies before
   * saving/committing.
   * @param errors A vector of ErrorObjects, which may be added to..
   * @return an ArrayList of TaskId's of the preceding tasks.
   */
  private ArrayList checkDependencies(Vector errors)
  {
    ArrayList result = new ArrayList();
    // Verify dependencies/predecessors, if any
    if (_tfDeps.getText().length() > 0) {
      StringTokenizer st = new StringTokenizer(_tfDeps.getText(), ",");
      // Parse and inspect each predecessor ...
      while (st.hasMoreTokens()) {
        String anItem = st.nextToken().trim();
        int item=0;
        // Verify that this (comma-separated) item is an integer
        try {
          item = Integer.parseInt(anItem);
        } catch (NumberFormatException nfe) {
          errors.addElement(new ErrorObject(ERRFOCUS_TP_DEPENDENCIES,
     "Dependencies must be a comma-separated list of integer task numbers."));
          break;  // break out of deps checking
        }
        // Verify that this dependency (predecessor) exists & has this
	// task as a successor
        String tempId = _tpTableModel.taskNum2TaskId(item);
        if (0==tempId.length()) {
          String errString =
	      MessageFormat.format("Task number {0} does not exist.",
				   new Object[] { "" + item }  );
          errors.addElement(new ErrorObject(ERRFOCUS_TP_DEPENDENCIES,
					    errString));
        }
	else {
	  log.debug("Adding predecessor ID " + tempId);
	  result.add(tempId);
	}
      }
    }
    return result;
  }

  /** Handle the TaskEditor "Clear" button. */
  public void doClear() {
//    inform("** TPE4P.doClear(): Not implemented yet!!");
    clearAllTaskInputFields();
  }
  /** Handle the TaskEditor "Help" button. */
  public void doHelp() {
    inform("** TPE4P.doHelp(): Not implemented yet!!");
  }

  /**
   * Handles the Plan Summary "Change" button.
   */
  private void doChangePlanInfo ()
  {
    //TaskPlanDetailsEditor editor = new TaskPlanDetailsEditor(
    //                            GuiUtil.getFrame(this), _tpTableModel.getTaskPlan(),
    //                             _parent);
    //editor.displayEditor();
    TaskPlan tp = _tpTableModel.getTaskPlan();
    if (validateChangePlanInfo()) {
      tp.setPlanName(_tfPlanTitle.getText());
      tp.setPlanStartDate(_planStartDate);
    }
  }

  /** Validate the input fields for the Plan Details panel */
  private boolean validateChangePlanInfo()
  {
    Vector errors = new Vector();
    // Plan Title must be non-empty
    if (0==_tfPlanTitle.getText().length()) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_DETAILS_TITLE,
					"The Plan Title cannot be empty."));
    }
    DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT,
						   DateFormat.SHORT);
    df.setLenient(true);

    // The Plan Start Date must parse tolerably as a date.
    try {
      _planStartDate
	  = new Date(df.parse(_tfPlanStartDate.getText()).getTime());
    }
    catch (ParseException pe1) {
      String exampleDateString = df.format(AgentTime.getDate());
      String formatBadDateString
	  = "{0}: Bad format.  Please format like this: {1}";
      String badEStartString
	  = MessageFormat.format(formatBadDateString,
				 new Object[]{"Plan Start Date",
					      exampleDateString });
      errors.addElement(new ErrorObject(ERRFOCUS_TP_DETAILS_ESTART,
					badEStartString));
    }
    // At the end, if there are no errors, return true;
    // else display errors in focusing Dialog and return false.
    log.debug("Plan deadline: "
	      +_tpTableModel.getTaskPlan().getPlanDeadline());
    return handleErrors(errors);
  }

  /**
   * Handles the Plan Summary "Revert" button.
   */
  private void doRevertPlanInfo ()
  {
    inform("Revert button not implemented");
  }

  /**
   * Handles the Plan Summary "Reschedule" button.  This is done by
   * passing the plan through a PlanTWAdjuster with a slack value of
   * 1.0 and a default TaskDurationAdjuster.
   */
  private void doReschedule ()
  {
    TaskPlan tp = _tpTableModel.getTaskPlan();
    // First, we remove all the start and finish times from the
    // individual tasks
    try {
      for (Iterator sts = tp.iterator(); sts.hasNext(); ) {
	SubtaskRequest sr = (SubtaskRequest)sts.next();
	sr.setEarliestTimeToStart(null);
	sr.setDeadline(null);
      }
    }
    catch (InvalidDataException ide) {
      inform("Exception when nulling task times");
      return;
    }
    // Next we pick the start and deadline dates out of their
    // respective fields
    DateFormat df1 = DateFormat.getDateTimeInstance(DateFormat.SHORT,
						    DateFormat.SHORT);
    df1.setLenient(true);
    Date start = null;
    Date deadline = null;
    try {
      start = df1.parse(_tfPlanStartDate.getText());
      if (!_tfPlanDeadline.getText().equals(""))
	// only use deadline field if it's non-empty
	deadline = df1.parse(_tfPlanDeadline.getText());
    }
    catch (ParseException pe) {
      inform("ParseException trying to read plan start or deadline");
      return;
    }

    // Then we run the scheduler across the plan
    TaskBidPlan tbp = new TaskBidPlan(tp);
    PlanTWAdjuster pta
	= PlanTWAdjuster.generate(MagnetProperties.getProperties());
    pta.setTaskDurationAdjuster(new TaskDurationAdjuster());
    // here's where the damage is done...
    if (deadline == null)
      pta.setTimeWindows(tbp, start, 1.0);
    else
      pta.setTimeWindows(tbp, start, deadline);

    if (log.isDebugEnabled()) {
      for (Iterator sts = tp.iterator(); sts.hasNext(); ) {
	SubtaskRequest sr = (SubtaskRequest)sts.next();
	log.debug("Task " + sr.getId()
		  + " start: " + sr.getEarliestTimeToStart()
		  + ", finish: " + sr.getDeadline());
      }
    }
    tp.firePlanDataChanged("Rescheduled");
    recalcSummaryPanel();
    log.debug("Plan deadline: "
	      +_tpTableModel.getTaskPlan().getPlanDeadline());
    _parent.setNeedSave();
  }

  /** Handle the TaskEditor "Submit RFQ" button. */
  public void doSubmitRFQ() {
    if (!checkBeforeSubmitRFQ()) { return; }
    RFQ rfq = null;//new RFQ();
    Collection subtasks = null;
    BidCommand cmd = null;
    PlanStatus ps = null;
    BidManager mgr = null;
    RFQStatus rStatus = null;

    // This will create the BidCommand that is needed to monitor this RFQ
    // This was done based off of the steps that John presented in the
    // Interactive Operation of the MAGNET Customer Agent components

    subtasks = _tpTableModel.getTaskPlan().getTasks();
    AgentStatus _as = AgentStatus.getInstance();
    if ((_as == null) || (_tpTableModel == null) || (subtasks == null)) {
      log.info("Null variables ");
      /** @todo: Do we want to display an error? */
      return;
    }


    // Set the start & end date
    /** @todo:  Anything else we need to make sure to set?? */
    try {
      ps = _as.createPlanStatus(_tpTableModel.getTaskPlan());
      if (ps == null) {
        log.error("Unable to create a Plan Status");
        GuiUtil.inform(_parent, "Unable to create the Plan Status",
                      "Error Creating Plan Status");
        return;
      }
      ps.createSessionStatus("test");
      ps.setPlanStart(_tpTableModel.getTaskPlan().getPlanStartDate());
      ps.setPlanDeadline(_tpTableModel.getTaskPlan().getPlanDeadline());
    } catch (MagnetException ide) {
      inform("Error building RFQ: "+ide.getLocalizedMessage());
      return;
    }

    /** @todo: assuming that cannot create an RFQ if not subtasks */
    if (subtasks == null) return;

    /** @todo:  Need to get Latest Bid Offer Date - now using dateEBidOffer
          should be configurable.  Also, need to get "end of slot" value -
          this is also right now going to be dateEBidOffer - this should be the
          same as the latest bid offer date */
    SessionStatus ss = GuiUtil.getSessionStatus(_as,
                                    _tpTableModel.getTaskPlan());
    if (ss == null) return; /** @todo:: Do I want to do error display? */

    mgr = new BidManager(ss, new Properties(), false);
    MagnetClientMain.setBidManager(mgr);

    cmd = new BidCommand(subtasks, dateBidDeadline, dateEBidOffer,
                        dateEBidOffer, dateEBidOffer);

    // The BidManager's composeRFQ does steps 3.1-3.4 - I just need to call
    // that and then create the RFQ Status
    rfq = mgr.composeRFQ(cmd);
    try {
      rStatus = mgr.createRFQStatus(rfq);
      rStatus.acquireBids(); // this sends out the RFQ
      new RFQDialog(GuiUtil.getFrame(this), rStatus);
    } catch (MagnetException ide) {
      inform("Error Creating RFQ Status: "+ide.getLocalizedMessage());
      return;
    }

    setEditable(false);  // don't allow edit once it is an RFQ
    _parent.invokeEditor(MagnetClientMain.EDITOR_TASKBIDPLAN);
    /** finally we need to make sure that we can save */
    _parent.setNeedSave();
    return;
  }

  /** Do rule checking before it's safe to perform Submit RFQ.
   */
  public boolean checkBeforeSubmitRFQ() {
    Vector errors = new Vector();    // Keep adding errors to the error object.

    // Task Plan
    if (null==_tpTableModel  ||  null==_tpTableModel.getTaskPlan()) {
      errors.addElement(new ErrorObject(ERRFOCUS_GENERAL,
			 "No Task Plan is present;  Unrecoverable error?"));
      handleErrors(errors);
      return false;
    }
    TaskPlan tp = _tpTableModel.getTaskPlan();

    // Verify that the EarliestBidConsider, EarliestBidOffer and
    // BidDeadline fields are non-empty.
    boolean legibleBidDates = true;
//      if (0==_tfEarliestBidConsider.getText().length()) {
//        legibleBidDates = false;
//        errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_EBID_CONSIDER,
//                           "RFQ: Earliest Bid Consider must be non-empty."));
//      }
    if (0==_tfEarliestBidOffer.getText().length()) {
      legibleBidDates = false;
      errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_EBID_OFFER,
                         "RFQ: Earliest Bid Offer must be non-empty."));
    }
    if (0==_tfBidDeadline.getText().length()) {
      legibleBidDates = false;
      errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_BID_DEADLINE,
                                  "RFQ: Bid Deadline must be non-empty."));
    }
    dateEBidConsider = null;
    dateEBidOffer = null;
    dateBidDeadline = null;
    String exampleDateString
	= DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT)
	  .format(AgentTime.getDate());  // example string
    String formatBadDateString
	= "{0}: Bad format.  Please format like this: {1}";

    if (legibleBidDates) {
      // Verify that they successfully parse.
//        try {
//          DateFormat df1 =
//  	    DateFormat.getDateTimeInstance(DateFormat.SHORT,
//  					   DateFormat.SHORT);
//          df1.setLenient(false);
//          dateEBidConsider
//  	    = new Date(df1.parse(_tfEarliestBidConsider.getText()).getTime());
//        } catch (ParseException pe1) {
//          legibleBidDates = false;
//          String badEBidConsiderString
//  	    = MessageFormat.format(formatBadDateString,
//  				   new Object[]{ "Earliest Bid Consider",
//  						 exampleDateString }  );
//          errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_EBID_CONSIDER,
//  					  badEBidConsiderString));
//        }
      try {
        DateFormat df2 =
	    DateFormat.getDateTimeInstance(DateFormat.SHORT,
					   DateFormat.SHORT);
        df2.setLenient(true);
        dateEBidOffer
	    = new Date(df2.parse(_tfEarliestBidOffer.getText()).getTime());
      } catch (ParseException pe1) {
        legibleBidDates = false;
        String badEBidOfferString =
	    MessageFormat.format(formatBadDateString,
				 new Object[]{ "Earliest Bid Offer",
					       exampleDateString });
        errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_EBID_OFFER,
					  badEBidOfferString));
      }
      try {
        DateFormat df3 =
	    DateFormat.getDateTimeInstance(DateFormat.SHORT,
					   DateFormat.SHORT);
        df3.setLenient(true);
        dateBidDeadline =
	    new Date( df3.parse(_tfBidDeadline.getText()).getTime());
      } catch (ParseException pe1) {
        legibleBidDates = false;
        String badBidDeadlineString =
	    MessageFormat.format(formatBadDateString,
				 new Object[]{ "Bid Deadline",
					       exampleDateString });
        errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_BID_DEADLINE,
					  badBidDeadlineString));
      }
    }

    // @todo Are there any additional temporal rules governing these 3
    // fields?


    // etc...

    /* At the end, if there are no errors, return true;
     *  else display errors in focusing Dialog and return false.
     */

    return handleErrors(errors);
  }

  /** Handle the TaskEditor "Submit RFQ w/ Defaults" button. */
  public void doSubmitRFQWithDefaults()
  {
    // Start by validating the time interval fields.
    Vector errors = new Vector();    // Keep adding errors to the error object.
    long bidInterval = 0L;
    long evalInterval = 0L;

    // Task Plan
    if (null == _tpTableModel
	|| null == _tpTableModel.getTaskPlan()
	|| 0 == _tpTableModel.getTaskPlan().size()) {
      errors.addElement(new ErrorObject(ERRFOCUS_GENERAL,
			 "No Task Plan is present;  Unrecoverable error?"));
      handleErrors(errors);
      return;
    }
    // make sure the two interval fields are non-empty and have valid
    // intervals.
    String bidIntervalS = _tfBidDeadline.getText();
    if (bidIntervalS.length() == 0) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_BID_DEADLINE,
                         "RFQ: Bid Interval must be non-empty."));
    }
    else {
      try {
	bidInterval = GuiUtil.string2interval(bidIntervalS);
      }
      catch (ParseException pe) {
	errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_BID_DEADLINE,
                           "RFQ: Can't parse Bid Interval."));
      }
    }
    String evalIntervalS = _tfEarliestBidOffer.getText();
    if (evalIntervalS.length() == 0) {
      errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_EBID_OFFER,
                         "RFQ: Eval Time must be non-empty."));
    }
    else {
      try {
	evalInterval = GuiUtil.string2interval(evalIntervalS);
      }
      catch (ParseException pe) {
	errors.addElement(new ErrorObject(ERRFOCUS_TP_RFQ_EBID_OFFER,
                           "RFQ: Can't parse Eval Time."));
      }
    }

    if (!handleErrors(errors))
      return;

    /** @todo: assuming that cannot create an RFQ if not subtasks */
    Collection subtasks = _tpTableModel.getTaskPlan().getTasks();
    if (subtasks == null) return;

    RFQ rfq = null;//new RFQ();

    // fill in default timings
    Date now = AgentTime.getDate();
    dateBidDeadline = new Date(now.getTime() + bidInterval);
    dateEBidConsider = new Date(dateBidDeadline.getTime()
				+ getDefaultEarlyConsider());
    dateEBidOffer = new Date(dateEBidConsider.getTime() + evalInterval);

    BidCommand cmd = null;
    PlanStatus ps = null;
    BidManager mgr = null;
    RFQStatus rStatus = null;

    // This will create the BidCommand that is needed to monitor this RFQ
    // This was done based off of the steps that John presented in the
    // Interactive Operation of the MAGNET Customer Agent components

    AgentStatus _as = AgentStatus.getInstance();
    if ((_as == null) || (_tpTableModel == null) || (subtasks == null)) {
     log.info("Null variables ");
      return;
    }

    // Set the start & end date
    /** @todo:  Anything else we need to make sure to set?? */
    try {
      ps = _as.createPlanStatus(_tpTableModel.getTaskPlan());
      if (ps == null) {
        log.error("Unable to create a Plan Status");
        GuiUtil.inform(_parent, "Error creating the plan status",
                        "Error creating Plan Status");
        return;
      }
      ps.createSessionStatus("test");
      /** @todo:  This needs to be hooked to a server session eventually */
      ps.setPlanStart(_tpTableModel.getTaskPlan().getPlanStartDate());
      ps.setPlanDeadline(_tpTableModel.getTaskPlan().getPlanDeadline());
    } catch (MagnetException ide) {
      inform("Error building RFQ: " + ide.getLocalizedMessage());
      return;
    }

    /** @todo:  Need to get Latest Bid Offer Date - now using dateEBidOffer
    // should be configurable.  Also, need to get "end of slot" value -
    // this is also right now going to be dateEBidOffer - this should be the
    // same as the latest bid offer date */
    SessionStatus ss = GuiUtil.getSessionStatus(_as,
                                    _tpTableModel.getTaskPlan());
    if (ss == null) return; /** @todo: Do I want to do error display? */

    mgr = new BidManager(ss,
			 MagnetProperties.getProperties(),
			 false);
    MagnetClientMain.setBidManager(mgr);

    cmd = new BidCommand(subtasks, dateBidDeadline, dateEBidOffer,
			 dateEBidOffer, dateEBidOffer, false);

    // The BidManager's composeRFQ does steps 3.1-3.4 - I just need to call
    // that and then create the RFQ Status
    //try {
    //  for (Iterator sts = subtasks.iterator(); sts.hasNext(); ) {
    //	SubtaskRequest sr = (SubtaskRequest)sts.next();
    //	sr.setEarliestTimeToStart(null);
    //	sr.setDeadline(null);
    //  }
    //}
    //catch (InvalidDataException ide) {
    //  inform("Exception when nulling task times");
    //  return;
    //}
    rfq = mgr.composeRFQ(cmd);
    try {
      rStatus = mgr.createRFQStatus(rfq);
      rStatus.acquireBids(); // this sends out the RFQ
      new RFQDialog(GuiUtil.getFrame(this),rStatus);
    } catch (MagnetException ide) {
      inform("Error Creating RFQ Status: "+ide.getMessage());
      return;
    }
    // now disable the ability to edit this plan any further
    setEditable(false);
    _parent.invokeEditor(MagnetClientMain.EDITOR_TASKBIDPLAN);
    _parent.setNeedSave();
    return;
  }


  /**
   * Handles the errors by determining what to do for each error
   * @param errorObject the error to resolve
   */
  public void setErrorFocus(ErrorObject errorObject) {
    if (null==errorObject) {  return;  }
    switch (errorObject.getField()) {
      case ERRFOCUS_GENERAL:
          this.requestFocus();    // ???  no focus necessary !!
          break;

      case ERRFOCUS_TP_TASK_ID:
          _tfTaskID.requestFocus();
          break;

      case ERRFOCUS_TP_TASK_NAME:
          _tfTaskName.requestFocus();
          break;

      case ERRFOCUS_TP_DEPENDENCIES:
          _tfDeps.requestFocus();
          break;

      case ERRFOCUS_TP_TASK_ESTART:
          _tfEarliestStart.requestFocus();
          break;

      case ERRFOCUS_TP_TASK_DEADLINE:
          _tfDeadline.requestFocus();
          break;

      case ERRFOCUS_TP_TASK_JOBINFO:
          _cbTaskType.requestFocus();
          break;

//        case ERRFOCUS_TP_RFQ_EBID_CONSIDER:
//            _tfEarliestBidConsider.requestFocus();
//            break;

      case ERRFOCUS_TP_RFQ_EBID_OFFER:
          _tfEarliestBidOffer.requestFocus();
          break;

      case ERRFOCUS_TP_RFQ_BID_DEADLINE:
          _tfBidDeadline.requestFocus();
          break;

      case ERRFOCUS_TP_DETAILS_TITLE:
          _tfPlanTitle.requestFocus();
          break;

      case ERRFOCUS_TP_DETAILS_ESTART:
          _tfPlanStartDate.requestFocus();
          break;

      //etc..

      default:
          // DEFAULT MESSAGE IF ERROR NOT HANDLED
          inform("The error you selected is not currently being handled.");
    }//switch
    return;
  }

  /**
   * Returns the default bid deadline interval, after ensuring that
   * the property has been read.
   */
  private long getDefaultBidDeadline ()
  {
    ensureDefaults();
    return _defaultBidDeadline;
  }

  /**
   * Returns the default early consider interval, after ensuring that
   * the property has been read.
   */
  private long getDefaultEarlyConsider ()
  {
    ensureDefaults();
    return _defaultEarlyConsider;
  }

  /**
   * Returns the default early offer interval, after ensuring that
   * the property has been read.
   */
  private long getDefaultEarlyOffer ()
  {
    ensureDefaults();
    return _defaultEarlyOffer;
  }

  /**
   * Read properties for default bid deadline, early consider, and
   * early offer intervals.
   */
  private void ensureDefaults ()
  {
    if (_defaultsRead)
      return;
    String prop;
    prop = MagnetProperties.getProperty("BidCycle.BidDeadline");
    if (prop != null) {
      try {
	_defaultBidDeadline = Long.parseLong(prop);
	log.info("BidCycle.BidDeadline: " + _defaultBidDeadline);
      }
      catch (NumberFormatException nfe) {
	log.error("Property parse error for BidCycle.BidDeadline: " + prop);
      }
    }
    prop = MagnetProperties.getProperty("BidCycle.EarlyConsider");
    if (prop != null) {
      try {
	_defaultEarlyConsider = Long.parseLong(prop);
	log.info("BidCycle.EarlyConsider: " + _defaultEarlyConsider);
      }
      catch (NumberFormatException nfe) {
	log.error("Property parse error for BidCycle.EarlyConsider: " + prop);
      }
    }
    prop = MagnetProperties.getProperty("BidCycle.EarlyOffer");
    if (prop != null) {
      try {
	_defaultEarlyOffer = Long.parseLong(prop);
	log.info("BidCycle.EarlyOffer: " + _defaultEarlyOffer);
      }
      catch (NumberFormatException nfe) {
	log.error("Property parse error for BidCycle.EarlyOffer: " + prop);
      }
    }
    _defaultsRead = true;
  }


  /** setEditable(val) -- this method will either allow all of plan editing or
   *  disallow all of plan editing.  If val is true - it will be allowed, if val
   *  is false it will not be editable
   */
  public void setEditable(boolean val) {

    _btnChangeJobInfo.setEnabled(val);
    _btnChangePlanInfo.setEnabled(val);
    _btnRevertPlanInfo.setEnabled(val);
    _btnReschedule.setEnabled(val);
    _btnCommit.setEnabled(val);
    _btnDelTask.setEnabled(val);
    _btnSaveNew.setEnabled(val);
    //_btnSubmitRFQ.setEnabled(val);
    _btnSubmitRFQWithDefaults.setEnabled(val);
    _cbTaskType.setEditable(val);
    _spComments.setEnabled(val);
    _tfBidDeadline.setEditable(val);
    _tfDeadline.setEditable(val);
    _tfDeps.setEditable(val);
    //_tfEarliestBidConsider.setEditable(val);
    _tfEarliestBidOffer.setEditable(val);
    _tfEarliestStart.setEditable(val);
    _tfTaskID.setEditable(val);
    _tfTaskID.setEditable(val);
    _tfTaskName.setEditable(val);
    _tfPlanTitle.setEditable(val);
    _tfPlanStartDate.setEditable(val);
    _tfPlanDeadline.setEditable(val);
    return;
  } // end of setEditable


  public static void main(String[] args) {
    TaskPlanEditor4Panel taskPlanEditor4Panel1 = new TaskPlanEditor4Panel();
  }
}
