/*------------------------------------------------------------------------
 * Magnet search utility components
 * $Id: MinimumIsBest.java,v 1.2.2.1 2002/05/05 19:42:05 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Thu Jan 24 18:24:01 2002  John Collins  #0000174
 *   Implement java.util.Comparator instead of util.search.Comparator
 * 
 * Thu Dec 31 10:13:07 1998  John E. Collins
 *   Initial Entry
 */

package edu.umn.magnet.util.search;

/**
 * This is a Comparator that sorts nodes by increasing value. Note
 * that this comparator is <strong>NOT</strong> consistent with equals
 * - two different nodes with the same value will compare equal.
 * @author John Collins
 */
public class MinimumIsBest implements java.util.Comparator
{
  public static final double SLOP = 0.0000001;

  /**
   * Default constructor.
   */
  public MinimumIsBest ()
  {
    super();
  }

  /**
   * Returns <0 just in case node2 has a greater value than node1, >0
   * when node2.value() < node1.value(), and 0 when node2.value() ==
   * node1.value().
   */
  public int compare (Object node1, Object node2)
  {
    double result = (((SearchNode)node1).value()
		     - ((SearchNode)node2).value());
    if (result < -SLOP)
      return -1;
    else if (result > SLOP)
      return 1;
    else
      return 0;
  }

  /**
   * Returns true just in case compare() would return 0.
   */
  public boolean equals (Object node1, Object node2)
  {
    return (compare(node1, node2) == 0);
  }
}
