;;; MAGNET script: hook definition
;;; $Id: hooks.scm,v 1.3 2001/05/03 23:00:54 jcollins Exp $
;;; -------------------------------------------------------------------
;;; Revision History:
;;; 
;;; Sun Apr 15 14:57:05 2001  John Collins
;;;   Print out hook name when hook not found
;;; 
;;; Sun Mar 18 14:32:00 2001  John Collins
;;;   Initial Entry
;;;
;;; -------------------------------------------------------------------
;;; This is the basic scheme setup for experimenting with MAGNET bid
;;; evaluation .
;;; 

;;; This is the hook list.  A hook is a list consisting of a name and
;;; a list of procedures.  When the hook is called, each procedure is
;;; invoked in sequence.
(define hook-map '())

;;; Here we add the new proc to the front of the named hook list
(define (add-hook name proc)
  (let ((hook (assoc name hook-map)))
    (if (not hook)
	(begin
	  (set! hook (cons name '()))
	  (set! hook-map (cons hook hook-map))))
    ;;(.println System.out$ (.concat "add-hook" name))
    (set-cdr! hook (cons proc (cdr hook)))))

;;; Here's how we run a hook
(define (run-hook name arg)
  (letrec ((hooks (assoc (string->symbol name) hook-map))
	   (run-next
	    (lambda (hook-list)
	      (if (null? hook-list)
		  '()
		  (begin
		    ((car hook-list) arg)
		    (run-next (cdr hook-list)))))))
    (if hooks
	(run-next (cdr hooks))
	(.println System.out$ (string-append "Hook " name " not found")))))
