/*------------------------------------------------------------------------
 * Magnet Utilities
 * $Id: Collector.java,v 1.1.2.2 2002/05/05 19:42:05 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 *   Wed Nov  4 14:34:25 CST 1998 Max Tsvetovat
 *   Initial entry
 */

package edu.umn.magnet.util.stats;

/**
 * Collects numeric observations, keeps summary statistics.
 */
public interface Collector {

   /**
    Returns a string representation of the summary data, giving observation
    count, mean, and standard deviation.
    */
    public String toString();

    public String toMatlabCode(String varName);

    /**
    Returns the number of observations made so far.
    */
    public long getObservationCount();

    /**
    Adds an observation, type = double
    */
    public void newObservation(double value);

    /**
    Adds a weighted observation (v, r) where v is the value and
    r is the repeat count.
    */
    public void newObservation(double value, long repeatCount);

    /**
    Adds an observation, type = Number
    */
    public void newObservation(Number value);

    /**
    Re-initializes this DataCollector, removing all observations.
    */
    public void reInitialize();

    public boolean isStatOnly();

}

