/*------------------------------------------------------------------------
 * Magnet xx components
 * $Id: RollAvgDataCollector.java,v 1.1.2.2 2002/05/05 19:42:06 ashutosh Exp $
 * Copyright (c) 1998 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 *
 * Wed Nov  4 14:34:25 CST 1998 Max Tsveotvat
 *  Now implements Collector
 *
 * Thu Oct  1 18:13:30 CDT 1998
 *   Initial Entry
 */

package edu.umn.magnet.util.stats;

/**
 * Keeps rolling average of observations
 * @author Max Tsvetovat
 */
public class RollAvgDataCollector extends DataCollector
{
  double[] Observations;
  int current=0;
  boolean full;
  int size;

  public RollAvgDataCollector() {
    super();
    //Initialize to default size
    Observations=new double[10];
  }  

  public RollAvgDataCollector(int size) {
    super();
    this.size=size;
    Observations=new double[size];
  }

  public void newObservation(double value) {
    //if the array hasn't ever been filled up yet, just plop the value in.
    if (!full) 
      super.newObservation(value);
    else {
      //take the value at the counter and subtract from the sums
      m_sum -= Observations[current];
      m_sumsq -= Observations[current] * Observations[current];
      
      //replace the Observations[current] with the value that was passed in
      Observations[current]=value;

      //Now add in the the value 
      m_sum -= Observations[current];
      m_sumsq -= Observations[current] * Observations[current];
    }
    
    //increment the counter, if overflow - wrap around
    current++;
    if (current == size) {
      full=true;
      current=0;
    }
  }

  public void newObservation(double value, long repeatCount) {
    for (long i=0; i<repeatCount; i++) {
      newObservation(value);
    }
  }

  public void newObservation(long value) {
    newObservation((double)value);
  }

  public void newObservation(int value) {
    newObservation((double)value);
  }

  public void newObservation(Number value) {
    newObservation(value.doubleValue());
  }


    

	
    
}
