/*---------------------------------------------------------------------
 * Title:        Magnet Server
 * Version:      $Id: Timer.java,v 1.1 2001/04/09 15:58:26 jcollins Exp $
 * Copyright:    Copyright (c) 1998
 * Author:       John Collins
 * Company:      University of Minnesota
 * Description:  Magnet server component: timers
 *---------------------------------------------------------------------
 * Revision History:
 * 
 * Sat Jun 27 06:38:52 1998  John E. Collins
 *   Move to tools package
 */

package edu.umn.magnet.util.time;

import java.util.Date;

/**
 * This is a Timer that executes its action at a specific time in the
 * future.
 * @author John Collins
 */
public class Timer extends AbstractTimer
{
  Date m_deadline = null;

  /**
   * Creates and starts a new Timer. At the appointed time, the
   * <tt>action()</tt> method of the TimerAction object will be invoked.
   * Note that <tt>delay</tt> is actually the minimum time before which
   * <tt>action</tt> will take place. The actual time is dependent on the
   * Java scheduler and other thread activity.
   * @param  time    the time at which the <tt>action</tt> is to take
   *     place, according to the AgentTime clock.
   * @param action   a TimerAction instance whose <tt>action()</tt> method
   *     is invoked at the end of the delay period.
   */
  public Timer(Date time, TimerAction action)
  {
    super();
    m_deadline = time;
    this.setAction(action);
    this.start();
  }
  
  /**
   * When you create one of these, it runs out its time and then
   * triggers its action.  The actual delay is handled by the
   * delay method, which may be overridden in subclasses.
   */
  protected void delay () throws InterruptedException
  {
    AgentTime.waitUntil(m_deadline);
  }
}
