/*------------------------------------------------------------------------
 * MAGNET unit test
 * $Id: BFSearchQueueTest.java,v 1.4.2.1 2002/05/05 19:42:06 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Fri Jan 25 15:55:22 2002  John Collins  #0000174
 *   Test situations with multiple node having equal values.
 * 
 * Sun Nov 19 13:51:10 2000  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.util.search;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.ArrayList;

/**
 * This is a tester for edu.umn.magnet.client.Something
 * @author John Collins
 */
public class BFSearchQueueTest extends TestCase
{
  static final double SLOP = 0.000001;
  
  public BFSearchQueueTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(BFSearchQueueTest.class);
  }
  
  public void testFirst ()
  {

  }

  // ------------ data and setups --------------
  private SearchQueue sq = null;

  private void setup1 ()
  {
    sq = SearchQueue.create(SearchQueue.BEST_FIRST);
  }
  
  public void testInit ()
  {
    setup1();
    assert("Correct class", sq instanceof BFSearchQueue);
    assertEquals("Correct nodeCount", 0, sq.nodeCount());
    assertEquals("Correct size", 0, sq.size());
    assert("Correct default first value",
	   (Math.abs(0.0 - sq.firstValue()) < SLOP));
    assert("Correct default last value",
	   (Math.abs(0.0 - sq.lastValue()) < SLOP));
    assert("Initially empty", sq.isEmpty());
  }

  // --------------- Best first ---------------

  public void testBestFirstA ()
  {
    setup1();
    Node n1 = new Node(100.0);
    sq.insert(n1);
    //System.out.println(sq.toString());
    assertEquals("Correct count 1", 1, sq.size());
    assertEquals("Correct first element", n1, sq.select());
    assert("Not empty", !sq.isEmpty());
    assertEquals("Correct extract", n1, sq.extract());
    assertEquals("Correct count 2", 0, sq.size());
    assert("Finally empty", sq.isEmpty());
  }

  public void testBestFirstB ()
  {
    setup1();
    Node n1 = new Node(100.0);
    sq.insert(n1);
    Node n2 = new Node(200.0);
    sq.insert(n2);
    Node n3 = new Node(150.0);
    sq.insert(n3);
    //System.out.println(sq.toString());
    assertEquals("Correct count 1", 3, sq.size());
    assertEquals("Correct first element", n1, sq.select());
    assert("Correct first value",
	   (Math.abs(100.0 - sq.select().value()) < SLOP));
    assert("Correct last value",
	   (Math.abs(200.0 - sq.lastValue()) < SLOP));
    assertEquals("Correct extract", n1, sq.extract());
    assertEquals("Correct count 2", 2, sq.size());
    assertEquals("Correct first element b", n3, sq.select());
    assertEquals("Correct extract", n3, sq.extract());
    assertEquals("Correct count 3", 1, sq.size());
  }

  public void testBestFirstC ()
  {
    setup1();
    Node n1 = new Node(100.0);
    sq.insert(n1);
    Node n2 = new Node(200.0);
    sq.insert(n2);
    Node n3 = new Node(150.0);
    sq.insert(n3);
    Node n4 = new Node(99.0);
    sq.insert(n4);
    Node n5 = new Node(201.0);
    sq.insert(n5);
    assertEquals("Correct count 1", 5, sq.size());
    assertEquals("Correct first element", n4, sq.select());
    assert("Correct first value",
	   (Math.abs(99.0 - sq.select().value()) < SLOP));
    assert("Correct last value",
	   (Math.abs(201.0 - sq.lastValue()) < SLOP));
  }

  public void testBestFirstMax ()
  {
    setup1();
    sq.setMaxSize(20);
    assertEquals("Correct maxSize", 20, sq.getMaxSize());

    for (int i = 0; i < 10; i++) {
      sq.insert(new Node(((double)(i + 1) * 10.0)));
    } // end of for ()
    assertEquals("Correct size a", 10, sq.size());
    assert("Correct first value a",
	   (Math.abs(10.0 - sq.select().value()) < SLOP));
    assert("Correct lastValue a",
	   (Math.abs(100.0 - sq.lastValue()) < SLOP));

    for (int i = 0; i < 10; i++) {
      sq.insert(new Node((double)(i + 1) * 10.0 + 5.0));
    } // end of for ()
    assertEquals("Correct size b", 20, sq.size());
    assert("Correct first value b",
	   (Math.abs(10.0 - sq.select().value()) < SLOP));
    assert("Correct lastValue b",
	   (Math.abs(105.0 - sq.lastValue()) < SLOP));

    // now we create the overflow - add three nodes to overcome the
    // tolerance (tolerance is 10% of max size)
    sq.insert(new Node(21.0));
    sq.insert(new Node(32.0));
    sq.insert(new Node(53.0));
    // System.out.println(sq.toString());
    assertEquals("Correct size c", 20, sq.size());
    assert("Correct first value c",
	   (Math.abs(10.0 - sq.select().value()) < SLOP));
    assert("Correct firstValue c",
	   (Math.abs(10.0 - sq.firstValue()) < SLOP));
    assert("Correct lastValue c",
	   (Math.abs(90.0 - sq.lastValue()) < SLOP));
  }

  public void testTBP ()
  {
    setup1();
    sq.setMaxSize(50);
    QueueTest tester = new QueueTest();
    for (int i = 0; i < 30; i++) {
      sq.insert(new Node(((double)(i + 1) * 10.0)));
    } // end of for ()
    tester.m_target = 142.0;
    SearchNode node = sq.selectByPredicate(tester);
    //System.out.println(node.value());
    assert("Correct node found", 
	   (Math.abs(140.0 - node.value()) < SLOP));
  }

  // make the top bucket split
  public void testSplit1 ()
  {
    setup1();
    sq.setMaxSize(1000);
    for (double i = 1.0; i < 300.0; i += 2.0) {
      sq.insert(new Node(i));
    }
    assertEquals("150 entries", 150, sq.size());
    //System.out.println(sq);
    for (double i = 2.0; i < 301.0; i += 2.0) {
      sq.insert(new Node(i));
    }
    assertEquals("300 entries", 300, sq.size());
    //System.out.println(sq);
  }

  // stress the trim function
  public void testSplitTrim ()
  {
    setup1();
    sq.setMaxSize(300);
    for (double i = 1.0; i < 300.0; i += 2.0) {
      sq.insert(new Node(i));
    }
    assertEquals("150 entries", 150, sq.size());
    for (double i = 2.0; i < 301.0; i += 2.0) {
      sq.insert(new Node(i));
    }
    assertEquals("300 entries", 300, sq.size());
    for (double i = 1.5; i < 150.0; i += 1.0) {
      sq.insert(new Node(i));
    }
    assert("max 330 entries", (sq.size() <= 330));
    //System.out.println(sq);
    assert("Max value < 187", (sq.lastValue() < 187.0));
  }

  // test limits of selectByPredicate
  public void testSBP ()
  {
    setup1();
    assertEquals("initial size = 0", 0, sq.size());
    QueueTest tester = new QueueTest();
    tester.m_target = 42.0;
    SearchNode node = sq.selectByPredicate(tester);
    assertEquals("SBP returns null", null, node);
    Node n1 = new Node(100.0);
    sq.insert(n1);
    node = sq.selectByPredicate(tester);
    assertEquals("SBP still returns null", null, node);
    Node n2 = new Node(200.0);
    sq.insert(n2);
    tester.m_target = 142.0;
    node = sq.selectByPredicate(tester);
    assertEquals("SBP now returns n1", n1, node);
  }

  // test node removal
  public void testRemove ()
  {
    setup1();
    assertEquals("initial size = 0", 0, sq.size());
    Node n1 = new Node(100.0);
    sq.insert(n1);
    Node n2 = new Node(200.0);
    sq.insert(n2);
    QueueTest tester = new QueueTest();
    tester.m_target = 142.0;
    SearchNode node = sq.selectByPredicate(tester);
    assertEquals("SBP returns n1", n1, node);
    assertEquals("2 nodes", 2, sq.size());
    sq.remove(n1);
    assertEquals("1 node left", 1, sq.size());
    assertEquals("remaining node is n2", n2, sq.select());
    node = sq.selectByPredicate(tester);
    assertEquals("SBP now returns null", null, node);
    sq.remove(n2);
    assertEquals("0 nodes left", 0, sq.size());
    assert("Queue empty", sq.isEmpty());
    node = sq.selectByPredicate(tester);
    assertEquals("SBP returns null for empty queue", null, node);
  }

  // Create some equal-value nodes
  public void testEBucket1 ()
  {
    setup1();
    sq.setMaxSize(30);
    assertEquals("Correct maxSize", 30, sq.getMaxSize());

    for (int i = 0; i < 10; i++) {
      sq.insert(new Node(((double)(i + 1) * 10.0)));
    } // end of for ()
    assertEquals("Correct size a", 10, sq.size());
    assert("Correct first value a",
	   (Math.abs(10.0 - sq.select().value()) < SLOP));
    assert("Correct lastValue a",
	   (Math.abs(100.0 - sq.lastValue()) < SLOP));

    for (int i = 0; i < 10; i++) {
      sq.insert(new Node((double)(i + 1) * 10.0 + 5.0));
    } // end of for ()
    assertEquals("Correct size b", 20, sq.size());
    assert("Correct first value b",
	   (Math.abs(10.0 - sq.select().value()) < SLOP));
    assert("Correct lastValue b",
	   (Math.abs(105.0 - sq.lastValue()) < SLOP));

    // Double every other value...
    for (int i = 0; i < 10; i++) {
      sq.insert(new Node((double)(i + 1) * 10.0 + 5.0));
    } // end of for ()
    assertEquals("Correct size b", 30, sq.size());
    assert("Correct first value b",
	   (Math.abs(10.0 - sq.select().value()) < SLOP));
    assert("Correct lastValue b",
	   (Math.abs(105.0 - sq.lastValue()) < SLOP));

    // now we create the overflow - add three nodes to overcome the
    // tolerance (tolerance is 10% of max size)
    sq.insert(new Node(21.0));
    sq.insert(new Node(32.0));
    sq.insert(new Node(53.0));
    sq.insert(new Node(75.0));
    // System.out.println(sq.toString());
    assertEquals("Correct size c", 30, sq.size());
    assert("Correct firstValue c",
	   (Math.abs(10.0 - sq.firstValue()) < SLOP));
    assert("Correct lastValue c",
	   (Math.abs(95.0 - sq.lastValue()) < SLOP));
  }

  // test random results of selectByPredicate
  public void testRandomSBP ()
  {
    setup1();
    QueueTest tester = new QueueTest();
    tester.m_target = 42.0;
    Node n1 = new Node(100.0);
    Node n2 = new Node(20.0);
    sq.insert(n1);
    sq.insert(n2);
    SearchNode node = sq.selectByPredicate(tester);
    assertEquals("SBP returns n2", n2, node);

    ArrayList nodes = new ArrayList(10);
    int[] counts = new int[10];
    for (int i = 0; i < 10; i++) {
      Node nn = new Node(33.0);
      sq.insert(nn);
      nodes.add(nn);
      counts[i] = 0;
    }
    for (int i = 0; i < 100; i++) {
      node = sq.selectByPredicate(tester);
      int j = nodes.indexOf(node);
      if (j < 0)
	fail("Failed to locate node");
      counts[j] += 1;
    }
    StringBuffer sb = new StringBuffer();
    sb.append("Counts: (");
    String sep = "";
    for (int i = 0; i < nodes.size(); i++) {
      sb.append(sep).append(Integer.toString(counts[i]));
      sep = ", ";
      if (counts[i] == 0)
	fail("count = 0");
    }
    sb.append(")");
    System.out.println(sb.toString());
  }

  // ----------------- local SearchNode class -------------------
  class Node implements SearchNode
  {
    private double value = 0.0;

    protected Node (double val)
    {
      super();
      value = val;
    }

    public double value ()
    {
      return value;
    }
  }

  // ============ Tester class for search queue extraction =============
  class QueueTest implements Tester
  {
    // you have to set the target to the value you are testing for
    // before calling the test method.
    double m_target = 0.0;

    public boolean test (edu.umn.magnet.util.search.SearchNode node) {
      return node.value() < m_target;
    }
  }
}
