/*------------------------------------------------------------------------
 * MAGNET unit test
 * $Id: AgentTimeTest.java,v 1.1.2.1 2002/05/05 19:42:07 ashutosh Exp $
 * Copyright (c) 2000 Regents of the University of Minnesota.
 * All Rights Reserved by the Regents of the University of Minnesota.
 * This file contains Original Code and/or Modifications of Original
 * Code as defined in the MAGNET Public Source License, which is
 * available at http://www.cs.umn.edu/magnet/source_license_v1.txt.
 * You may not use this file except in compliance with the License.
 *------------------------------------------------------------------------
 * Revision History:
 * 
 * Sun Apr  1 10:20:16 2001  John Collins
 *   Initial Entry
 */

package edu.umn.magnet.util.time;

import java.util.Date;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

/**
 * This is a tester for edu.umn.magnet.client.status.AgentTime
 * @author John Collins
 */
public class AgentTimeTest extends TestCase
{
  static final long HOUR = 3600L * 1000L;
  static final long DAY = HOUR * 24L;
  static final double SLOP = 20.0; // measurement slop
  
  public AgentTimeTest (String testName)
  {
    super(testName);
  }

  /**
   * You run the main method to exercise this thing.  There are no
   * command-line args.
   */
  public static void main (String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

  public static Test suite ()
  {
    return new TestSuite(AgentTimeTest.class);
  }

  // first test: set Agent time to match system time, wait a second,
  // make sure it still matches.  
  public void testSystemTime ()
  {
    Date now = new Date();
    AgentTime.setDate(now);
    assert("time set correctly",
	   (Math.abs(now.getTime()
		     - AgentTime.getDate().getTime())
	    < SLOP));
    // delay a known period of time by system clock, check agent time
    try {
      Thread.sleep(2000);
    }
    catch (InterruptedException ie) {
      fail("test interrupted");
    }
    assert("time correct after 2 system sec",
	   (Math.abs(now.getTime() + 2000
		     - AgentTime.getDate().getTime())
	    < SLOP)); // slop allowed.
  }

  // Second test: set Agent time offset from system time
  public void testOffsetTime ()
  {
    Date base = new Date();
    AgentTime.setDate(base.getTime(), -DAY, 2.0);
    assert("time set correctly",
	   (Math.abs(base.getTime() - DAY
		     - AgentTime.getDate().getTime())
	    < SLOP));
    try {
      Thread.sleep(2000);
    }
    catch (InterruptedException ie) {
      fail("test interrupted");
    }
    Date atime = AgentTime.getDate();
    assert("time correct after 2 system sec: " 
	   + Math.abs(base.getTime() - DAY + 4000
		      - atime.getTime()),
	   (Math.abs(base.getTime() - DAY + 4000
		     - atime.getTime())
	    < SLOP));

    // delay a known period of time by agent clock, check system time
    Date now = new Date();
    try {
      AgentTime.waitFor(2000);
    }
    catch (InterruptedException ie) {
      fail("test interrupted");
    }
    assert("time correct after 2 agent sec",
	   (Math.abs(now.getTime() + 1000
		     - new Date().getTime())
	    < SLOP));
  }

  // Third test: check the waitUntil function
  public void testWaitUntil ()
  {
    Date base = new Date();
    AgentTime.setDate(base.getTime(), -DAY, 2.0);
    assertEquals("time set correctly",
		 base.getTime() - DAY,
		 AgentTime.getDate().getTime());
    try {
      AgentTime.waitUntil(new Date(base.getTime() - DAY + 2000));
    }
    catch (InterruptedException ie) {
      fail("test interrupted");
    }
    assert("time correct after 2 agent sec",
	   (Math.abs(base.getTime() + 1000
		     - new Date().getTime())
	    < SLOP));
  }
}
